@@ -155,31 +155,28 @@ DiscoverObject::DiscoverObject(CompactMode mode, const QVariantMap &initialPrope
                 found |= b->hasApplications();
 
             if (!found) {
-                QString errorText = i18n(
-                    "Discover currently cannot be used to install any apps or "
-                    "perform system updates because none of its app backends are "
-                    "available.");
-                QString errorExplanation = xi18nc("@info",
-                    "You can install some on the Settings page, under the "
-                    "<interface>Missing Backends</interface> section.<nl/><nl/>"
-                    "Also please consider reporting this as a packaging issue to "
-                    "your distribution.");
+                const QString distroName = KOSRelease().name();
+
+                QString errorText =
+                    i18nc("@title %1 is the distro name", "%1 is not configured for installing apps through Discover—only app add-ons", distroName);
+                QString errorExplanation = xi18nc("@info:usagetip %1 is the distro name",
+                                                  "To use Discover for apps, install your preferred module on the <interface>Settings"
+                                                  "</interface> page, under <interface>Missing Backends</interface>.");
                 QString buttonIcon = QStringLiteral("tools-report-bug");
-                QString buttonText = i18n("Report This Issue");
+                QString buttonText = i18nc("@action:button %1 is the distro name", "Report This Issue to %1", distroName);
                 QString buttonUrl = KOSRelease().bugReportUrl();
 
-                if (KOSRelease().name().contains(QStringLiteral("Arch Linux"))) {
-                    errorExplanation = xi18nc("@info",
-                        "You can use <command>pacman</command> to "
-                        "install the optional dependencies that are needed to "
-                        "enable the application backends.<nl/><nl/>Please note "
-                        "that Arch Linux developers recommend using "
-                        "<command>pacman</command> for managing software because "
-                        "the PackageKit backend is not well-integrated on Arch "
-                        "Linux.");
-                    buttonIcon = QStringLiteral("help-about");
-                    buttonText = i18n("Learn More");
-                    buttonUrl = KOSRelease().supportUrl();
+                if (distroName.contains(QStringLiteral("Arch Linux"))) {
+                    errorExplanation = xi18nc("@info:usagetip %1 is the distro name; in this case it always contains 'Arch Linux'",
+                                              "To use Discover for apps, install"
+                                              " <link url='https://wiki.archlinux.org/title/Flatpak#Installation'>Flatpak</link> or"
+                                              " <link url='https://wiki.archlinux.org/title/KDE#Discover_does_not_show_any_applications'>PackageKit</link>"
+                                              " using the <command>pacman</command> package manager.<nl/><nl/>"
+                                              " Review <link url='https://archlinux.org/packages/extra/x86_64/discover/'>%1's packaging for Discover</link>",
+                                              distroName);
+                    buttonIcon = QString();
+                    buttonText = QString();
+                    buttonUrl = QString();
                 }
 
                 Q_EMIT openErrorPage(errorText, errorExplanation, buttonText, buttonIcon, buttonUrl);
@@ -266,7 +263,7 @@ void DiscoverObject::openCategory(const QString &category)
                 Q_EMIT listCategoryInternal(cat);
             } else {
                 openMode(QStringLiteral("Browsing"));
-                showPassiveNotification(i18n("Could not find category '%1'", category));
+                showError(i18n("Could not find category '%1'", category));
             }
         },
         this);
@@ -281,7 +278,7 @@ void DiscoverObject::openCategory(const QString &category)
 void DiscoverObject::openLocalPackage(const QUrl &localfile)
 {
     if (!QFile::exists(localfile.toLocalFile())) {
-        showPassiveNotification(i18n("Trying to open inexisting file '%1'", localfile.toString()));
+        showError(i18n("Trying to open inexisting file '%1'", localfile.toString()));
         openMode(QStringLiteral("Browsing"));
         return;
     }
@@ -291,9 +288,9 @@ void DiscoverObject::openLocalPackage(const QUrl &localfile)
             AbstractResourcesBackend::Filters f;
             f.resourceUrl = localfile;
             auto stream = new StoredResultsStream({ResourcesModel::global()->search(f)});
-            connect(stream, &StoredResultsStream::finishedResources, this, [this, localfile](const QVector<AbstractResource *> &res) {
+            connect(stream, &StoredResultsStream::finishedResources, this, [this, localfile](const QVector<StreamResult> &res) {
                 if (res.count() == 1) {
-                    Q_EMIT openApplicationInternal(res.first());
+                    Q_EMIT openApplicationInternal(res.first().resource);
                 } else {
                     QMimeDatabase db;
                     auto mime = db.mimeTypeForUrl(localfile);
@@ -303,11 +300,11 @@ void DiscoverObject::openLocalPackage(const QUrl &localfile)
                     if (mime.name().startsWith(QLatin1String("application/vnd.flatpak"))
                         && !kContains(ResourcesModel::global()->backends(), fIsFlatpakBackend)) {
                         openApplication(QUrl(QStringLiteral("appstream://org.kde.discover.flatpak")));
-                        showPassiveNotification(i18n("Cannot interact with flatpak resources without the flatpak backend %1. Please install it first.",
-                                                     localfile.toDisplayString()));
+                        showError(i18n("Cannot interact with flatpak resources without the flatpak backend %1. Please install it first.",
+                                       localfile.toDisplayString()));
                     } else {
                         openMode(QStringLiteral("Browsing"));
-                        showPassiveNotification(i18n("Could not open %1", localfile.toDisplayString()));
+                        showError(i18n("Could not open %1", localfile.toDisplayString()));
                     }
                 }
             });
@@ -330,7 +327,7 @@ void DiscoverObject::openApplication(const QUrl &url)
             AbstractResourcesBackend::Filters f;
             f.resourceUrl = url;
             auto stream = new StoredResultsStream({ResourcesModel::global()->search(f)});
-            connect(stream, &StoredResultsStream::finishedResources, this, [this, url](const QVector<AbstractResource *> &res) {
+            connect(stream, &StoredResultsStream::finishedResources, this, [this, url](const QVector<StreamResult> &res) {
                 if (res.count() >= 1) {
                     QPointer<QTimer> timeout = new QTimer(this);
                     timeout->setSingleShot(true);
@@ -339,11 +336,11 @@ void DiscoverObject::openApplication(const QUrl &url)
 
                     auto openResourceOrWait = [this, res, timeout] {
                         auto idx = kIndexOf(res, [](auto res) {
-                            return res->isInstalled();
+                            return res.resource->isInstalled();
                         });
                         if (idx < 0) {
                             bool oneBroken = kContains(res, [](auto res) {
-                                return res->state() == AbstractResource::Broken;
+                                return res.resource->state() == AbstractResource::Broken;
                             });
                             if (oneBroken && timeout) {
                                 return false;
@@ -351,15 +348,15 @@ void DiscoverObject::openApplication(const QUrl &url)
 
                             idx = 0;
                         }
-                        Q_EMIT openApplicationInternal(res[idx]);
+                        Q_EMIT openApplicationInternal(res[idx].resource);
                         return true;
                     };
 
                     if (!openResourceOrWait()) {
                         auto f = new OneTimeAction(0, openResourceOrWait, this);
                         for (auto r : res) {
-                            if (r->state() == AbstractResource::Broken) {
-                                connect(r, &AbstractResource::stateChanged, f, &OneTimeAction::trigger);
+                            if (r.resource->state() == AbstractResource::Broken) {
+                                connect(r.resource, &AbstractResource::stateChanged, f, &OneTimeAction::trigger);
                             }
                         }
                         timeout->start();
@@ -369,7 +366,7 @@ void DiscoverObject::openApplication(const QUrl &url)
                     }
                 } else if (url.scheme() == QLatin1String("snap")) {
                     openApplication(QUrl(QStringLiteral("appstream://org.kde.discover.snap")));
-                    showPassiveNotification(i18n("Please make sure Snap support is installed"));
+                    showError(i18n("Please make sure Snap support is installed"));
                 } else {
                     const QString errorText = i18n("Could not open %1 because it "
                     "was not found in any available software repositories.",
@@ -438,9 +435,11 @@ bool DiscoverObject::quitWhenIdle()
 
         m_sni = new KStatusNotifierItem(this);
         m_sni->setStatus(KStatusNotifierItem::Active);
-        m_sni->setIconByName("plasmadiscover");
+        m_sni->setIconByName(QStringLiteral("plasmadiscover"));
         m_sni->setTitle(i18n("Discover"));
-        m_sni->setToolTip("process-working-symbolic", i18n("Discover"), i18n("Discover was closed before certain tasks were done, waiting for it to finish."));
+        m_sni->setToolTip(QStringLiteral("process-working-symbolic"),
+                          i18n("Discover"),
+                          i18n("Discover was closed before certain tasks were done, waiting for it to finish."));
         m_sni->setStandardActionsEnabled(false);
 
         connect(TransactionModel::global(), &TransactionModel::countChanged, this, &DiscoverObject::reconsiderQuit);
@@ -611,16 +610,10 @@ QQuickWindow *DiscoverObject::rootObject() const
     return qobject_cast<QQuickWindow *>(m_engine->rootObjects().at(0));
 }
 
-void DiscoverObject::showPassiveNotification(const QString &msg)
+void DiscoverObject::showError(const QString &msg)
 {
-    QTimer::singleShot(100, this, [this, msg]() {
-        QMetaObject::invokeMethod(rootObject(),
-                                  "showPassiveNotification",
-                                  Qt::QueuedConnection,
-                                  Q_ARG(QVariant, msg),
-                                  Q_ARG(QVariant, {}),
-                                  Q_ARG(QVariant, {}),
-                                  Q_ARG(QVariant, {}));
+    QTimer::singleShot(100, this, [msg]() {
+        Q_EMIT ResourcesModel::global()->passiveMessage(msg);
     });
 }
 
