@@ -13,7 +13,6 @@
 #include <KCrash>
 #include <KDBusService>
 #include <KLocalizedString>
-#include <KStartupInfo>
 #include <KWindowSystem>
 #include <QApplication>
 #include <QCommandLineParser>
@@ -28,13 +27,13 @@
 #include <QProcessEnvironment>
 
 typedef QHash<QString, DiscoverObject::CompactMode> StringCompactMode;
-Q_GLOBAL_STATIC_WITH_ARGS(StringCompactMode,
+Q_GLOBAL_STATIC(StringCompactMode,
                           s_decodeCompactMode,
-                          (StringCompactMode{
+                          StringCompactMode{
                               {QLatin1String("auto"), DiscoverObject::Auto},
                               {QLatin1String("compact"), DiscoverObject::Compact},
                               {QLatin1String("full"), DiscoverObject::Full},
-                          }))
+                          })
 
 QCommandLineParser *createParser()
 {
@@ -68,8 +67,6 @@ void processArgs(QCommandLineParser *parser, DiscoverObject *mainWindow)
         mainWindow->openCategory(parser->value(QStringLiteral("category")));
     else if (parser->isSet(QStringLiteral("mode")))
         mainWindow->openMode(parser->value(QStringLiteral("mode")));
-    else
-        mainWindow->openMode(QStringLiteral("Browsing"));
 
     if (parser->isSet(QStringLiteral("search")))
         Q_EMIT mainWindow->openSearch(parser->value(QStringLiteral("search")));
@@ -87,6 +84,10 @@ void processArgs(QCommandLineParser *parser, DiscoverObject *mainWindow)
         else
             mainWindow->openApplication(url);
     }
+
+    if (mainWindow->rootObject()->property("pageStack").value<QObject *>()->property("depth").toInt() == 0) {
+        mainWindow->openMode(QStringLiteral("Browsing"));
+    }
 }
 
 static void raiseWindow(QWindow *window)
@@ -100,6 +101,10 @@ int main(int argc, char **argv)
     // needs to be set before we create the QGuiApplication
     QCoreApplication::setAttribute(Qt::AA_DisableSessionManager, true);
 #if WITH_QTWEBVIEW
+    { // as required by a QtWebEngine warning
+        QCoreApplication::setAttribute(Qt::AA_ShareOpenGLContexts, true);
+        QQuickWindow::setGraphicsApi(QSGRendererInterface::OpenGLRhi);
+    }
     QtWebView::initialize();
 #endif
 
@@ -110,9 +115,6 @@ int main(int argc, char **argv)
     QApplication app(argc, argv);
     app.setWindowIcon(QIcon::fromTheme(QStringLiteral("plasmadiscover")));
     app.setAttribute(Qt::AA_DontCreateNativeWidgetSiblings);
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    app.setAttribute(Qt::AA_UseHighDpiPixmaps, true);
-#endif
     KCrash::initialize();
     KLocalizedString::setApplicationDomain("plasma-discover");
     KAboutData about(QStringLiteral("discover"),
@@ -170,7 +172,7 @@ int main(int argc, char **argv)
             if (!options.isEmpty() || !parser->positionalArguments().isEmpty())
                 initialProperties = {{QStringLiteral("currentTopLevel"), QStringLiteral("qrc:/qml/LoadingPage.qml")}};
             if (feedback) {
-                initialProperties.insert("visible", false);
+                initialProperties.insert(QStringLiteral("visible"), false);
             }
             mainWindow = new DiscoverObject(s_decodeCompactMode->value(parser->value(QStringLiteral("compact")), DiscoverObject::Full), initialProperties);
         }
