@@ -52,7 +52,6 @@ QHash<int, QByteArray> UpdateModel::roleNames() const
     ret.insert(SizeRole, "size");
     ret.insert(SectionRole, "section");
     ret.insert(ChangelogRole, "changelog");
-    ret.insert(UpgradeTextRole, "upgradeText");
     ret.insert(ExtendedRole, "extended");
     return ret;
 }
@@ -74,8 +73,9 @@ void UpdateModel::setBackend(ResourcesUpdatesModel *updates)
 void UpdateModel::resourceHasProgressed(AbstractResource *res, qreal progress, AbstractBackendUpdater::State state)
 {
     UpdateItem *item = itemFromResource(res);
-    if (!item)
+    if (!item) {
         return;
+    }
     item->setProgress(progress);
     item->setState(state);
 
@@ -90,11 +90,12 @@ void UpdateModel::activityChanged()
             m_updates->prepare();
             setResources(m_updates->toUpdate());
 
-            for (auto item : qAsConst(m_updateItems)) {
+            for (auto item : std::as_const(m_updateItems)) {
                 item->setProgress(0);
             }
-        } else
+        } else {
             setResources(m_updates->toUpdate());
+        }
     }
 }
 
@@ -150,16 +151,18 @@ QVariant UpdateModel::data(const QModelIndex &index, int role) const
 
 void UpdateModel::checkResources(const QList<AbstractResource *> &resource, bool checked)
 {
-    if (checked)
+    if (checked) {
         m_updates->addResources(resource);
-    else
+    } else {
         m_updates->removeResources(resource);
+    }
 }
 
 Qt::ItemFlags UpdateModel::flags(const QModelIndex &index) const
 {
-    if (!index.isValid())
+    if (!index.isValid()) {
         return Qt::NoItemFlags;
+    }
 
     return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
 }
@@ -200,8 +203,9 @@ void UpdateModel::fetchUpdateDetails(int row)
 {
     UpdateItem *item = itemFromIndex(index(row, 0));
     Q_ASSERT(item);
-    if (!item)
+    if (!item) {
         return;
+    }
 
     item->app()->fetchUpdateDetails();
 }
@@ -211,8 +215,9 @@ void UpdateModel::integrateChangelog(const QString &changelog)
     auto app = qobject_cast<AbstractResource *>(sender());
     Q_ASSERT(app);
     auto item = itemFromResource(app);
-    if (!item)
+    if (!item) {
         return;
+    }
 
     item->setChangelog(changelog);
 
@@ -277,7 +282,7 @@ int UpdateModel::toUpdateCount() const
 {
     int ret = 0;
     QSet<QString> packages;
-    for (UpdateItem *item : qAsConst(m_updateItems)) {
+    for (UpdateItem *item : std::as_const(m_updateItems)) {
         const auto packageName = item->resource()->packageName();
         if (packages.contains(packageName)) {
             continue;
@@ -292,7 +297,7 @@ int UpdateModel::totalUpdatesCount() const
 {
     int ret = 0;
     QSet<QString> packages;
-    for (UpdateItem *item : qAsConst(m_updateItems)) {
+    for (UpdateItem *item : std::as_const(m_updateItems)) {
         const auto packageName = item->resource()->packageName();
         if (packages.contains(packageName)) {
             continue;
@@ -305,9 +310,10 @@ int UpdateModel::totalUpdatesCount() const
 
 UpdateItem *UpdateModel::itemFromResource(AbstractResource *res)
 {
-    for (UpdateItem *item : qAsConst(m_updateItems)) {
-        if (item->app() == res)
+    for (UpdateItem *item : std::as_const(m_updateItems)) {
+        if (item->app() == res) {
             return item;
+        }
     }
     return nullptr;
 }
@@ -336,13 +342,14 @@ UpdateItem *UpdateModel::itemFromIndex(const QModelIndex &index) const
 void UpdateModel::resourceDataChanged(AbstractResource *res, const QVector<QByteArray> &properties)
 {
     auto item = itemFromResource(res);
-    if (!item)
+    if (!item) {
         return;
+    }
 
     const auto index = indexFromItem(item);
-    if (properties.contains("state"))
-        Q_EMIT dataChanged(index, index, {SizeRole, UpgradeTextRole});
-    else if (properties.contains("size")) {
+    if (properties.contains("state")) {
+        Q_EMIT dataChanged(index, index, {SizeRole});
+    } else if (properties.contains("size")) {
         Q_EMIT dataChanged(index, index, {SizeRole});
         m_updateSizeTimer->start();
     }
@@ -350,16 +357,36 @@ void UpdateModel::resourceDataChanged(AbstractResource *res, const QVector<QByte
 
 void UpdateModel::checkAll()
 {
-    for (int i = 0, c = rowCount(); i < c; ++i)
-        if (index(i, 0).data(Qt::CheckStateRole) != Qt::Checked)
-            setData(index(i, 0), Qt::Checked, Qt::CheckStateRole);
+    QList<AbstractResource *> updatedItems;
+
+    for (int i = 0, c = rowCount(); i < c; ++i) {
+        auto idx = index(i);
+        if (idx.data(Qt::CheckStateRole) != Qt::Checked) {
+            updatedItems.append(itemFromIndex(idx)->app());
+        }
+    }
+
+    checkResources(updatedItems, true);
+
+    Q_EMIT dataChanged(index(0), index(rowCount() - 1), {Qt::CheckStateRole});
+    Q_EMIT toUpdateChanged();
 }
 
 void UpdateModel::uncheckAll()
 {
-    for (int i = 0, c = rowCount(); i < c; ++i)
-        if (index(i, 0).data(Qt::CheckStateRole) != Qt::Unchecked)
-            setData(index(i, 0), Qt::Unchecked, Qt::CheckStateRole);
+    QList<AbstractResource *> updatedItems;
+
+    for (int i = 0, c = rowCount(); i < c; ++i) {
+        auto idx = index(i);
+        if (idx.data(Qt::CheckStateRole) != Qt::Unchecked) {
+            updatedItems.append(itemFromIndex(idx)->app());
+        }
+    }
+
+    checkResources(updatedItems, false);
+
+    Q_EMIT dataChanged(index(0), index(rowCount() - 1), {Qt::CheckStateRole});
+    Q_EMIT toUpdateChanged();
 }
 
 #include "moc_UpdateModel.cpp"
