@@ -5,8 +5,7 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef ODRSREVIEWSBACKEND_H
-#define ODRSREVIEWSBACKEND_H
+#pragma once
 
 #include <ReviewsBackend/AbstractReviewsBackend.h>
 #include <ReviewsBackend/ReviewsModel.h>
@@ -23,6 +22,7 @@ class DISCOVERCOMMON_EXPORT OdrsReviewsBackend : public AbstractReviewsBackend
 {
     Q_OBJECT
 public:
+    static QSharedPointer<OdrsReviewsBackend> global();
     explicit OdrsReviewsBackend();
     ~OdrsReviewsBackend() override;
 
@@ -59,6 +59,15 @@ public:
     void submitUsefulness(Review *review, bool useful) override;
     bool isResourceSupported(AbstractResource *res) const override;
     void emitRatingFetched(AbstractResourcesBackend *backend, const QList<AbstractResource *> &res) const;
+    QString errorMessage() const override
+    {
+        return m_errorMessage;
+    }
+
+    QVector<Rating *> top() const
+    {
+        return m_top;
+    }
 
 private Q_SLOTS:
     void ratingsFetched(KJob *job);
@@ -74,14 +83,16 @@ protected:
     QString userName() const override;
 
 private:
+    void fetchRatings();
     void setFetching(bool);
     QNetworkAccessManager *nam();
     void parseRatings();
     void parseReviews(const QJsonDocument &document, AbstractResource *resource);
 
+    QString m_errorMessage;
     QHash<QString, Rating *> m_ratings;
     bool m_isFetching = false;
     CachedNetworkAccessManager *m_delayedNam = nullptr;
-};
 
-#endif // ODRSREVIEWSBACKEND_H
+    QVector<Rating *> m_top;
+};
