@@ -13,7 +13,6 @@
 
 #include <ReviewsBackend/Rating.h>
 #include <Transaction/Transaction.h>
-#include <appstream/AppStreamIntegration.h>
 #include <appstream/AppStreamUtils.h>
 #include <appstream/OdrsReviewsBackend.h>
 #include <resources/SourcesModel.h>
@@ -56,6 +55,8 @@
 
 DISCOVER_BACKEND_PLUGIN(FlatpakBackend)
 
+using namespace Qt::Literals::StringLiterals;
+
 class FlatpakSource
 {
 public:
@@ -87,7 +88,7 @@ public:
 
     QString url() const
     {
-        return m_remote ? flatpak_remote_get_url(m_remote) : QString();
+        return m_remote ? QString::fromUtf8(flatpak_remote_get_url(m_remote)) : QString();
     }
 
     bool isEnabled() const
@@ -141,10 +142,6 @@ public:
 
         Q_ASSERT(!m_resources.contains(resource->uniqueId()) || m_resources.value(resource->uniqueId()) == resource);
         m_resources.insert(resource->uniqueId(), resource);
-        if (!resource->extends().isEmpty()) {
-            m_backend->m_extends.append(resource->extends());
-            m_backend->m_extends.removeDuplicates();
-        }
 
         QObject::connect(resource, &FlatpakResource::sizeChanged, m_backend, [this, resource] {
             if (!m_backend->isFetching())
@@ -157,31 +154,24 @@ public:
         return m_remote;
     }
 
-    QList<AppStream::Component> componentsByName(const QString &name)
+    AppStream::ComponentBox componentsByName(const QString &name)
     {
-        QList<AppStream::Component> comps = m_pool->componentsById(name);
-        if (!comps.isEmpty())
+        auto comps = m_pool->componentsById(name);
+        if (!comps.isEmpty()) {
             return comps;
+        }
 
         comps = m_pool->componentsByProvided(AppStream::Provided::KindId, name);
         return comps;
     }
 
-    QList<AppStream::Component> componentsByFlatpakId(const QString &ref)
+    AppStream::ComponentBox componentsByFlatpakId(const QString &ref)
     {
-#if ASQ_CHECK_VERSION(0, 16, 0)
-        QList<AppStream::Component> comps = m_pool->componentsByBundleId(AppStream::Bundle::KindFlatpak, ref, false);
-#else
-        QList<AppStream::Component> comps = m_pool->components();
-        comps = kFilter<QList<AppStream::Component>>(comps, [&ref](const AppStream::Component &component) {
-            const QString id = component.bundle(AppStream::Bundle::KindFlatpak).id();
-            return id == ref;
-        });
-#endif
+        AppStream::ComponentBox comps = m_pool->componentsByBundleId(AppStream::Bundle::KindFlatpak, ref, false);
         if (!comps.isEmpty())
             return comps;
 
-        comps = m_pool->componentsByProvided(AppStream::Provided::KindId, ref.section('/', 1, 1));
+        comps = m_pool->componentsByProvided(AppStream::Provided::KindId, ref.section('/'_L1, 1, 1));
         return comps;
     }
 
@@ -227,8 +217,8 @@ QDebug operator<<(QDebug debug, const FlatpakResource::Id &id)
 static FlatpakResource::Id idForComponent(const AppStream::Component &component)
 {
     // app/app.getspace.Space/x86_64/stable
-    const auto bundleId = component.bundle(AppStream::Bundle::KindFlatpak).id();
-    auto parts = bundleId.splitRef('/');
+    const QString bundleId = component.bundle(AppStream::Bundle::KindFlatpak).id();
+    auto parts = QStringView(bundleId).split('/'_L1);
     Q_ASSERT(parts.size() == 4);
 
     return {
@@ -238,7 +228,7 @@ static FlatpakResource::Id idForComponent(const AppStream::Component &component)
     };
 }
 
-static FlatpakResource::Id idForInstalledRef(FlatpakInstalledRef *ref, const QString &postfix)
+static FlatpakResource::Id idForInstalledRef(FlatpakInstalledRef *ref, const QString &postfix = QString())
 {
     const QString appId = QLatin1String(flatpak_ref_get_name(FLATPAK_REF(ref))) + postfix;
     const QString arch = QString::fromUtf8(flatpak_ref_get_arch(FLATPAK_REF(ref)));
@@ -274,7 +264,11 @@ static std::optional<AppStream::Metadata> metadataFromBytes(GBytes *appstreamGz,
     gconstpointer data = g_bytes_get_data(appstream, &len);
 
     AppStream::Metadata metadata;
+#if ASQ_CHECK_VERSION(0, 16, 0)
+    metadata.setFormatStyle(AppStream::Metadata::FormatStyleCatalog);
+#else
     metadata.setFormatStyle(AppStream::Metadata::FormatStyleCollection);
+#endif
     AppStream::Metadata::MetadataError error = metadata.parse(QString::fromUtf8((char *)data, len), AppStream::Metadata::FormatKindXml);
     if (error != AppStream::Metadata::MetadataErrorNoError) {
         qWarning() << "Failed to parse appstream metadata: " << error;
@@ -286,7 +280,7 @@ static std::optional<AppStream::Metadata> metadataFromBytes(GBytes *appstreamGz,
 FlatpakBackend::FlatpakBackend(QObject *parent)
     : AbstractResourcesBackend(parent)
     , m_updater(new StandardBackendUpdater(this))
-    , m_reviews(AppStreamIntegration::global()->reviews())
+    , m_reviews(OdrsReviewsBackend::global())
     , m_cancellable(g_cancellable_new())
     , m_checkForUpdatesTimer(new QTimer(this))
 {
@@ -330,7 +324,7 @@ FlatpakBackend::~FlatpakBackend()
     }
     m_threadPool.clear();
 
-    for (auto inst : qAsConst(m_installations))
+    for (auto inst : std::as_const(m_installations))
         g_object_unref(inst);
     m_installations.clear();
     g_object_unref(m_cancellable);
@@ -448,10 +442,13 @@ FlatpakInstalledRef *FlatpakBackend::getInstalledRefForApp(const FlatpakResource
     return ref;
 }
 
-QString refToBundleId(FlatpakRef *ref)
+static QString refToBundleId(FlatpakRef *ref)
 {
-    return QString(flatpak_ref_get_kind(ref) == FLATPAK_REF_KIND_APP ? "app/" : "runtime/") + flatpak_ref_get_name(ref) + '/' + flatpak_ref_get_arch(ref) + '/'
-        + flatpak_ref_get_branch(ref);
+    const QString typeAsString = flatpak_ref_get_kind(ref) == FLATPAK_REF_KIND_APP ? QStringLiteral("app") : QStringLiteral("runtime");
+    const QString flatpakName = QString::fromUtf8(flatpak_ref_get_name(ref));
+    const QString arch = QString::fromUtf8(flatpak_ref_get_arch(ref));
+    const QString branch = QString::fromUtf8(flatpak_ref_get_branch(ref));
+    return QStringLiteral("%1/%2/%3/%4").arg(typeAsString, flatpakName, arch, branch);
 }
 
 FlatpakResource *FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *installation, FlatpakInstalledRef *ref, bool *freshResource) const
@@ -474,18 +471,18 @@ FlatpakResource *FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *inst
     const QString refId = refToBundleId(FLATPAK_REF(ref));
     AppStream::Component cid;
     if (source && source->m_pool) {
-        QList<AppStream::Component> comps = source->componentsByFlatpakId(refId);
+        auto comps = source->componentsByFlatpakId(refId);
         if (comps.isEmpty()) {
-            g_autoptr(GBytes) metadata = flatpak_installed_ref_load_appdata(ref, 0, 0);
+            g_autoptr(GBytes) metadata = flatpak_installed_ref_load_appdata(ref, m_cancellable, nullptr);
             if (metadata) {
                 auto meta = metadataFromBytes(metadata, m_cancellable);
                 comps = meta->components();
             }
         }
 
-        if (comps.count() >= 1) {
-            Q_ASSERT(comps.count() == 1);
-            cid = comps.constFirst();
+        if (comps.size() >= 1) {
+            Q_ASSERT(comps.size() == 1);
+            cid = *comps.indexSafe(0);
         }
     }
 
@@ -534,7 +531,7 @@ FlatpakResource *FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *inst
     if (freshResource)
         *freshResource = true;
 
-    Q_ASSERT(resource->uniqueId() == idForInstalledRef(ref, {}) || resource->uniqueId() == idForInstalledRef(ref, {".desktop"}));
+    Q_ASSERT(resource->uniqueId() == idForInstalledRef(ref) || resource->uniqueId() == idForInstalledRef(ref, QStringLiteral(".desktop")));
     return resource;
 }
 
@@ -559,7 +556,7 @@ FlatpakResource *FlatpakBackend::getRuntimeForApp(FlatpakResource *resource) con
 {
     FlatpakResource *runtime = nullptr;
     const QString runtimeName = resource->runtime();
-    const auto runtimeInfo = runtimeName.splitRef(QLatin1Char('/'));
+    const auto runtimeInfo = QStringView(runtimeName).split(QLatin1Char('/'));
 
     if (runtimeInfo.count() != 3) {
         return runtime;
@@ -626,9 +623,8 @@ void FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url, ResultsStream *str
             return;
         }
 
-        const QList<AppStream::Component> components = metadata->components();
-        if (components.size()) {
-            asComponent = components.first();
+        if (std::optional<AppStream::Component> firstComponent = metadata->components().indexSafe(0); firstComponent.has_value()) {
+            asComponent = *firstComponent;
         } else {
             qWarning() << "Failed to parse appstream metadata";
             return;
@@ -715,7 +711,7 @@ void FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url, ResultsStream *str
 
 QString composeRef(bool isRuntime, const QString &name, const QString &branch)
 {
-    return (isRuntime ? "runtime/" : "app/") + name + '/' + flatpak_get_default_arch() + '/' + branch;
+    return (isRuntime ? "runtime/"_L1 : "app/"_L1) + name + '/'_L1 + QString::fromUtf8(flatpak_get_default_arch()) + '/'_L1 + branch;
 }
 
 AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancellable *cancellable)
@@ -752,14 +748,14 @@ AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancel
         QDir(path).removeRecursively();
     });
 
-    g_autoptr(FlatpakRemote) tempRemote = flatpak_remote_new(remoteName.toUtf8());
+    g_autoptr(FlatpakRemote) tempRemote = flatpak_remote_new(remoteName.toUtf8().constData());
     populateRemote(tempRemote,
                    remoteName,
                    settings.value(QStringLiteral("Flatpak Ref/Url")).toString(),
-                   settings.value(QStringLiteral("Flatpak Ref/GPGKey")).toString().toUtf8());
+                   settings.value(QStringLiteral("Flatpak Ref/GPGKey")).toString());
     if (!flatpak_installation_modify_remote(tempInstallation, tempRemote, cancellable, &localError)) {
         qDebug() << "error adding temporary remote" << localError->message;
-        return {asComponent};
+        return asComponent;
     }
 
     auto cb = [](const char *status, guint progress, gboolean /*estimating*/, gpointer /*user_data*/) {
@@ -767,17 +763,28 @@ AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancel
     };
 
     gboolean changed;
-    if (!flatpak_installation_update_appstream_full_sync(tempInstallation, remoteName.toUtf8(), nullptr, cb, nullptr, &changed, cancellable, &localError)) {
+    if (!flatpak_installation_update_appstream_full_sync(tempInstallation,
+                                                         remoteName.toUtf8().constData(),
+                                                         nullptr,
+                                                         cb,
+                                                         nullptr,
+                                                         &changed,
+                                                         cancellable,
+                                                         &localError)) {
         qDebug() << "error fetching appstream" << localError->message;
-        return {asComponent};
+        return asComponent;
     }
     Q_ASSERT(changed);
-    const QString appstreamLocation = path + "/appstream/" + remoteName + '/' + flatpak_get_default_arch() + "/active";
+    const QString appstreamLocation = path + "/appstream/"_L1 + remoteName + '/'_L1 + QString::fromUtf8(flatpak_get_default_arch()) + "/active"_L1;
 
     AppStream::Pool pool;
 #ifdef APPSTREAM_NEW_POOL_API
     pool.setLoadStdDataLocations(false);
+#if ASQ_CHECK_VERSION(0, 16, 0)
+    pool.addExtraDataLocation(appstreamLocation, AppStream::Metadata::FormatStyleCatalog);
+#else
     pool.addExtraDataLocation(appstreamLocation, AppStream::Metadata::FormatStyleCollection);
+#endif
 #else
     pool.clearMetadataLocations();
     pool.addMetadataLocation(appstreamLocation);
@@ -791,12 +798,12 @@ AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancel
 
     if (!pool.load()) {
         qDebug() << "error loading pool" << pool.lastError();
-        return {asComponent};
+        return asComponent;
     }
 
     // TODO optimise, this lookup should happen in libappstream
     auto comps = pool.components();
-    comps = kFilter<QList<AppStream::Component>>(comps, [name, branch](const AppStream::Component &component) {
+    kFilterInPlace<AppStream::ComponentBox>(comps, [name, branch](const AppStream::Component &component) {
         const QString id = component.bundle(AppStream::Bundle::KindFlatpak).id();
         // app/app.getspace.Space/x86_64/stable
         return id.section(QLatin1Char('/'), 1, 1) == name && (branch.isEmpty() || id.section(QLatin1Char('/'), 3, 3) == branch);
@@ -805,7 +812,7 @@ AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancel
         qDebug() << "could not find" << name << "in" << remoteName;
         return asComponent;
     }
-    return comps.constFirst();
+    return *comps.indexSafe(0);
 }
 
 void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream)
@@ -821,7 +828,8 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
 
     // If we already added the remote, just go with it
     g_autoptr(FlatpakRemote) remote = flatpak_installation_get_remote_by_name(preferredInstallation(), remoteName.toUtf8().constData(), m_cancellable, &error);
-    if (remote && flatpak_remote_get_url(remote) != refurl) {
+    g_autofree char *remoteUrl = flatpak_remote_get_url(remote);
+    if (remote && QString::fromUtf8(remoteUrl) != refurl) {
         remote = nullptr;
     }
     if (remote) {
@@ -832,7 +840,7 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
             const QString ref = composeRef(isRuntime, name, branch);
             auto searchComponent = [this, stream, source, ref] {
                 auto comps = source->componentsByFlatpakId(ref);
-                auto resources = kTransform<QVector<AbstractResource *>>(comps, [this, source](const auto &comp) {
+                auto resources = kTransform<QVector<StreamResult>>(comps, [this, source](const auto &comp) {
                     return resourceForComponent(comp, source);
                 });
                 Q_EMIT stream->resourcesFound(resources);
@@ -863,7 +871,7 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
     resource->setOrigin(remoteName);
     resource->setDisplayOrigin(remote ? QString::fromUtf8(flatpak_remote_get_title(remote)) : QString());
     resource->setFlatpakName(name);
-    resource->setArch(flatpak_get_default_arch());
+    resource->setArch(QString::fromUtf8(flatpak_get_default_arch()));
     resource->setBranch(branch);
     resource->setType(isRuntime ? FlatpakResource::Runtime : FlatpakResource::DesktopApp);
 
@@ -886,17 +894,14 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
                 auto runtime = getRuntimeForApp(resource);
                 if (!runtime || (runtime && !runtime->isInstalled())) {
                     auto repoStream = new ResultsStream(QLatin1String("FlatpakStream-searchrepo-") + runtimeUrl.toString());
-                    connect(repoStream,
-                            &ResultsStream::resourcesFound,
-                            this,
-                            [this, resource, stream, refSource](const QVector<AbstractResource *> &resources) {
-                                for (auto res : resources) {
-                                    installApplication(res);
-                                }
-                                refSource->addResource(resource);
-                                Q_EMIT stream->resourcesFound({resource});
-                                stream->finish();
-                            });
+                    connect(repoStream, &ResultsStream::resourcesFound, this, [this, resource, stream, refSource](const QVector<StreamResult> &resources) {
+                        for (auto res : resources) {
+                            installApplication(res.resource);
+                        }
+                        refSource->addResource(resource);
+                        Q_EMIT stream->resourcesFound({resource});
+                        stream->finish();
+                    });
 
                     auto fetchRemoteResource = new FlatpakFetchRemoteResourceJob(runtimeUrl, repoStream, this);
                     fetchRemoteResource->start();
@@ -972,7 +977,7 @@ void FlatpakBackend::addSourceFromFlatpakRepo(const QUrl &url, ResultsStream *st
 
 void FlatpakBackend::loadAppsFromAppstreamData()
 {
-    for (auto installation : qAsConst(m_installations)) {
+    for (auto installation : std::as_const(m_installations)) {
         // Load applications from appstream metadata
         if (g_cancellable_is_cancelled(m_cancellable))
             break;
@@ -1009,13 +1014,13 @@ void FlatpakBackend::loadRemote(FlatpakInstallation *installation, FlatpakRemote
     m_refreshAppstreamMetadataJobs.insert(remote);
 
     g_autofree char *path_str = g_file_get_path(fileTimestamp);
-    QFileInfo fileInfo(QFile::encodeName(path_str));
+    QFileInfo fileInfo(QFile::decodeName(path_str));
     if (!fileInfo.exists() || fileInfo.lastModified().toUTC().secsTo(QDateTime::currentDateTimeUtc()) > 21600) {
         // Refresh appstream metadata in case they have never been refreshed or the cache is older than 6 hours
         checkForRemoteUpdates(installation, remote);
     } else {
         auto source = integrateRemote(installation, remote);
-        Q_ASSERT(findSource(installation, flatpak_remote_get_name(remote)) == source);
+        Q_ASSERT(findSource(installation, QString::fromUtf8(flatpak_remote_get_name(remote))) == source);
     }
 }
 
@@ -1023,7 +1028,7 @@ void FlatpakBackend::unloadRemote(FlatpakInstallation *installation, FlatpakRemo
 {
     acquireFetching(true);
     for (auto it = m_flatpakSources.begin(); it != m_flatpakSources.end();) {
-        if ((*it)->url() == flatpak_remote_get_url(remote) && (*it)->installation() == installation) {
+        if ((*it)->url() == QString::fromUtf8(flatpak_remote_get_url(remote)) && (*it)->installation() == installation) {
             qDebug() << "unloading remote" << (*it) << remote;
             it = m_flatpakSources.erase(it);
         } else {
@@ -1038,7 +1043,7 @@ void FlatpakBackend::metadataRefreshed(FlatpakRemote *remote)
     const bool removed = m_refreshAppstreamMetadataJobs.remove(remote);
     Q_ASSERT(removed);
     if (m_refreshAppstreamMetadataJobs.isEmpty()) {
-        for (auto installation : qAsConst(m_installations)) {
+        for (auto installation : std::as_const(m_installations)) {
             // Load local updates, comparing current and latest commit
             loadLocalUpdates(installation);
 
@@ -1061,7 +1066,7 @@ void FlatpakBackend::createPool(QSharedPointer<FlatpakSource> source)
         return;
     }
 
-    AppStream::Pool *pool = new AppStream::Pool(this);
+    AppStream::Pool *pool = new AppStream::Pool;
     auto fw = new QFutureWatcher<bool>(this);
     connect(fw, &QFutureWatcher<bool>::finished, this, [this, fw, pool, source]() {
         source->m_pool = pool;
@@ -1079,7 +1084,11 @@ void FlatpakBackend::createPool(QSharedPointer<FlatpakSource> source)
 
 #ifdef APPSTREAM_NEW_POOL_API
     pool->setLoadStdDataLocations(false);
+#if ASQ_CHECK_VERSION(0, 16, 0)
+    pool->addExtraDataLocation(appstreamDirPath, AppStream::Metadata::FormatStyleCatalog);
+#else
     pool->addExtraDataLocation(appstreamDirPath, AppStream::Metadata::FormatStyleCollection);
+#endif
 #else
     pool->clearMetadataLocations();
     pool->addMetadataLocation(appstreamDirPath);
@@ -1091,23 +1100,25 @@ void FlatpakBackend::createPool(QSharedPointer<FlatpakSource> source)
     QDir().mkpath(pool->cacheLocation());
 #endif
 
-    fw->setFuture(QtConcurrent::run(&m_threadPool, pool, &AppStream::Pool::load));
+    fw->setFuture(QtConcurrent::run(&m_threadPool, [pool] {
+        return pool->load();
+    }));
 }
 
 QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote)
 {
     Q_ASSERT(m_refreshAppstreamMetadataJobs.contains(remote));
     m_sources->addRemote(remote, flatpakInstallation);
-    for (auto source : qAsConst(m_flatpakSources)) {
-        if (source->url() == flatpak_remote_get_url(remote) && source->installation() == flatpakInstallation
-            && source->name() == flatpak_remote_get_name(remote)) {
+    for (auto source : std::as_const(m_flatpakSources)) {
+        if (source->url() == QString::fromUtf8(flatpak_remote_get_url(remote)) && source->installation() == flatpakInstallation
+            && source->name() == QString::fromUtf8(flatpak_remote_get_name(remote))) {
             createPool(source);
             return source;
         }
     }
-    for (auto source : qAsConst(m_flatpakLoadingSources)) {
-        if (source->url() == flatpak_remote_get_url(remote) && source->installation() == flatpakInstallation
-            && source->name() == flatpak_remote_get_name(remote)) {
+    for (auto source : std::as_const(m_flatpakLoadingSources)) {
+        if (source->url() == QString::fromUtf8(flatpak_remote_get_url(remote)) && source->installation() == flatpakInstallation
+            && source->name() == QString::fromUtf8(flatpak_remote_get_name(remote))) {
             createPool(source);
             return source;
         }
@@ -1402,6 +1413,14 @@ int FlatpakBackend::updatesCount() const
     return m_updater->updatesCount();
 }
 
+bool FlatpakBackend::flatpakResourceLessThan(const StreamResult &l, const StreamResult &r) const
+{
+    if (l.sortScore == r.sortScore) {
+        return flatpakResourceLessThan(l.resource, r.resource);
+    }
+    return l.sortScore < r.sortScore;
+}
+
 bool FlatpakBackend::flatpakResourceLessThan(AbstractResource *l, AbstractResource *r) const
 {
     // clang-format off
@@ -1422,7 +1441,7 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
         return stream;
     } else if (filter.resourceUrl.scheme() == QLatin1String("appstream")) {
         return findResourceByPackageName(filter.resourceUrl);
-    } else if (!filter.resourceUrl.isEmpty() || (!filter.extends.isEmpty() && !m_extends.contains(filter.extends)))
+    } else if (!filter.resourceUrl.isEmpty())
         return new ResultsStream(QStringLiteral("FlatpakStream-void"), {});
     else if (filter.state == AbstractResource::Upgradeable) {
         auto stream = new ResultsStream(QStringLiteral("FlatpakStream-upgradeable"));
@@ -1436,10 +1455,10 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                 }
 
                 const auto refs = fw->result();
-                QVector<AbstractResource *> resources;
+                QVector<StreamResult> resources;
                 for (auto it = refs.constBegin(), itEnd = refs.constEnd(); it != itEnd; ++it) {
                     resources.reserve(resources.size() + it->size());
-                    for (auto ref : qAsConst(it.value())) {
+                    for (auto ref : std::as_const(it.value())) {
                         bool fresh;
                         auto resource = getAppForInstalledRef(it.key(), ref, &fresh);
                         g_object_unref(ref);
@@ -1508,7 +1527,7 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
     } else if (filter.state == AbstractResource::Installed) {
         auto stream = new ResultsStream(QStringLiteral("FlatpakStream-installed"));
         auto f = [this, stream, filter] {
-            QVector<AbstractResource *> resources;
+            QVector<StreamResult> resources;
             for (auto installation : std::as_const(m_installations)) {
                 g_autoptr(GError) localError = nullptr;
                 g_autoptr(GPtrArray) refs = flatpak_installation_list_installed_refs(installation, m_cancellable, &localError);
@@ -1529,9 +1548,9 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                     if (!resource) {
                         continue;
                     }
-                    if (!filter.search.isEmpty() && !resource->name().contains(filter.search, Qt::CaseInsensitive))
+                    if (!filter.search.isEmpty() && !resource->name().contains(filter.search, Qt::CaseInsensitive)
+                        && !resource->appstreamId().contains(filter.search, Qt::CaseInsensitive))
                         continue;
-
                     if (resource->resourceType() == FlatpakResource::Runtime) {
                         resources.prepend(resource);
                     } else {
@@ -1554,8 +1573,8 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
 
     auto stream = new ResultsStream(QStringLiteral("FlatpakStream"));
     auto f = [this, stream, filter]() {
-        QVector<AbstractResource *> prioritary, rest;
-        for (const auto &source : qAsConst(m_flatpakSources)) {
+        QVector<StreamResult> prioritary, rest;
+        for (const auto &source : std::as_const(m_flatpakSources)) {
             QList<FlatpakResource *> resources;
             if (source->m_pool) {
                 const auto a = !filter.search.isEmpty() ? source->m_pool->search(filter.search)
@@ -1590,10 +1609,14 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                     prioritary += r;
                 } else if (r->comment().contains(filter.search, Qt::CaseInsensitive)) {
                     rest += r;
+                    // trust The search terms provided by appstream are relevant, this makes possible finding "gimp"
+                    // since the name() is "GNU Image Manipulation Program"
+                } else if (r->appstreamId().contains(filter.search, Qt::CaseInsensitive)) {
+                    rest += r;
                 }
             }
         }
-        auto f = [this](AbstractResource *l, AbstractResource *r) {
+        auto f = [this](auto l, auto r) {
             return flatpakResourceLessThan(l, r);
         };
         std::sort(rest.begin(), rest.end(), f);
@@ -1622,18 +1645,18 @@ bool FlatpakBackend::isTracked(FlatpakResource *resource) const
     return false;
 }
 
-QVector<AbstractResource *> FlatpakBackend::resourcesByAppstreamName(const QString &name) const
+QVector<StreamResult> FlatpakBackend::resultsByAppstreamName(const QString &name) const
 {
-    QVector<AbstractResource *> resources;
+    QVector<StreamResult> resources;
     for (const auto &source : m_flatpakSources) {
         if (source->m_pool) {
             auto comps = source->componentsByName(name);
-            resources << kTransform<QVector<AbstractResource *>>(comps, [this, source](const auto &comp) {
-                return resourceForComponent(comp, source);
+            resources << kTransform<QVector<StreamResult>>(comps, [this, source](const AppStream::Component &comp) -> StreamResult {
+                return {resourceForComponent(comp, source), comp.sortScore()};
             });
         }
     }
-    auto f = [this](AbstractResource *l, AbstractResource *r) {
+    auto f = [this](auto l, auto r) {
         return flatpakResourceLessThan(l, r);
     };
     std::sort(resources.begin(), resources.end(), f);
@@ -1650,11 +1673,11 @@ ResultsStream *FlatpakBackend::findResourceByPackageName(const QUrl &url)
             auto stream = new ResultsStream(QStringLiteral("FlatpakStream-AppStreamUrl"));
             auto f = [this, stream, appstreamIds] {
                 std::set<AbstractResource *> resources;
-                QVector<AbstractResource *> resourcesVector;
+                QVector<StreamResult> resourcesVector;
                 for (const auto &appstreamId : appstreamIds) {
-                    const auto resourcesFound = resourcesByAppstreamName(appstreamId);
+                    const auto resourcesFound = resultsByAppstreamName(appstreamId);
                     for (auto res : resourcesFound) {
-                        auto [x, inserted] = resources.insert(res);
+                        auto [x, inserted] = resources.insert(res.resource);
                         if (inserted) {
                             resourcesVector.append(res);
                         }
@@ -1717,8 +1740,8 @@ void FlatpakBackend::checkRepositories(const QMap<QString, QStringList> &names)
     g_autoptr(GError) localError = nullptr;
     for (auto it = names.begin(), itEnd = names.end(); it != itEnd; ++it) {
         FlatpakInstallation *installation = flatpakInstallationByPath(it.key());
-        for (const QString &name : qAsConst(*it)) {
-            auto remote = flatpak_installation_get_remote_by_name(installation, name.toUtf8(), m_cancellable, &localError);
+        for (const QString &name : std::as_const(*it)) {
+            auto remote = flatpak_installation_get_remote_by_name(installation, name.toUtf8().constData(), m_cancellable, &localError);
             if (!remote) {
                 qWarning() << "Could not find remote" << name << "in" << it.key();
                 continue;
@@ -1749,7 +1772,7 @@ FlatpakRemote *FlatpakBackend::installSource(FlatpakResource *resource)
 
     g_autoptr(GError) error = nullptr;
     if (!flatpak_installation_add_remote(preferredInstallation(), remote, false, cancellable, &error)) {
-        Q_EMIT passiveMessage(i18n("Failed to add source '%1': %2", resource->flatpakName(), error->message));
+        Q_EMIT passiveMessage(i18n("Failed to add source '%1': %2", resource->flatpakName(), QString::fromUtf8(error->message)));
         qWarning() << "Failed to add source " << resource->flatpakName() << error->message;
         return nullptr;
     }
@@ -1830,7 +1853,7 @@ Transaction *FlatpakBackend::removeApplication(AbstractResource *app)
 void FlatpakBackend::checkForUpdates()
 {
     disconnect(this, &FlatpakBackend::initialized, m_checkForUpdatesTimer, qOverload<>(&QTimer::start));
-    for (const auto &source : qAsConst(m_flatpakSources)) {
+    for (const auto &source : std::as_const(m_flatpakSources)) {
         if (source->remote()) {
             Q_ASSERT(!m_refreshAppstreamMetadataJobs.contains(source->remote()));
             m_refreshAppstreamMetadataJobs.insert(source->remote());
@@ -1880,4 +1903,16 @@ InlineMessage *FlatpakBackend::explainDysfunction() const
     return AbstractResourcesBackend::explainDysfunction();
 }
 
+bool FlatpakBackend::extends(const QString &extends) const
+{
+    for (const auto &source : m_flatpakSources) {
+        if (source->m_pool && !source->m_pool->lastError().isEmpty()) {
+            if (!source->m_pool->componentsByExtends(extends).isEmpty()) {
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
 #include "FlatpakBackend.moc"
