@@ -13,21 +13,11 @@
 
 #include <Transaction/AddonList.h>
 
-#ifdef DISCOVER_USE_STABLE_APPSTREAM
-#include <AppStreamQt5/developer.h>
-#include <AppStreamQt5/icon.h>
-#include <AppStreamQt5/release.h>
-#include <AppStreamQt5/screenshot.h>
-#include <AppStreamQt5/utils.h>
-#include <AppStreamQt5/version.h>
-#else
+#include <AppStreamQt/developer.h>
 #include <AppStreamQt/icon.h>
-#include <AppStreamQt/release.h>
 #include <AppStreamQt/screenshot.h>
 #include <AppStreamQt/utils.h>
 #include <AppStreamQt/version.h>
-#endif
-
 #include <appstream/AppStreamUtils.h>
 
 #include <KConfigGroup>
@@ -36,6 +26,7 @@
 #include <KIO/ApplicationLauncherJob>
 #include <KLocalizedString>
 
+#include <AppStreamQt/release.h>
 #include <QDebug>
 #include <QDesktopServices>
 #include <QDir>
@@ -53,19 +44,22 @@
 #include <QUrlQuery>
 #include <QtConcurrentRun>
 
+using namespace Qt::StringLiterals;
+
 static QString iconCachePath(const AppStream::Icon &icon)
 {
     Q_ASSERT(icon.kind() == AppStream::Icon::KindRemote);
     return QStringLiteral("%1/icons/%2").arg(QStandardPaths::writableLocation(QStandardPaths::CacheLocation), icon.url().fileName());
 }
 
-const QStringList FlatpakResource::s_objects({
+const QStringList FlatpakResource::s_topObjects({
     QStringLiteral("qrc:/qml/FlatpakAttention.qml"),
     QStringLiteral("qrc:/qml/FlatpakRemoveData.qml"),
     QStringLiteral("qrc:/qml/FlatpakOldBeta.qml"),
     QStringLiteral("qrc:/qml/FlatpakEolReason.qml"),
 });
 const QStringList FlatpakResource::s_bottomObjects({QStringLiteral("qrc:/qml/PermissionsList.qml")});
+
 Q_GLOBAL_STATIC(QNetworkAccessManager, manager)
 
 FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakInstallation *installation, FlatpakBackend *parent)
@@ -187,11 +181,6 @@ void FlatpakResource::updateFromAppStream()
     updateFromRef(ref);
 }
 
-QString FlatpakResource::ref() const
-{
-    return typeAsString() + QLatin1Char('/') + flatpakName() + QLatin1Char('/') + arch() + QLatin1Char('/') + branch();
-}
-
 QStringList FlatpakResource::categories()
 {
     auto cats = m_appdata.categories();
@@ -236,7 +225,8 @@ QVariant FlatpakResource::icon() const
             case AppStream::Icon::KindCached: {
                 const QString path = icon.url().toLocalFile();
                 if (QDir::isRelativePath(path)) {
-                    const QString appstreamLocation = installationPath() + "/appstream/" + origin() + '/' + flatpak_get_default_arch() + "/active/icons/";
+                    const QString appstreamLocation =
+                        installationPath() + "/appstream/"_L1 + origin() + '/'_L1 + QString::fromUtf8(flatpak_get_default_arch()) + "/active/icons/"_L1;
                     QDirIterator dit(appstreamLocation, QDirIterator::Subdirectories);
                     while (dit.hasNext()) {
                         const auto currentPath = dit.next();
@@ -280,7 +270,7 @@ QString FlatpakResource::installedVersion() const
         const char *appdataVersion = flatpak_installed_ref_get_appdata_version(ref);
 
         if (appdataVersion) {
-            return appdataVersion;
+            return QString::fromUtf8(appdataVersion);
         }
     }
     return branch();
@@ -403,9 +393,21 @@ QString FlatpakResource::displayOrigin() const
     return !m_displayOrigin.isEmpty() ? m_displayOrigin : m_origin;
 }
 
+// Note: The following three methods are not using each other for optimization purposes:
+// use string builder with arguments directly instead of temporary allocated strings.
 QString FlatpakResource::packageName() const
 {
-    return flatpakName() + QLatin1Char('/') + arch() + QLatin1Char('/') + branch();
+    return QStringLiteral("%1/%2/%3").arg(flatpakName(), arch(), branch());
+}
+
+QString FlatpakResource::ref() const
+{
+    return QStringLiteral("%1/%2/%3/%4").arg(typeAsString(), flatpakName(), arch(), branch());
+}
+
+QString FlatpakResource::installPath() const
+{
+    return QStringLiteral("%1/%2/%3/%4/%5/active").arg(installationPath(), typeAsString(), flatpakName(), arch(), branch());
 }
 
 FlatpakResource::PropertyState FlatpakResource::propertyState(FlatpakResource::PropertyKind kind) const
@@ -439,25 +441,14 @@ quint64 FlatpakResource::size()
 
 QString FlatpakResource::sizeDescription()
 {
-    KFormat f;
-    if (!isInstalled() || canUpgrade()) {
-        if (propertyState(DownloadSize) == NotKnownYet || propertyState(InstalledSize) == NotKnownYet || propertyState(DownloadSize) == Fetching
-            || propertyState(InstalledSize) == Fetching) {
-            qobject_cast<FlatpakBackend *>(backend())->updateAppSize(this);
-            return i18n("Retrieving size information");
-        } else if (propertyState(DownloadSize) == UnknownOrFailed || propertyState(InstalledSize) == UnknownOrFailed) {
-            return i18n("Unknown size");
-        } else {
-            return i18nc("@info app size", "%1 to download, %2 on disk", f.formatByteSize(downloadSize()), f.formatByteSize(installedSize()));
-        }
+    if (propertyState(InstalledSize) == NotKnownYet || propertyState(InstalledSize) == Fetching) {
+        qobject_cast<FlatpakBackend *>(backend())->updateAppSize(this);
+        return i18n("Retrieving size information");
+    } else if (propertyState(InstalledSize) == UnknownOrFailed) {
+        return i18nc("@label app size", "Unknown");
     } else {
-        if (propertyState(InstalledSize) == NotKnownYet || propertyState(InstalledSize) == Fetching) {
-            return i18n("Retrieving size information");
-        } else if (propertyState(InstalledSize) == UnknownOrFailed) {
-            return i18n("Unknown size");
-        } else {
-            return i18nc("@info app size", "%1 on disk", f.formatByteSize(installedSize()));
-        }
+        const KFormat f;
+        return f.formatByteSize(installedSize());
     }
 }
 
@@ -640,23 +631,18 @@ QString FlatpakResource::installationPath(FlatpakInstallation *flatpakInstallati
     return QString::fromUtf8(path_str);
 }
 
-QString FlatpakResource::installPath() const
-{
-    return installationPath() + QStringLiteral("/app/%1/%2/%3/active").arg(flatpakName(), arch(), branch());
-}
-
 QUrl FlatpakResource::url() const
 {
     if (!m_resourceFile.isEmpty()) {
         return m_resourceFile;
     }
 
-    QUrl ret(QStringLiteral("appstream://") + appstreamId());
+    QUrl ret(QStringLiteral("appstream:") + appstreamId());
     const AppStream::Provided::Kind AppStream_Provided_KindId = (AppStream::Provided::Kind)12; // Should be AppStream::Provided::KindId when released
     const auto provided = m_appdata.provided(AppStream_Provided_KindId).items();
     if (!provided.isEmpty()) {
         QUrlQuery qq;
-        qq.addQueryItem("alt", provided.join(QLatin1Char(',')));
+        qq.addQueryItem(u"alt"_s, provided.join(QLatin1Char(',')));
         ret.setQuery(qq);
     }
     return ret;
@@ -665,8 +651,8 @@ QUrl FlatpakResource::url() const
 QDate FlatpakResource::releaseDate() const
 {
 #if ASQ_CHECK_VERSION(1, 0, 0)
-    if (const auto optional = m_appdata.releasesPlain().indexSafe(0); optional.has_value()) {
-        auto release = optional.value();
+    if (!m_appdata.releasesPlain().isEmpty()) {
+        auto release = m_appdata.releasesPlain().indexSafe(0).value();
 #else
     if (const auto releases = m_appdata.releases(); !releases.isEmpty()) {
         auto release = releases.constFirst();
@@ -693,11 +679,7 @@ QString FlatpakResource::sourceIcon() const
 
 QString FlatpakResource::author() const
 {
-#if ASQ_CHECK_VERSION(1, 0, 0)
     QString name = m_appdata.developer().name();
-#else
-    QString name = m_appdata.developerName();
-#endif
 
     if (name.isEmpty()) {
         name = m_appdata.projectGroup();
@@ -733,7 +715,7 @@ QString FlatpakResource::versionString()
     if (isInstalled()) {
         auto ref = qobject_cast<FlatpakBackend *>(backend())->getInstalledRefForApp(this);
         if (ref) {
-            version = flatpak_installed_ref_get_appdata_version(ref);
+            version = QString::fromUtf8(flatpak_installed_ref_get_appdata_version(ref));
         }
 #if ASQ_CHECK_VERSION(1, 0, 0)
     } else if (!m_appdata.releasesPlain().isEmpty()) {
@@ -786,9 +768,18 @@ QString FlatpakResource::eolReason()
     return m_eolReason.value_or(QString());
 }
 
+QString createHtmlList(const QStringList &itemList)
+{
+    QString str = QStringLiteral("<ul>");
+    for (const QString &itemText : std::as_const(itemList))
+      str += QStringLiteral("<li>%1</li>").arg(itemText.toHtmlEscaped());
+    str += QStringLiteral("</ul>");
+    return str;
+}
+
 void FlatpakResource::loadPermissions()
 {
-    QByteArray metaDataBytes = FlatpakRunnables::fetchMetadata(this, NULL);
+    QByteArray metaDataBytes = FlatpakRunnables::fetchMetadata(this, nullptr);
 
     QTemporaryFile f;
     if (!f.open()) {
@@ -806,72 +797,78 @@ void FlatpakResource::loadPermissions()
 
     const KConfigGroup contextGroup = parser.group("Context");
     const QString shared = contextGroup.readEntry("shared", QString());
-    if (shared.contains("network")) {
+    if (shared.contains("network"_L1)) {
         brief = i18n("Network Access");
         description = i18n("Can access the internet");
-        m_permissions.append(FlatpakPermission(brief, description, "network-wireless"));
+        m_permissions.append(FlatpakPermission(brief, description, u"network-wireless-symbolic"_s));
     }
 
     const QString sockets = contextGroup.readEntry("sockets", QString());
-    if (sockets.contains("session-bus")) {
+    const QString filesystems = contextGroup.readEntry("filesystems", QString());
+    const auto dirs = QStringView(filesystems).split(';'_L1, Qt::SkipEmptyParts);
+
+    if (sockets.contains("pulseaudio"_L1) || filesystems.contains("xdg-run/pipewire-0"_L1)) {
+        brief = i18n("Sound system access");
+        description = i18n("Can play audio");
+        m_permissions.append(FlatpakPermission(brief, description, u"audio-speakers-symbolic"_s));
+    }
+    if (sockets.contains("session-bus"_L1)) {
         brief = i18n("Session Bus Access");
-        description = i18n("Access is granted to the entire Session Bus");
-        m_permissions.append(FlatpakPermission(brief, description, "system-save-session"));
+        description = i18n("Can communicate with all other applications and processes run in this user account");
+        m_permissions.append(FlatpakPermission(brief, description, u"security-medium-symbolic"_s));
         fullSessionBusAccess = true;
     }
-    if (sockets.contains("system-bus")) {
+    if (sockets.contains("system-bus"_L1)) {
         brief = i18n("System Bus Access");
-        description = i18n("Access is granted to the entire System Bus");
-        m_permissions.append(FlatpakPermission(brief, description, "system-save-session"));
+        description = i18n("Can communicate with all other applications and processes on the system");
+        m_permissions.append(FlatpakPermission(brief, description, u"security-medium-symbolic"_s));
         fullSystemBusAccess = true;
     }
-    if (sockets.contains("ssh-auth")) {
+    if (sockets.contains("ssh-auth"_L1)) {
         brief = i18n("Remote Login Access");
         description = i18n("Can initiate remote login requests using the SSH protocol");
-        m_permissions.append(FlatpakPermission(brief, description, "x-shape-connection"));
+        m_permissions.append(FlatpakPermission(brief, description, u"x-shape-connection-symbolic"_s));
     }
-    if (sockets.contains("pcsc")) {
+    if (sockets.contains("pcsc"_L1)) {
         brief = i18n("Smart Card Access");
         description = i18n("Can integrate and communicate with smart cards");
-        m_permissions.append(FlatpakPermission(brief, description, "network-card"));
+        m_permissions.append(FlatpakPermission(brief, description, u"auth-sim-symbolic"_s));
     }
-    if (sockets.contains("cups")) {
+    if (sockets.contains("cups"_L1)) {
         brief = i18n("Printer Access");
         description = i18n("Can integrate and communicate with printers");
-        m_permissions.append(FlatpakPermission(brief, description, "document-print"));
+        m_permissions.append(FlatpakPermission(brief, description, u"printer-symbolic"_s));
     }
-    if (sockets.contains("gpg-agent")) {
+    if (sockets.contains("gpg-agent"_L1)) {
         brief = i18n("GPG Agent");
         description = i18n("Allows access to the GPG cryptography service, generally used for signing and reading signed documents");
-        m_permissions.append(FlatpakPermission(brief, description, "gpg"));
+        m_permissions.append(FlatpakPermission(brief, description, u"document-edit-sign-encrypt"_s));
     }
 
     const QString features = contextGroup.readEntry("features", QString());
-    if (features.contains("bluetooth")) {
+    if (features.contains("bluetooth"_L1)) {
         brief = i18n("Bluetooth Access");
         description = i18n("Can integrate and communicate with Bluetooth devices");
-        m_permissions.append(FlatpakPermission(brief, description, "network-bluetooth"));
+        m_permissions.append(FlatpakPermission(brief, description, u"network-bluetooth-symbolic"_s));
     }
-    if (features.contains("devel")) {
+    if (features.contains("devel"_L1)) {
         brief = i18n("Low-Level System Access");
         description = i18n("Can make low-level system calls (e.g. ptrace)");
-        m_permissions.append(FlatpakPermission(brief, description, "project-development"));
+        m_permissions.append(FlatpakPermission(brief, description, u"run-build-symbolic"_s));
     }
 
     const QString devices = contextGroup.readEntry("devices", QString());
-    if (devices.contains("all")) {
+    if (devices.contains("all"_L1)) {
         brief = i18n("Device Access");
         description = i18n("Can communicate with and control built-in or connected hardware devices");
-        m_permissions.append(FlatpakPermission(brief, description, "preferences-devices-tree"));
+        m_permissions.append(FlatpakPermission(brief, description, u"device-notifier-symbolic"_s));
     }
-    if (devices.contains("kvm")) {
+    if (devices.contains("kvm"_L1)) {
         brief = i18n("Kernel-based Virtual Machine Access");
         description = i18n("Allows running other operating systems as guests in virtual machines");
-        m_permissions.append(FlatpakPermission(brief, description, "virtualbox"));
+        m_permissions.append(FlatpakPermission(brief, description, u"virt-manager-symbolic"_s));
     }
 
-    const QString filesystems = contextGroup.readEntry("filesystems", QString());
-    const auto dirs = QStringView(filesystems).split(';', Qt::SkipEmptyParts);
     QStringList homeList, systemList;
     bool home_ro = false, home_rw = false, home_cr = false, homeAccess = false;
     bool system_ro = false, system_rw = false, system_cr = false, systemAccess = false;
@@ -884,11 +881,16 @@ void FlatpakResource::loadPermissions()
             continue;
         }
 
-        int separator = dir.lastIndexOf(':');
+        if (dir.startsWith("xdg-run/pipewire-0"_L1)) {
+            // This is already handled as sound system above
+            continue;
+        }
+
+        int separator = dir.lastIndexOf(':'_L1);
         const QStringView postfix = separator > 0 ? dir.mid(separator) : QStringView();
         const QStringView symbolicName = dir.left(separator);
         const QString id = translateSymbolicName(symbolicName);
-        if ((dir.contains(QLatin1String("home")) || dir.contains(QChar('~')))) {
+        if ((dir.contains(QLatin1String("home")) || dir.contains(QLatin1Char('~')))) {
             if (postfix == QLatin1String(":ro")) {
                 homeList << i18n("%1 (read-only)", id);
                 home_ro = true;
@@ -921,7 +923,7 @@ void FlatpakResource::loadPermissions()
         }
     }
 
-    QString appendText = "\n- " + homeList.join("\n- ");
+    QString appendText = createHtmlList(homeList);
     if (homeAccess) {
         brief = i18n("Home Folder Access");
         if (home_rw && home_ro && home_cr) {
@@ -934,9 +936,9 @@ void FlatpakResource::loadPermissions()
         } else {
             description = i18n("Can access files in the following locations in your home folder without asking permission first: %1", appendText);
         }
-        m_permissions.append(FlatpakPermission(brief, description, "inode-directory"));
+        m_permissions.append(FlatpakPermission(brief, description, u"user-home-symbolic"_s));
     }
-    appendText = "\n- " + systemList.join("\n- ");
+    appendText = createHtmlList(systemList);
     if (systemAccess) {
         brief = i18n("System Folder Access");
         if (system_rw && system_ro && system_cr) {
@@ -948,7 +950,7 @@ void FlatpakResource::loadPermissions()
         } else {
             description = i18n("Can access system files in the following locations without asking permission first: %1", appendText);
         }
-        m_permissions.append(FlatpakPermission(brief, description, "inode-directory"));
+        m_permissions.append(FlatpakPermission(brief, description, u"drive-harddisk-root-symbolic"_s));
     }
 
     if (!fullSessionBusAccess) {
@@ -957,8 +959,8 @@ void FlatpakResource::loadPermissions()
             const QStringList busList = sessionBusGroup.keyList();
             brief = i18n("Session Bus Access");
             description = i18n("Can communicate with other applications and processes in the same desktop session using the following communication protocols: %1",
-                           "\n- " + busList.join("\n- "));
-            m_permissions.append(FlatpakPermission(brief, description, "system-save-session"));
+                           createHtmlList(busList));
+            m_permissions.append(FlatpakPermission(brief, description, "plugins-symbolic"_L1));
         }
     }
 
@@ -968,15 +970,15 @@ void FlatpakResource::loadPermissions()
             const QStringList busList = systemBusGroup.keyList();
             brief = i18n("System Bus Access");
             description =
-                i18n("Can communicate with all applications and system services using the following communication protocols: %1", "\n- " + busList.join("\n- "));
-            m_permissions.append(FlatpakPermission(brief, description, "system-save-session"));
+                i18n("Can communicate with all applications and system services using the following communication protocols: %1", createHtmlList(busList));
+            m_permissions.append(FlatpakPermission(brief, description, "plugins-symbolic"_L1));
         }
     }
 }
 
 QString FlatpakResource::dataLocation() const
 {
-    auto id = m_appdata.bundle(AppStream::Bundle::KindFlatpak).id().section('/', 0, 1);
+    auto id = m_appdata.bundle(AppStream::Bundle::KindFlatpak).id().section('/'_L1, 0, 1);
     if (id.isEmpty()) {
         return {};
     }
