@@ -21,6 +21,8 @@
 #include <resources/DiscoverAction.h>
 #include <resources/StoredResultsStream.h>
 
+using namespace Qt::StringLiterals;
+
 class FlatpakSourceItem : public QStandardItem
 {
 public:
@@ -85,8 +87,8 @@ FlatpakSourcesBackend::FlatpakSourcesBackend(const QVector<FlatpakInstallation *
     : AbstractSourcesBackend(parent)
     , m_preferredInstallation(installations.constFirst())
     , m_sources(new QStandardItemModel(this))
-    , m_flathubAction(new DiscoverAction("flatpak-discover", i18n("Add Flathub"), this))
-    , m_saveAction(new DiscoverAction("dialog-ok-apply", i18n("Apply Changes"), this))
+    , m_flathubAction(new DiscoverAction(u"flatpak-discover"_s, i18n("Add Flathub"), this))
+    , m_saveAction(new DiscoverAction(u"dialog-ok-apply"_s, i18n("Apply Changes"), this))
     , m_noSourcesItem(new QStandardItem(QStringLiteral("-")))
 {
     m_saveAction->setVisible(false);
@@ -96,7 +98,7 @@ FlatpakSourcesBackend::FlatpakSourcesBackend(const QVector<FlatpakInstallation *
     m_flathubAction->setObjectName(QStringLiteral("flathub"));
     m_flathubAction->setToolTip(i18n("Makes it possible to easily install the applications listed in https://flathub.org"));
     connect(m_flathubAction, &DiscoverAction::triggered, this, [this]() {
-        addSource(QStringLiteral("https://flathub.org/repo/flathub.flatpakrepo"));
+        addSource(QStringLiteral("https://dl.flathub.org/repo/flathub.flatpakrepo"));
     });
 
     m_noSourcesItem->setEnabled(false);
@@ -156,9 +158,9 @@ bool FlatpakSourcesBackend::addSource(const QString &id)
     if (id.isEmpty() || !flatpakrepoUrl.isValid())
         return false;
 
-    auto addSource = [=](AbstractResource *res) {
-        if (res)
-            backend->installApplication(res);
+    auto addSource = [=](const StreamResult &res) {
+        if (res.resource)
+            backend->installApplication(res.resource);
         else
             Q_EMIT backend->passiveMessage(i18n("Could not add the source %1", flatpakrepoUrl.toDisplayString()));
     };
@@ -166,7 +168,7 @@ bool FlatpakSourcesBackend::addSource(const QString &id)
     if (flatpakrepoUrl.isLocalFile()) {
         auto stream = new ResultsStream(QStringLiteral("FlatpakSource-") + flatpakrepoUrl.toDisplayString());
         backend->addSourceFromFlatpakRepo(flatpakrepoUrl, stream);
-        connect(stream, &ResultsStream::resourcesFound, this, [addSource](const QVector<AbstractResource *> &res) {
+        connect(stream, &ResultsStream::resourcesFound, this, [addSource](const QVector<StreamResult> &res) {
             addSource(res.constFirst());
         });
     } else {
@@ -264,7 +266,7 @@ bool FlatpakSourcesBackend::removeSource(const QString &id)
                     g_autoptr(GError) localError = nullptr;
                     g_autoptr(GCancellable) cancellable = g_cancellable_new();
                     g_autoptr(FlatpakTransaction) transaction = flatpak_transaction_new_for_installation(installation, cancellable, &localError);
-                    for (const QString &instRef : qAsConst(toRemoveRefs)) {
+                    for (const QString &instRef : std::as_const(toRemoveRefs)) {
                         const QByteArray refString = instRef.toUtf8();
                         flatpak_transaction_add_uninstall(transaction, refString.constData(), &localError);
                         if (localError)
