@@ -12,17 +12,14 @@
 #include <QStandardPaths>
 #include <QTimer>
 
-// Attica includes
-#include <Attica/Content>
-#include <Attica/ProviderManager>
-
 // KDE includes
 #include <KConfig>
 #include <KConfigGroup>
 #include <KLocalizedString>
-#include <KNSCore/Engine>
+#include <KNSCore/Provider>
+#include <KNSCore/Question>
 #include <KNSCore/QuestionManager>
-#include <knewstuffcore_version.h>
+#include <KNSCore/ResultsStream>
 
 // DiscoverCommon includes
 #include "Category/Category.h"
@@ -33,6 +30,7 @@
 #include "KNSBackend.h"
 #include "KNSResource.h"
 #include "KNSReviews.h"
+#include "KNSTransaction.h"
 #include "utils.h"
 #include <resources/StandardBackendUpdater.h>
 
@@ -46,16 +44,37 @@ class KNSBackendFactory : public AbstractResourcesBackendFactory
 public:
     KNSBackendFactory()
     {
-        connect(KNSCore::QuestionManager::instance(), &KNSCore::QuestionManager::askQuestion, this, [](KNSCore::Question *q) {
-            qWarning() << q->question() << q->questionType();
-            q->setResponse(KNSCore::Question::InvalidResponse);
+        connect(KNSCore::QuestionManager::instance(), &KNSCore::QuestionManager::askQuestion, this, [](KNSCore::Question *question) {
+            const auto transactions = TransactionModel::global()->transactions();
+            for (auto t : transactions) {
+                const auto transaction = dynamic_cast<KNSTransaction *>(t);
+                if (!transaction) {
+                    continue;
+                }
+
+                if (question->entry().uniqueId() == transaction->uniqueId()) {
+                    switch (question->questionType()) {
+                    case KNSCore::Question::ContinueCancelQuestion:
+                        transaction->addQuestion(question);
+                        return;
+                    default:
+                        transaction->passiveMessage(i18n("Unsupported question:\n%1", question->question()));
+                        question->setResponse(KNSCore::Question::InvalidResponse);
+                        transaction->setStatus(Transaction::CancelledStatus);
+                        break;
+                    }
+                    return;
+                }
+            }
+            qWarning() << "Question for unknown transaction" << question->question() << question->questionType();
+            question->setResponse(KNSCore::Question::InvalidResponse);
         });
     }
 
     QVector<AbstractResourcesBackend *> newInstance(QObject *parent, const QString & /*name*/) const override
     {
         QVector<AbstractResourcesBackend *> ret;
-        const QStringList availableConfigFiles = KNSCore::Engine::availableConfigFiles();
+        const QStringList availableConfigFiles = KNSCore::EngineBase::availableConfigFiles();
         for (const QString &configFile : availableConfigFiles) {
             auto bk = new KNSBackend(parent, QStringLiteral("plasma"), configFile);
             if (bk->isValid())
@@ -67,9 +86,36 @@ public:
     }
 };
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-Q_DECLARE_METATYPE(KNSCore::EntryInternal)
-#endif
+class KNSResultsStream : public ResultsStream
+{
+    Q_OBJECT
+public:
+    KNSResultsStream(KNSBackend *backend, const QString &objectName)
+        : ResultsStream(objectName)
+        , m_backend(backend)
+    {
+    }
+
+    void setRequest(const KNSCore::Provider::SearchRequest &request)
+    {
+        KNSCore::ResultsStream *job = m_backend->engine()->search(request);
+        connect(job, &KNSCore::ResultsStream::entriesFound, this, &KNSResultsStream::addEntries);
+        connect(job, &KNSCore::ResultsStream::finished, this, &KNSResultsStream::finish);
+        connect(this, &ResultsStream::fetchMore, job, &KNSCore::ResultsStream::fetchMore);
+        job->fetch();
+    }
+
+    void addEntries(const KNSCore::Entry::List &entries)
+    {
+        const auto res = kTransform<QList<StreamResult>>(entries, [this](const auto &entry) {
+            return StreamResult{m_backend->resourceForEntry(entry), 0};
+        });
+        Q_EMIT resourcesFound(res);
+    }
+
+private:
+    KNSBackend *const m_backend;
+};
 
 KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &knsrc)
     : AbstractResourcesBackend(parent)
@@ -86,7 +132,7 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
 
     const KConfig conf(m_name, KConfig::SimpleConfig);
     if (!conf.hasGroup("KNewStuff3")) {
-        markInvalid(QStringLiteral("Config group not found! Check your KNS3 installation."));
+        markInvalid(QStringLiteral("Config group not found! Check your KNSCore installation."));
         return;
     }
 
@@ -105,11 +151,10 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
     QTimer::singleShot(60000, this, [this]() {
         if (!m_initialized) {
             markInvalid(i18n("Backend %1 took too long to initialize", m_displayName));
-            setResponsePending(false);
         }
     });
 
-    const QVector<QPair<FilterType, QString>> filters = {{CategoryFilter, fileName}};
+    const CategoryFilter filter = {CategoryFilter::CategoryNameFilter, fileName};
     const QSet<QString> backendName = {name()};
     m_displayName = group.readEntry("Name", QString());
     if (m_displayName.isEmpty()) {
@@ -124,16 +169,16 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
         m_categories += cats;
         for (const auto &cat : cats) {
             if (m_hasApplications)
-                categories << new Category(cat, QStringLiteral("applications-other"), {{CategoryFilter, cat}}, backendName, {}, true);
+                categories << new Category(cat, QStringLiteral("applications-other"), {CategoryFilter::CategoryNameFilter, cat}, backendName, {}, true);
             else
-                categories << new Category(cat, QStringLiteral("plasma"), {{CategoryFilter, cat}}, backendName, {}, true);
+                categories << new Category(cat, QStringLiteral("plasma"), {CategoryFilter::CategoryNameFilter, cat}, backendName, {}, true);
         }
     }
 
     QVector<Category *> topCategories{categories};
-    for (const auto &cat : qAsConst(categories)) {
+    for (const auto &cat : std::as_const(categories)) {
         const QString catName = cat->name().append(QLatin1Char('/'));
-        for (const auto &potentialSubCat : qAsConst(categories)) {
+        for (const auto &potentialSubCat : std::as_const(categories)) {
             if (potentialSubCat->name().startsWith(catName)) {
                 cat->addSubcategory(potentialSubCat);
                 topCategories.removeOne(potentialSubCat);
@@ -141,38 +186,37 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
         }
     }
 
-    m_engine = new KNSCore::Engine(this);
-    connect(m_engine, &KNSCore::Engine::signalErrorCode, this, &KNSBackend::slotErrorCode);
-    connect(m_engine, &KNSCore::Engine::signalEntryEvent, this, &KNSBackend::slotEntryEvent);
-    connect(m_engine, &KNSCore::Engine::signalEntriesLoaded, this, &KNSBackend::receivedEntries, Qt::QueuedConnection);
-    connect(m_engine, &KNSCore::Engine::signalProvidersLoaded, this, &KNSBackend::fetchInstalled);
-    connect(m_engine, &KNSCore::Engine::signalUpdateableEntriesLoaded, this, [this] {
-        if (m_responsePending) {
-            setResponsePending(false);
-        }
+    m_engine = new KNSCore::EngineBase(this);
+    connect(m_engine, &KNSCore::EngineBase::signalErrorCode, this, &KNSBackend::slotErrorCode);
+    connect(m_engine, &KNSCore::EngineBase::providersChanged, this, [this] {
+        setFetching(false);
     });
-    connect(m_engine, &KNSCore::Engine::signalCategoriesMetadataLoded, this, [categories](const QList<KNSCore::Provider::CategoryMetadata> &categoryMetadatas) {
-        for (const KNSCore::Provider::CategoryMetadata &category : categoryMetadatas) {
-            for (Category *cat : qAsConst(categories)) {
-                if (cat->orFilters().count() > 0 && cat->orFilters().constFirst().second == category.name) {
-                    cat->setName(category.displayName);
-                    break;
+
+    connect(m_engine,
+            &KNSCore::EngineBase::signalCategoriesMetadataLoded,
+            this,
+            [categories](const QList<KNSCore::Provider::CategoryMetadata> &categoryMetadatas) {
+                for (const KNSCore::Provider::CategoryMetadata &category : categoryMetadatas) {
+                    for (Category *cat : std::as_const(categories)) {
+                        if (cat->matchesCategoryName(category.name)) {
+                            cat->setName(category.displayName);
+                            break;
+                        }
+                    }
                 }
-            }
-        }
-    });
-    m_engine->setPageSize(ENGINE_PAGE_SIZE);
+            });
     m_engine->init(m_name);
 
     if (m_hasApplications) {
-        auto actualCategory = new Category(m_displayName, QStringLiteral("applications-other"), filters, backendName, topCategories, false);
+        auto actualCategory = new Category(m_displayName, QStringLiteral("applications-other"), filter, backendName, topCategories, false);
         auto applicationCategory = new Category(i18n("Applications"), //
                                                 QStringLiteral("applications-internet"),
-                                                filters,
+                                                filter,
                                                 backendName,
                                                 {actualCategory},
                                                 false);
-        applicationCategory->setAndFilter({{CategoryFilter, QLatin1String("Application")}});
+        const QVector<CategoryFilter> filters = {{CategoryFilter::CategoryNameFilter, QLatin1String("Application")}, filter};
+        applicationCategory->setFilter({CategoryFilter::AndFilter, filters});
         m_categories.append(applicationCategory->name());
         m_rootCategories = {applicationCategory};
         // Make sure we filter out any apps which won't run on the current system architecture
@@ -208,11 +252,11 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
             QStringLiteral("ksplash.knsrc"),       QStringLiteral("window-decorations.knsrc"),
         };
         const auto iconName = knsrcPlasma.contains(fileName) ? QStringLiteral("plasma") : QStringLiteral("applications-other");
-        auto actualCategory = new Category(m_displayName, iconName, filters, backendName, categories, true);
+        auto actualCategory = new Category(m_displayName, iconName, filter, backendName, categories, true);
         actualCategory->setParent(this);
 
         const auto topLevelName = knsrcPlasma.contains(fileName) ? i18n("Plasma Addons") : i18n("Application Addons");
-        auto addonsCategory = new Category(topLevelName, iconName, filters, backendName, {actualCategory}, true);
+        auto addonsCategory = new Category(topLevelName, iconName, filter, backendName, {actualCategory}, true);
         m_rootCategories = {addonsCategory};
     }
 
@@ -233,93 +277,11 @@ void KNSBackend::markInvalid(const QString &message)
     Q_EMIT initialized();
 }
 
-void KNSBackend::setResponsePending(bool pending)
-{
-    Q_ASSERT(m_responsePending != pending);
-    m_responsePending = pending;
-    if (pending) {
-        Q_EMIT startingSearch();
-    } else {
-        Q_EMIT availableForQueries();
-        setFetching(false);
-        m_onePage = false;
-    }
-}
-
-void KNSBackend::fetchInstalled()
-{
-    auto search = new OneTimeAction(
-        666,
-        [this]() {
-            // First we ensure we've got data loaded on what we've got installed already
-            if (m_responsePending) {
-                // Slot already taken, will need to wait again
-                return false;
-            }
-            m_onePage = true;
-            setResponsePending(true);
-            m_engine->checkForInstalled();
-            // And then we check for updates - we could do only one, if all we cared about was updates,
-            // but to have both a useful initial list, /and/ information on updates, we want to get both.
-            // The reason we are not doing a checkUpdates() overload for this is that the caching for this
-            // information is done by KNSEngine, and we want to actually load it every time we initialize.
-            auto updateChecker = new OneTimeAction(
-                666,
-                [this] {
-                    //No need to check for updates if there's no resources
-                    if (m_resourcesByName.isEmpty()) {
-                        return true;
-                    }
-
-                    if (m_responsePending) {
-                        // Slot already taken, will need to wait again
-                        return false;
-                    }
-
-                    m_onePage = true;
-                    setResponsePending(true);
-                    m_engine->checkForUpdates();
-                    return true;
-                },
-                this);
-            connect(this, &KNSBackend::availableForQueries, updateChecker, &OneTimeAction::trigger, Qt::QueuedConnection);
-            return true;
-        },
-        this);
-
-    if (m_responsePending) {
-        connect(this, &KNSBackend::availableForQueries, search, &OneTimeAction::trigger, Qt::QueuedConnection);
-    } else {
-        search->trigger();
-    }
-}
-
 void KNSBackend::checkForUpdates()
 {
-    // Since we load the updates during initialization already, don't overburden
-    // the machine with multiple of these, because that would just be silly.
-    if (m_initialized) {
-        auto updateChecker = new OneTimeAction(
-            666,
-            [this] {
-                if (m_responsePending) {
-                    // Slot already taken, will need to wait again
-                    return false;
-                }
-
-                m_onePage = true;
-                setResponsePending(true);
-                m_engine->checkForUpdates();
-                return true;
-            },
-            this);
-
-        if (m_responsePending) {
-            connect(this, &KNSBackend::availableForQueries, updateChecker, &OneTimeAction::trigger, Qt::QueuedConnection);
-        } else {
-            updateChecker->trigger();
-        }
-    }
+    AbstractResourcesBackend::Filters filter;
+    filter.state = AbstractResource::Upgradeable;
+    search(filter);
 }
 
 void KNSBackend::setFetching(bool f)
@@ -339,11 +301,14 @@ bool KNSBackend::isValid() const
     return m_isValid;
 }
 
-KNSResource *KNSBackend::resourceForEntry(const KNSCore::EntryInternal &entry)
+KNSResource *KNSBackend::resourceForEntry(const KNSCore::Entry &entry)
 {
     KNSResource *r = static_cast<KNSResource *>(m_resourcesByName.value(entry.uniqueId()));
     if (!r) {
-        QStringList categories{name(), m_rootCategories.first()->name()};
+        QStringList categories{name()};
+        if (!m_rootCategories.isEmpty()) {
+            categories << m_rootCategories.first()->name();
+        }
         const auto cats = m_engine->categoriesMetadata();
         const int catIndex = kIndexOf(cats, [&entry](const KNSCore::Provider::CategoryMetadata &cat) {
             return entry.category() == cat.id;
@@ -362,41 +327,7 @@ KNSResource *KNSBackend::resourceForEntry(const KNSCore::EntryInternal &entry)
     return r;
 }
 
-void KNSBackend::receivedEntries(const KNSCore::EntryInternal::List &entries)
-{
-    if (!m_isValid)
-        return;
-
-    const auto filtered = kFilter<KNSCore::EntryInternal::List>(entries, [](const KNSCore::EntryInternal &entry) {
-        return entry.isValid();
-    });
-    const auto resources = kTransform<QVector<AbstractResource *>>(filtered, [this](const KNSCore::EntryInternal &entry) {
-        return resourceForEntry(entry);
-    });
-
-    if (!resources.isEmpty()) {
-        Q_EMIT receivedResources(resources);
-    }
-
-    setResponsePending(false);
-    if (m_onePage || resources.count() < ENGINE_PAGE_SIZE) {
-        Q_EMIT searchFinished();
-    }
-}
-
-void KNSBackend::fetchMore()
-{
-    if (m_responsePending)
-        return;
-
-    // We _have_ to set this first. If we do not, we may run into a situation where the
-    // data request will conclude immediately, causing m_responsePending to remain true
-    // for perpetuity as the slots will be called before the function returns.
-    setResponsePending(true);
-    m_engine->requestMoreData();
-}
-
-void KNSBackend::statusChanged(const KNSCore::EntryInternal &entry)
+void KNSBackend::statusChanged(const KNSCore::Entry &entry)
 {
     resourceForEntry(entry);
 }
@@ -450,7 +381,7 @@ void KNSBackend::slotErrorCode(const KNSCore::ErrorCode &errorCode, const QStrin
             // query to the user) but we can give them an idea of how to deal with the
             // situation some other way.
             // TODO: Once Discover has a way to forward queries to the user from transactions, this likely will no longer be needed
-            if (r->entry().status() == KNS3::Entry::Updateable) {
+            if (r->entry().status() == KNSCore::Entry::Updateable) {
                 error = i18n(
                     "Unable to complete the update of %1. You can try and perform this action through the Get Hot New Stuff dialog, which grants tighter "
                     "control. The reported error was:\n%2",
@@ -469,109 +400,27 @@ void KNSBackend::slotErrorCode(const KNSCore::ErrorCode &errorCode, const QStrin
         error = i18n("Unhandled error in %1 backend. Contact your distributor.", m_displayName);
         break;
     }
-    if (m_responsePending) {
-        setResponsePending(false);
-    }
     qWarning() << "kns error" << objectName() << error;
     if (!invalidFile)
         Q_EMIT passiveMessage(i18n("%1: %2", name(), error));
 }
 
-void KNSBackend::slotEntryEvent(const KNSCore::EntryInternal &entry, KNSCore::EntryInternal::EntryEvent event)
+void KNSBackend::slotEntryEvent(const KNSCore::Entry &entry, KNSCore::Entry::EntryEvent event)
 {
     switch (event) {
-    case KNSCore::EntryInternal::StatusChangedEvent:
+    case KNSCore::Entry::StatusChangedEvent:
         statusChanged(entry);
         break;
-    case KNSCore::EntryInternal::DetailsLoadedEvent:
+    case KNSCore::Entry::DetailsLoadedEvent:
         detailsLoaded(entry);
         break;
-    case KNSCore::EntryInternal::AdoptedEvent:
-    case KNSCore::EntryInternal::UnknownEvent:
+    case KNSCore::Entry::AdoptedEvent:
+    case KNSCore::Entry::UnknownEvent:
     default:
         break;
     }
 }
 
-class KNSTransaction : public Transaction
-{
-public:
-    KNSTransaction(QObject *parent, KNSResource *res, Transaction::Role role)
-        : Transaction(parent, res, role)
-        , m_id(res->entry().uniqueId())
-    {
-        setCancellable(false);
-
-        auto manager = res->knsBackend()->engine();
-        connect(manager, &KNSCore::Engine::signalEntryEvent, this, [this](const KNSCore::EntryInternal &entry, KNSCore::EntryInternal::EntryEvent event) {
-            switch (event) {
-            case KNSCore::EntryInternal::StatusChangedEvent:
-                anEntryChanged(entry);
-                break;
-            case KNSCore::EntryInternal::DetailsLoadedEvent:
-            case KNSCore::EntryInternal::AdoptedEvent:
-            case KNSCore::EntryInternal::UnknownEvent:
-            default:
-                break;
-            }
-        });
-        TransactionModel::global()->addTransaction(this);
-
-        std::function<void()> actionFunction;
-        auto engine = res->knsBackend()->engine();
-        if (role == RemoveRole)
-            actionFunction = [res, engine]() {
-                engine->uninstall(res->entry());
-            };
-        else if (res->entry().status() == KNS3::Entry::Updateable)
-            actionFunction = [res, engine]() {
-                engine->install(res->entry(), -1);
-            };
-        else if (res->linkIds().isEmpty())
-            actionFunction = [res]() {
-                qWarning() << "No installable candidates in the KNewStuff entry" << res->entry().name() << "with id" << res->entry().uniqueId()
-                           << "on the backend" << res->backend()->name()
-                           << "There should always be at least one downloadable item in an OCS entry, and if there isn't, we should consider it broken. OCS "
-                              "can technically show them, but if there is nothing to install, it cannot be installed.";
-            };
-        else
-            actionFunction = [res, engine]() {
-                engine->install(res->entry());
-            };
-        QTimer::singleShot(0, res, actionFunction);
-    }
-
-    void anEntryChanged(const KNSCore::EntryInternal &entry)
-    {
-        if (entry.uniqueId() == m_id) {
-            switch (entry.status()) {
-            case KNS3::Entry::Invalid:
-                qWarning() << "invalid status for" << entry.uniqueId() << entry.status();
-                break;
-            case KNS3::Entry::Installing:
-            case KNS3::Entry::Updating:
-                setStatus(CommittingStatus);
-                break;
-            case KNS3::Entry::Downloadable:
-            case KNS3::Entry::Installed:
-            case KNS3::Entry::Deleted:
-            case KNS3::Entry::Updateable:
-                if (status() != DoneStatus) {
-                    setStatus(DoneStatus);
-                }
-                break;
-            }
-        }
-    }
-
-    void cancel() override
-    {
-    }
-
-private:
-    const QString m_id;
-};
-
 Transaction *KNSBackend::removeApplication(AbstractResource *app)
 {
     auto res = qobject_cast<KNSResource *>(app);
@@ -612,18 +461,11 @@ ResultsStream *KNSBackend::search(const AbstractResourcesBackend::Filters &filte
     if (filter.resourceUrl.scheme() == QLatin1String("kns")) {
         return findResourceByPackageName(filter.resourceUrl);
     } else if (filter.state >= AbstractResource::Installed) {
-        auto stream = new ResultsStream(QStringLiteral("KNS-installed"));
-
+        auto stream = new KNSResultsStream(this, QStringLiteral("KNS-installed"));
         const auto start = [this, stream, filter]() {
             if (m_isValid) {
-                auto filterFunction = [&filter](AbstractResource *r) {
-                    return r->state() >= filter.state
-                        && (r->name().contains(filter.search, Qt::CaseInsensitive) || r->comment().contains(filter.search, Qt::CaseInsensitive));
-                };
-                const auto ret = kFilter<QVector<AbstractResource *>>(m_resourcesByName, filterFunction);
-
-                if (!ret.isEmpty())
-                    Q_EMIT stream->resourcesFound(ret);
+                const auto knsFilter = filter.state == AbstractResource::Installed ? KNSCore::Provider::Installed : KNSCore::Provider::Updates;
+                stream->setRequest(KNSCore::Provider::SearchRequest(KNSCore::Provider::Newest, knsFilter, {}, {}, -1, ENGINE_PAGE_SIZE));
             }
             stream->finish();
         };
@@ -639,18 +481,14 @@ ResultsStream *KNSBackend::search(const AbstractResourcesBackend::Filters &filte
                || (filter.category && kContains(m_categories, [&filter](const QString &cat) {
                        return filter.category->matchesCategoryName(cat);
                    }))) {
-        auto r = new ResultsStream(QLatin1String("KNS-search-") + name());
-        searchStream(r, filter.search);
-        return r;
+        return searchStream(filter.search);
     }
     return voidStream();
 }
 
-void KNSBackend::searchStream(ResultsStream *stream, const QString &searchText)
+KNSResultsStream *KNSBackend::searchStream(const QString &searchText)
 {
-    Q_EMIT startingSearch();
-
-    stream->setProperty("alreadyStarted", false);
+    auto stream = new KNSResultsStream(this, QLatin1String("KNS-search-") + name());
     auto start = [this, stream, searchText]() {
         Q_ASSERT(!isFetching());
         if (!m_isValid) {
@@ -658,30 +496,16 @@ void KNSBackend::searchStream(ResultsStream *stream, const QString &searchText)
             stream->finish();
             return;
         }
-
-        if (m_responsePending || stream->property("alreadyStarted").toBool()) {
-            return;
-        }
-        stream->setProperty("alreadyStarted", true);
-        setResponsePending(true);
-
-        // No need to explicitly launch a search, setting the search term already does that for us
-        m_engine->setSearchTerm(searchText);
-        m_onePage = false;
-
-        connect(stream, &ResultsStream::fetchMore, this, &KNSBackend::fetchMore);
-        connect(this, &KNSBackend::receivedResources, stream, &ResultsStream::resourcesFound);
-        connect(this, &KNSBackend::searchFinished, stream, &ResultsStream::finish);
-        connect(this, &KNSBackend::startingSearch, stream, &ResultsStream::finish);
+        KNSCore::Provider::SearchRequest p(KNSCore::Provider::Newest, KNSCore::Provider::None, searchText, {}, 0, ENGINE_PAGE_SIZE);
+        stream->setRequest(p);
     };
-    if (m_responsePending) {
-        connect(this, &KNSBackend::availableForQueries, stream, start, Qt::QueuedConnection);
-    } else if (isFetching()) {
-        connect(this, &KNSBackend::initialized, stream, start);
-        connect(this, &KNSBackend::availableForQueries, stream, start, Qt::QueuedConnection);
+    if (isFetching()) {
+        connect(this, &KNSBackend::initialized, stream, start, Qt::QueuedConnection);
+        connect(this, &KNSBackend::fetchingChanged, stream, start, Qt::QueuedConnection);
     } else {
         QTimer::singleShot(0, stream, start);
     }
+    return stream;
 }
 
 ResultsStream *KNSBackend::findResourceByPackageName(const QUrl &search)
@@ -697,44 +521,16 @@ ResultsStream *KNSBackend::findResourceByPackageName(const QUrl &search)
     const auto providerid = pathParts.at(0);
     const auto entryid = pathParts.at(1);
 
-    auto stream = new ResultsStream(QLatin1String("KNS-byname-") + entryid);
-    auto start = [this, entryid, stream, providerid]() {
-        if (m_responsePending) {
-            // Slot already taken, will need to wait again
-            return;
-        }
-        setResponsePending(true);
-        m_engine->fetchEntryById(entryid);
-        m_onePage = true;
-
-        connect(m_engine, &KNSCore::Engine::signalErrorCode, stream, &ResultsStream::finish);
-        connect(m_engine,
-                &KNSCore::Engine::signalEntryEvent,
-                stream,
-                [this, stream, entryid, providerid](const KNSCore::EntryInternal &entry, KNSCore::EntryInternal::EntryEvent event) {
-                    switch (event) {
-                    case KNSCore::EntryInternal::StatusChangedEvent:
-                        if (entry.uniqueId() == entryid && providerid == QUrl(entry.providerId()).host()) {
-                            Q_EMIT stream->resourcesFound({resourceForEntry(entry)});
-                        } else
-                            qWarning() << "found invalid" << entryid << entry.uniqueId() << providerid << QUrl(entry.providerId()).host();
-                        QTimer::singleShot(0, this, [this] {
-                            setResponsePending(false);
-                        });
-                        stream->finish();
-                        break;
-                    case KNSCore::EntryInternal::DetailsLoadedEvent:
-                    case KNSCore::EntryInternal::AdoptedEvent:
-                    case KNSCore::EntryInternal::UnknownEvent:
-                    default:
-                        break;
-                    }
-                });
+    auto stream = new KNSResultsStream(this, QLatin1String("KNS-byname-") + entryid);
+    auto start = [entryid, stream, providerid]() {
+        KNSCore::Provider::SearchRequest query(KNSCore::Provider::Newest, KNSCore::Provider::ExactEntryId, entryid, {}, 0, ENGINE_PAGE_SIZE);
+        stream->setRequest(query);
     };
-    if (m_responsePending) {
-        connect(this, &KNSBackend::availableForQueries, stream, start);
+    if (isFetching()) {
+        connect(this, &KNSBackend::initialized, stream, start, Qt::QueuedConnection);
+        connect(this, &KNSBackend::fetchingChanged, stream, start, Qt::QueuedConnection);
     } else {
-        start();
+        QTimer::singleShot(0, stream, start);
     }
     return stream;
 }
@@ -754,7 +550,7 @@ QString KNSBackend::displayName() const
     return QStringLiteral("KNewStuff");
 }
 
-void KNSBackend::detailsLoaded(const KNSCore::EntryInternal &entry)
+void KNSBackend::detailsLoaded(const KNSCore::Entry &entry)
 {
     auto res = resourceForEntry(entry);
     Q_EMIT res->longDescriptionChanged();
