@@ -14,6 +14,7 @@
 #include "PackageKitSourcesBackend.h"
 #include "PackageKitUpdater.h"
 #include <AppStreamQt/release.h>
+#include <AppStreamQt/systeminfo.h>
 #include <AppStreamQt/utils.h>
 #include <AppStreamQt/version.h>
 #include <appstream/AppStreamIntegration.h>
@@ -37,16 +38,32 @@
 #include <PackageKit/Details>
 #include <PackageKit/Offline>
 
+#include <KConfig>
+#include <KConfigGroup>
 #include <KLocalizedString>
 #include <KProtocolManager>
 
+#include <optional>
+
 #include "config-paths.h"
 #include "libdiscover_backend_debug.h"
 #include "utils.h"
 #include <Category/Category.h>
+#include <resources/ResourcesModel.h>
 
 DISCOVER_BACKEND_PLUGIN(PackageKitBackend)
 
+QDebug operator<<(QDebug dbg, const PackageOrAppId &value)
+{
+    QDebugStateSaver saver(dbg);
+    dbg.nospace();
+    dbg << "PackageOrAppId(";
+    dbg << "id: " << value.id << ',';
+    dbg << "isPkg: " << value.isPackageName;
+    dbg << ')';
+    return dbg;
+}
+
 bool operator==(const PackageOrAppId &a, const PackageOrAppId &b)
 {
     return a.isPackageName == b.isPackageName && a.id == b.id;
@@ -67,6 +84,15 @@ PackageOrAppId makePackageId(const QString &id)
     return {id, true};
 }
 
+PackageOrAppId makeResourceId(PackageKitResource *res)
+{
+    auto appstreamResource = qobject_cast<AppPackageKitResource *>(res);
+    if (appstreamResource) {
+        return {appstreamResource->appstreamId(), false};
+    }
+    return {res->packageName(), true};
+}
+
 template<typename T, typename W>
 static void setWhenAvailable(const QDBusPendingReply<T> &pending, W func, QObject *parent)
 {
@@ -100,7 +126,7 @@ PackageKitBackend::PackageKitBackend(QObject *parent)
     , m_updater(new PackageKitUpdater(this))
     , m_refresher(nullptr)
     , m_isFetching(0)
-    , m_reviews(AppStreamIntegration::global()->reviews())
+    , m_reviews(OdrsReviewsBackend::global())
 {
     QTimer *t = new QTimer(this);
     connect(t, &QTimer::timeout, this, &PackageKitBackend::checkForUpdates);
@@ -146,7 +172,7 @@ PackageKitBackend::PackageKitBackend(QObject *parent)
                     }
                 });
         connect(t, &PackageKit::Transaction::errorCode, this, [this, pkgids](PackageKit::Transaction::Error err, const QString &error) {
-            qWarning() << "error fetching updates:" << err << error;
+            qWarning() << "PackageKitBackend: Error fetching updates:" << err << error;
             for (const QString &pkgid : pkgids) {
                 const QSet<AbstractResource *> resources = resourcesByPackageName(PackageKit::Daemon::packageName(pkgid));
                 for (auto r : resources) {
@@ -159,7 +185,9 @@ PackageKitBackend::PackageKitBackend(QObject *parent)
         });
     });
 
-    connect(PackageKit::Daemon::global(), &PackageKit::Daemon::restartScheduled, m_updater, &PackageKitUpdater::enableNeedsReboot);
+    connect(PackageKit::Daemon::global(), &PackageKit::Daemon::restartScheduled, this, [this] {
+        m_updater->setNeedsReboot(true);
+    });
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::isRunningChanged, this, &PackageKitBackend::checkDaemonRunning);
     connect(m_reviews.data(), &OdrsReviewsBackend::ratingsReady, this, [this] {
         m_reviews->emitRatingFetched(this, kTransform<QList<AbstractResource *>>(m_packages.packages, [](AbstractResource *r) {
@@ -184,14 +212,14 @@ PackageKitBackend::PackageKitBackend(QObject *parent)
         [this](uint timeSince) {
             if (timeSince > 3600)
                 checkForUpdates();
-            else
+            else if (!PackageKit::Daemon::global()->offline()->upgradeTriggered())
                 fetchUpdates();
             acquireFetching(false);
         },
         this);
 
-    PackageKit::Daemon::global()->setHints(QStringList() << QStringLiteral("interactive=true")
-                                                         << QStringLiteral("locale=%1").arg(qEnvironmentVariable("LANG")));
+    m_globalHints = QStringList() << QStringLiteral("interactive=true") << QStringLiteral("locale=%1").arg(qEnvironmentVariable("LANG"));
+    PackageKit::Daemon::global()->setHints(m_globalHints);
 }
 
 PackageKitBackend::~PackageKitBackend()
@@ -200,18 +228,46 @@ PackageKitBackend::~PackageKitBackend()
     m_threadPool.clear();
 }
 
+QString proxyFor(KConfigGroup *config, const QString &protocol)
+{
+    const QString key = protocol + QLatin1String("Proxy");
+    QString proxyStr(config->readEntry(key));
+    const int index = proxyStr.lastIndexOf(QLatin1Char(' '));
+
+    if (index > -1) {
+        const QStringView portStr = QStringView(proxyStr).right(proxyStr.length() - index - 1);
+        const bool isDigits = std::all_of(portStr.cbegin(), portStr.cend(), [](const QChar c) {
+            return c.isDigit();
+        });
+
+        if (isDigits) {
+            proxyStr = QStringView(proxyStr).left(index) + QLatin1Char(':') + portStr;
+        } else {
+            proxyStr.clear();
+        }
+    }
+
+    return proxyStr;
+}
+
 void PackageKitBackend::updateProxy()
 {
     if (PackageKit::Daemon::isRunning()) {
-        static bool everHad = KProtocolManager::useProxy();
-        if (!everHad && !KProtocolManager::useProxy())
+        KConfig kioSettings(QStringLiteral("kioslaverc"));
+        KConfigGroup proxyConfig = kioSettings.group("Proxy Settings");
+
+        bool useProxy = proxyConfig.readEntry<int>("ProxyType", 0) != 0;
+
+        static bool everHad = useProxy;
+        if (!everHad && !useProxy)
             return;
 
-        everHad = KProtocolManager::useProxy();
-        PackageKit::Daemon::global()->setProxy(KProtocolManager::proxyFor(QStringLiteral("http")),
-                                               KProtocolManager::proxyFor(QStringLiteral("https")),
-                                               KProtocolManager::proxyFor(QStringLiteral("ftp")),
-                                               KProtocolManager::proxyFor(QStringLiteral("socks")),
+        everHad = useProxy;
+
+        PackageKit::Daemon::global()->setProxy(proxyFor(&proxyConfig, QStringLiteral("http")),
+                                               proxyFor(&proxyConfig, QStringLiteral("https")),
+                                               proxyFor(&proxyConfig, QStringLiteral("ftp")),
+                                               proxyFor(&proxyConfig, QStringLiteral("socks")),
                                                {},
                                                {});
     }
@@ -237,84 +293,51 @@ void PackageKitBackend::acquireFetching(bool f)
     Q_ASSERT(m_isFetching >= 0);
 }
 
-struct DelayedAppStreamLoad {
-    QVector<AppStream::Component> components;
-    QHash<QString, AppStream::Component> missingComponents;
-    bool correct = true;
-};
-
-static DelayedAppStreamLoad loadAppStream(AppStream::Pool *appdata)
+static bool loadAppStream(AppStream::Pool *appdata)
 {
-    DelayedAppStreamLoad ret;
-
-    ret.correct = appdata->load();
-    if (!ret.correct) {
-        qWarning() << "Could not open the AppStream metadata pool" << appdata->lastError();
+    bool correct = appdata->load();
+    if (!correct) {
+        qWarning() << "PackageKitBackend: Could not open the AppStream metadata pool" << appdata->lastError();
     }
-
-    const auto components = appdata->components();
-    ret.components.reserve(components.size());
-    for (const AppStream::Component &component : components) {
-        if (component.kind() == AppStream::Component::KindFirmware)
-            continue;
-
-        const auto pkgNames = component.packageNames();
-        if (pkgNames.isEmpty()) {
-            const auto entries = component.launchable(AppStream::Launchable::KindDesktopId).entries();
-            if (component.kind() == AppStream::Component::KindDesktopApp && !entries.isEmpty()) {
-                const QString file = PackageKitBackend::locateService(entries.first());
-                if (!file.isEmpty()) {
-                    ret.missingComponents[file] = component;
-                }
-            }
-        } else {
-            ret.components << component;
-        }
-    }
-    return ret;
+    return correct;
 }
 
 void PackageKitBackend::reloadPackageList()
 {
     acquireFetching(true);
-    if (m_refresher) {
-        disconnect(m_refresher.data(), &PackageKit::Transaction::finished, this, &PackageKitBackend::reloadPackageList);
-    }
 
     m_appdata.reset(new AppStream::Pool);
 
-    auto fw = new QFutureWatcher<DelayedAppStreamLoad>(this);
-    connect(fw, &QFutureWatcher<DelayedAppStreamLoad>::finished, this, [this, fw]() {
-        const auto data = fw->result();
+    auto fw = new QFutureWatcher<bool>(this);
+    connect(fw, &QFutureWatcher<bool>::finished, this, [this, fw]() {
+        const auto correct = fw->result();
         fw->deleteLater();
 
-        if (!data.correct && m_packages.packages.isEmpty()) {
+        if (!correct && m_packages.packages.isEmpty()) {
             QTimer::singleShot(0, this, [this]() {
                 Q_EMIT passiveMessage(i18n("Please make sure that Appstream is properly set up on your system"));
             });
         }
-        for (const auto &component : data.components) {
-            addComponent(component);
-        }
 
-        if (data.components.isEmpty()) {
-            qCDebug(LIBDISCOVER_BACKEND_LOG) << "empty appstream db";
-            if (PackageKit::Daemon::backendName() == QLatin1String("aptcc") || PackageKit::Daemon::backendName().isEmpty()) {
-                checkForUpdates();
-            }
-        }
+        // TODO
+        // if (data.components.isEmpty()) {
+        //     qCDebug(LIBDISCOVER_BACKEND_LOG) << "empty appstream db";
+        //     if (PackageKit::Daemon::backendName() == QLatin1String("aptcc") || PackageKit::Daemon::backendName().isEmpty()) {
+        //         checkForUpdates();
+        //     }
+        // }
         if (!m_appstreamInitialized) {
             m_appstreamInitialized = true;
             Q_EMIT loadedAppStream();
         }
         acquireFetching(false);
 
-        const QList<AppStream::Component> distroComponents = m_appdata->componentsById(AppStream::Utils::currentDistroComponentId());
+        const auto distroComponents = m_appdata->componentsById(AppStream::SystemInfo::currentDistroComponentId());
         if (distroComponents.isEmpty()) {
-            qWarning() << "no component found for" << AppStream::Utils::currentDistroComponentId();
+            qWarning() << "PackageKitBackend: No distro component found for" << AppStream::SystemInfo::currentDistroComponentId();
         }
         for (const AppStream::Component &dc : distroComponents) {
-            const auto releases = dc.releases();
+            const auto releases = dc.releasesPlain().entries();
             for (const auto &r : releases) {
                 int cmp = AppStream::Utils::vercmpSimple(r.version(), AppStreamIntegration::global()->osRelease()->versionId());
                 if (cmp == 0) {
@@ -338,28 +361,25 @@ void PackageKitBackend::reloadPackageList()
     fw->setFuture(QtConcurrent::run(&m_threadPool, &loadAppStream, m_appdata.get()));
 }
 
-AppPackageKitResource *PackageKitBackend::addComponent(const AppStream::Component &component)
+AppPackageKitResource *PackageKitBackend::addComponent(const AppStream::Component &component) const
 {
-    Q_ASSERT(isFetching());
     const QStringList pkgNames = component.packageNames();
     Q_ASSERT(!pkgNames.isEmpty());
+    Q_ASSERT(component.kind() != AppStream::Component::KindFirmware);
 
-    auto &resPos = m_packages.packages[makeAppId(component.id())];
-    AppPackageKitResource *res = qobject_cast<AppPackageKitResource *>(resPos);
+    auto appId = makeAppId(component.id());
+    AppPackageKitResource *res = qobject_cast<AppPackageKitResource *>(m_packages.packages.value(appId));
     if (!res) {
-        res = new AppPackageKitResource(component, pkgNames.at(0), this);
-        resPos = res;
-    } else {
-        res->clearPackageIds();
+        res = qobject_cast<AppPackageKitResource *>(m_packagesToAdd.value(appId));
+    }
+    if (!res) {
+        res = new AppPackageKitResource(component, pkgNames.at(0), const_cast<PackageKitBackend *>(this));
+        m_packagesToAdd.insert(appId, res);
+
     }
     for (const QString &pkg : pkgNames) {
         m_packages.packageToApp[pkg] += component.id();
     }
-
-    const auto componentExtends = component.extends();
-    for (const QString &pkg : componentExtends) {
-        m_packages.extendedBy[pkg] += res;
-    }
     return res;
 }
 
@@ -386,11 +406,10 @@ void PackageKitBackend::fetchUpdates()
     if (m_updater->isProgressing())
         return;
 
-    m_getUpdatesTransaction = PackageKit::Daemon::getUpdates();
-    connect(m_getUpdatesTransaction, &PackageKit::Transaction::finished, this, &PackageKitBackend::getUpdatesFinished);
-    connect(m_getUpdatesTransaction, &PackageKit::Transaction::package, this, &PackageKitBackend::addPackageToUpdate);
-    connect(m_getUpdatesTransaction, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
-    connect(m_getUpdatesTransaction, &PackageKit::Transaction::percentageChanged, this, &PackageKitBackend::fetchingUpdatesProgressChanged);
+    PackageKit::Transaction *tUpdates = PackageKit::Daemon::getUpdates();
+    connect(tUpdates, &PackageKit::Transaction::finished, this, &PackageKitBackend::getUpdatesFinished);
+    connect(tUpdates, &PackageKit::Transaction::package, this, &PackageKitBackend::addPackageToUpdate);
+    connect(tUpdates, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
     m_updatesPackageId.clear();
     m_hasSecurityUpdates = false;
 
@@ -422,9 +441,9 @@ void PackageKitBackend::addPackage(PackageKit::Transaction::Info info, const QSt
     if (r.isEmpty()) {
         auto pk = new PackageKitResource(packageName, summary, this);
         r = {pk};
-        m_packagesToAdd.insert(pk);
+        m_packagesToAdd.insert(makePackageId(packageName), pk);
     }
-    for (auto res : qAsConst(r))
+    for (auto res : std::as_const(r))
         static_cast<PackageKitResource *>(res)->addPackageId(info, packageId, arch);
 }
 
@@ -439,33 +458,27 @@ void PackageKitBackend::includePackagesToAdd()
         return;
 
     acquireFetching(true);
-    for (PackageKitResource *res : qAsConst(m_packagesToAdd)) {
-        m_packages.packages[makePackageId(res->packageName())] = res;
+    for (auto [id, res] : KeyValueRange(std::as_const(m_packagesToAdd))) {
+        m_packages.packages[id] = res;
     }
-    for (PackageKitResource *res : qAsConst(m_packagesToDelete)) {
+    m_packagesToAdd.clear();
+    for (PackageKitResource *res : std::as_const(m_packagesToDelete)) {
         const auto pkgs = m_packages.packageToApp.value(res->packageName(), {res->packageName()});
         for (const auto &pkg : pkgs) {
             auto res = m_packages.packages.take(makePackageId(pkg));
             if (res) {
-                if (AppPackageKitResource *ares = qobject_cast<AppPackageKitResource *>(res)) {
-                    const auto extends = res->extends();
-                    for (const auto &ext : extends)
-                        m_packages.extendedBy[ext].removeAll(ares);
-                }
-
                 Q_EMIT resourceRemoved(res);
                 res->deleteLater();
             }
         }
     }
-    m_packagesToAdd.clear();
     m_packagesToDelete.clear();
     acquireFetching(false);
 }
 
 void PackageKitBackend::transactionError(PackageKit::Transaction::Error, const QString &message)
 {
-    qWarning() << "Transaction error: " << message << sender();
+    qWarning() << "Transaction error:" << message << sender();
     Q_EMIT passiveMessage(message);
 }
 
@@ -473,7 +486,7 @@ void PackageKitBackend::packageDetails(const PackageKit::Details &details)
 {
     const QSet<AbstractResource *> resources = resourcesByPackageName(PackageKit::Daemon::packageName(details.packageId()));
     if (resources.isEmpty())
-        qWarning() << "couldn't find package for" << details.packageId();
+        qWarning() << "PackageKitBackend: Couldn't find package for" << details.packageId();
 
     for (AbstractResource *res : resources) {
         qobject_cast<PackageKitResource *>(res)->setDetails(details);
@@ -507,15 +520,25 @@ T PackageKitBackend::resourcesByPackageNames(const W &pkgnames) const
     for (const QString &pkg_name : pkgnames) {
         const QStringList app_names = m_packages.packageToApp.value(pkg_name, QStringList());
         if (app_names.isEmpty()) {
-            AbstractResource *res = m_packages.packages.value(makePackageId(pkg_name));
+            auto pkgId = makePackageId(pkg_name);
+            PackageKitResource *res = qobject_cast<PackageKitResource *>(m_packages.packages.value(pkgId));
+            if (!res) {
+                res = m_packagesToAdd.value(pkgId);
+            }
             if (res) {
                 ret += res;
             }
         } else {
             for (const QString &app_id : app_names) {
-                AbstractResource *res = m_packages.packages.value(makeAppId(app_id));
+                const auto appId = makeAppId(app_id);
+                AbstractResource *res = m_packages.packages.value(appId);
+                if (!res) {
+                    res = m_packagesToAdd.value(appId);
+                }
                 if (res) {
                     ret += res;
+                } else {
+                    ret += resourcesByComponents<T>(m_appdata->componentsByBundleId(AppStream::Bundle::KindPackage, pkg_name, false));
                 }
             }
         }
@@ -525,27 +548,33 @@ T PackageKitBackend::resourcesByPackageNames(const W &pkgnames) const
 
 void PackageKitBackend::checkForUpdates()
 {
-    if (PackageKit::Daemon::global()->offline()->updateTriggered()) {
+    if (auto offline = PackageKit::Daemon::global()->offline(); offline->updateTriggered() || offline->upgradeTriggered()) {
         qCDebug(LIBDISCOVER_BACKEND_LOG) << "Won't be checking for updates again, the system needs a reboot to apply the fetched offline updates.";
         return;
     }
 
     if (!m_refresher) {
         acquireFetching(true);
+        m_updater->clearDistroUpgrade();
         m_refresher = PackageKit::Daemon::refreshCache(false);
+        // Limit the cache-age so that we actually download new caches if necessary
+        m_refresher->setHints(globalHints() << QStringLiteral("cache-age=300" /* 5 minutes */));
 
         connect(m_refresher.data(), &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
+        connect(m_refresher.data(), &PackageKit::Transaction::percentageChanged, this, &PackageKitBackend::fetchingUpdatesProgressChanged);
         connect(m_refresher.data(), &PackageKit::Transaction::finished, this, [this]() {
             m_refresher = nullptr;
             fetchUpdates();
             acquireFetching(false);
         });
     } else {
-        qWarning() << "already resetting";
+        qWarning() << "PackageKitBackend: Already resetting";
     }
+
+    Q_EMIT fetchingUpdatesProgressChanged();
 }
 
-QList<AppStream::Component> PackageKitBackend::componentsById(const QString &id) const
+AppStream::ComponentBox PackageKitBackend::componentsById(const QString &id) const
 {
     Q_ASSERT(m_appstreamInitialized);
     auto comps = m_appdata->componentsById(id);
@@ -555,8 +584,8 @@ QList<AppStream::Component> PackageKitBackend::componentsById(const QString &id)
     return comps;
 }
 
-static const auto needsResolveFilter = [](AbstractResource *res) {
-    return res->state() == AbstractResource::Broken;
+static const auto needsResolveFilter = [](const StreamResult &res) {
+    return res.resource->state() == AbstractResource::Broken;
 };
 
 class PKResultsStream : public ResultsStream
@@ -568,7 +597,7 @@ private:
     {
     }
 
-    PKResultsStream(PackageKitBackend *backend, const QString &name, const QVector<AbstractResource *> &resources)
+    PKResultsStream(PackageKitBackend *backend, const QString &name, const QVector<StreamResult> &resources)
         : ResultsStream(name)
         , backend(backend)
     {
@@ -584,7 +613,7 @@ public:
         return new PKResultsStream(std::forward<Args>(args)...);
     }
 
-    void sendResources(const QVector<AbstractResource *> &res, bool waitForResolved = false)
+    void sendResources(const QVector<StreamResult> &res, bool waitForResolved = false)
     {
         if (res.isEmpty()) {
             finish();
@@ -592,10 +621,10 @@ public:
         }
 
         Q_ASSERT(res.size() == QSet(res.constBegin(), res.constEnd()).size());
-        const auto toResolve = kFilter<QVector<AbstractResource *>>(res, needsResolveFilter);
+        const auto toResolve = kFilter<QVector<StreamResult>>(res, needsResolveFilter);
         if (!toResolve.isEmpty()) {
-            auto transaction = backend->resolvePackages(kTransform<QStringList>(toResolve, [](AbstractResource *res) {
-                return res->packageName();
+            auto transaction = backend->resolvePackages(kTransform<QStringList>(toResolve, [](const StreamResult &res) {
+                return res.resource->packageName();
             }));
             if (waitForResolved) {
                 Q_ASSERT(transaction);
@@ -625,16 +654,15 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
             if (!stream) {
                 return;
             }
-            const auto resources = kTransform<QVector<AbstractResource *>>(m_packages.extendedBy.value(filter.extends), [](AppPackageKitResource *a) {
-                return a;
-            });
+            const auto extendingComponents = m_appdata->componentsByExtends(filter.extends);
+            auto resources = resultsByComponents(extendingComponents);
             stream->sendResources(resources, filter.state != AbstractResource::Broken);
         };
         runWhenInitialized(f, stream);
         return stream;
     } else if (filter.state == AbstractResource::Upgradeable) {
         return new ResultsStream(QStringLiteral("PackageKitStream-upgradeable"),
-                                 kTransform<QVector<AbstractResource *>>(upgradeablePackages())); // No need for it to be a PKResultsStream
+                                 kTransform<QVector<StreamResult>>(upgradeablePackages())); // No need for it to be a PKResultsStream
     } else if (filter.state == AbstractResource::Installed) {
         auto stream = PKResultsStream::create(this, QStringLiteral("PackageKitStream-installed"));
         auto f = [this, stream, filter] {
@@ -655,7 +683,9 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
                 connect(m_resolveTransaction, &PKResolveTransaction::allFinished, this, [stream, toResolve, installedAndNameFilter] {
                     const auto resolved = kFilter<QVector<AbstractResource *>>(toResolve, installedAndNameFilter);
                     if (!resolved.isEmpty())
-                        Q_EMIT stream->resourcesFound(resolved);
+                        Q_EMIT stream->resourcesFound(kTransform<QVector<StreamResult>>(resolved, [](auto resource) {
+                            return StreamResult{resource, 0};
+                        }));
                     stream->finish();
                 });
                 furtherSearch = true;
@@ -665,7 +695,9 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
             if (!resolved.isEmpty()) {
                 QTimer::singleShot(0, this, [resolved, toResolve, stream]() {
                     if (!resolved.isEmpty())
-                        Q_EMIT stream->resourcesFound(resolved);
+                        Q_EMIT stream->resourcesFound(kTransform<QVector<StreamResult>>(resolved, [](auto resource) {
+                            return StreamResult{resource, 0};
+                        }));
 
                     if (toResolve.isEmpty())
                         stream->finish();
@@ -688,7 +720,9 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
                 return res->type() != AbstractResource::Technical && !qobject_cast<PackageKitResource *>(res)->isCritical()
                     && !qobject_cast<PackageKitResource *>(res)->extendsItself();
             });
-            stream->sendResources(resources);
+            stream->sendResources(kTransform<QVector<StreamResult>>(resources, [](auto resource) {
+                return StreamResult{resource, 0};
+            }));
         };
         runWhenInitialized(f, stream);
         return stream;
@@ -698,19 +732,25 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
             if (!stream) {
                 return;
             }
-            const auto components = !filter.search.isEmpty() ? m_appdata->search(filter.search)
-#if ASQ_CHECK_VERSION(0, 15, 6)
-                                  : filter.category          ? AppStreamUtils::componentsByCategories(m_appdata.get(),
-                                                                                                      filter.category,
-                                                                                                      AppStream::Bundle::KindUnknown)
-#endif
-                                                             : m_appdata->components();
-            const QSet<QString> ids = kTransform<QSet<QString>>(components, [](const AppStream::Component &comp) {
-                return comp.id();
+            AppStream::ComponentBox components(AppStream::ComponentBox::FlagNone);
+            if (!filter.search.isEmpty()) {
+                components = m_appdata->search(filter.search);
+            } else if (filter.category) {
+                components = AppStreamUtils::componentsByCategories(m_appdata.get(), filter.category, AppStream::Bundle::KindUnknown);
+            } else {
+                components = m_appdata->components();
+            }
+
+            QSet<QString> ids;
+            kFilterInPlace<AppStream::ComponentBox>(components, [&ids](const AppStream::Component &comp) {
+                if (ids.contains(comp.id()))
+                    return false;
+                ids.insert(comp.id());
+                return true;
             });
             if (!ids.isEmpty()) {
-                const auto resources = kFilter<QVector<AbstractResource *>>(resourcesByAppNames<QVector<AbstractResource *>>(ids), [](AbstractResource *res) {
-                    return !qobject_cast<PackageKitResource *>(res)->extendsItself();
+                const auto resources = kFilter<QVector<StreamResult>>(resultsByComponents(components), [](const StreamResult &res) {
+                    return !qobject_cast<PackageKitResource *>(res.resource)->extendsItself();
                 });
                 stream->sendResources(resources, filter.state != AbstractResource::Broken);
             } else {
@@ -741,7 +781,7 @@ PKResultsStream *PackageKitBackend::findResourceByPackageName(const QUrl &url)
             || mime.inherits(QStringLiteral("application/x-tar")) //
             || mime.inherits(QStringLiteral("application/x-zstd-compressed-tar")) //
             || mime.inherits(QStringLiteral("application/x-xz-compressed-tar"))) {
-            return PKResultsStream::create(this, QStringLiteral("PackageKitStream-localpkg"), QVector<AbstractResource *>{new LocalFilePKResource(url, this)}).data();
+            return PKResultsStream::create(this, QStringLiteral("PackageKitStream-localpkg"), QVector<StreamResult>{new LocalFilePKResource(url, this)}).data();
         }
     } else if (url.scheme() == QLatin1String("appstream")) {
         const auto appstreamIds = AppStreamUtils::appstreamIds(url);
@@ -753,29 +793,29 @@ PKResultsStream *PackageKitBackend::findResourceByPackageName(const QUrl &url)
                 if (!stream) {
                     return;
                 }
-                auto toSend = QSet<AbstractResource *>();
+                auto toSend = QSet<StreamResult>();
                 toSend.reserve(appstreamIds.size());
                 for (const auto &appstreamId : appstreamIds) {
                     const auto comps = componentsById(appstreamId);
                     if (comps.isEmpty()) {
                         continue;
                     }
-                    auto resources = resourcesByComponents<QVector<AbstractResource *>>(comps);
+                    const auto resources = resultsByComponents(comps);
                     for (const auto &r : resources) {
                         toSend.insert(r);
                     }
                 }
-                stream->sendResources(QVector(toSend.constBegin(), toSend.constEnd()));
+                stream->sendResources(QList(toSend.constBegin(), toSend.constEnd()));
             };
             runWhenInitialized(f, stream);
             return stream;
         }
     }
-    return PKResultsStream::create(this, QStringLiteral("PackageKitStream-unknown-url"), QVector<AbstractResource *>{}).data();
+    return PKResultsStream::create(this, QStringLiteral("PackageKitStream-unknown-url"), QVector<StreamResult>{}).data();
 }
 
 template<typename T>
-T PackageKitBackend::resourcesByComponents(const QList<AppStream::Component> &comps) const
+T PackageKitBackend::resourcesByComponents(const AppStream::ComponentBox &comps) const
 {
     T ret;
     ret.reserve(comps.size());
@@ -785,8 +825,26 @@ T PackageKitBackend::resourcesByComponents(const QList<AppStream::Component> &co
             continue;
         }
         done += comp.id();
-        ret << m_packages.packages.value(makeAppId(comp.id()));
-        Q_ASSERT(ret.constLast());
+        auto r = addComponent(comp);
+        Q_ASSERT(r);
+        ret << r;
+    }
+    return ret;
+}
+
+QVector<StreamResult> PackageKitBackend::resultsByComponents(const AppStream::ComponentBox &comps) const
+{
+    QVector<StreamResult> ret;
+    ret.reserve(comps.size());
+    QSet<QString> done;
+    for (const auto &comp : comps) {
+        if (comp.packageNames().isEmpty() ||  comp.kind() == AppStream::Component::KindFirmware || done.contains(comp.id())) {
+            continue;
+        }
+        done += comp.id();
+        auto r = addComponent(comp);
+        Q_ASSERT(r);
+        ret << StreamResult{r, comp.sortScore()};
     }
     return ret;
 }
@@ -798,7 +856,7 @@ bool PackageKitBackend::hasSecurityUpdates() const
 
 int PackageKitBackend::updatesCount() const
 {
-    if (PackageKit::Daemon::global()->offline()->updateTriggered())
+    if (auto offline = PackageKit::Daemon::global()->offline(); offline->updateTriggered() || offline->upgradeTriggered())
         return 0;
 
     int ret = 0;
@@ -857,11 +915,11 @@ QSet<AbstractResource *> PackageKitBackend::upgradeablePackages() const
 
     QSet<AbstractResource *> ret;
     ret.reserve(m_updatesPackageId.size());
-    for (const QString &pkgid : qAsConst(m_updatesPackageId)) {
+    for (const QString &pkgid : std::as_const(m_updatesPackageId)) {
         const QString pkgname = PackageKit::Daemon::packageName(pkgid);
         const auto pkgs = resourcesByPackageName(pkgname);
         if (pkgs.isEmpty()) {
-            qWarning() << "couldn't find resource for" << pkgid;
+            qWarning() << "PackageKitBackend: Couldn't find resource for" << pkgid;
         }
         ret.unite(pkgs);
     }
@@ -908,6 +966,66 @@ void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
         connect(this, &PackageKitBackend::available, a, &OneTimeAction::trigger);
     } else
         Q_EMIT updatesCountChanged();
+
+    if (!m_updater->isDistroUpgrade() && !PackageKit::Daemon::global()->offline()->upgradeTriggered()) {
+        auto nextRelease = AppStreamIntegration::global()->getDistroUpgrade(m_appdata.get());
+        if (nextRelease)
+            foundNewMajorVersion(*nextRelease);
+    }
+}
+
+void PackageKitBackend::foundNewMajorVersion(const AppStream::Release &release)
+{
+    QString info;
+    // Message to display when:
+    // - A new major version is available
+    // - An update to the current version is available or pending a reboot
+    info = i18n(
+        "<b>A new major version of %1 has been released.</b>\n"
+        "To be able to upgrade to this new version, make sure to apply all available updates and then restart the system.",
+        AppStreamIntegration::global()->osRelease()->name());
+    QSharedPointer<InlineMessage> updateBeforeMajorUpgradeMessage =
+        QSharedPointer<InlineMessage>::create(InlineMessage::Positive, QStringLiteral("application-x-rpm"), info);
+
+    // Message to display when:
+    // - A new major version is available
+    // - No update to the current version are available or pending a reboot
+    DiscoverAction *majorUpgrade = new DiscoverAction(i18n("Upgrade to %1 %2", AppStreamIntegration::global()->osRelease()->name(), release.version()), this);
+    connect(majorUpgrade, &DiscoverAction::triggered, this, [this, release] {
+        if (m_updater->isProgressing())
+            return;
+
+        const QString& upgradeVersion = release.version();
+        m_updatesPackageId.clear();
+        m_updater->setProgressing(true);
+        m_refresher = PackageKit::Daemon::upgradeSystem(upgradeVersion,
+                                                        PackageKit::Transaction::UpgradeKind::UpgradeKindComplete,
+                                                        PackageKit::Transaction::TransactionFlagSimulate);
+        m_refresher->setHints(globalHints() << QStringLiteral("cache-age=86400" /* 24*60*60 */));
+        connect(m_refresher, &PackageKit::Transaction::package, this, &PackageKitBackend::addPackageToUpdate);
+        connect(m_refresher, &PackageKit::Transaction::percentageChanged, this, &PackageKitBackend::fetchingUpdatesProgressChanged);
+        connect(m_refresher, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
+        connect(m_refresher, &PackageKit::Transaction::finished, this, [this, release](PackageKit::Transaction::Exit e, uint x) {
+            m_updater->setDistroUpgrade(release);
+            getUpdatesFinished(e, x);
+        });
+        Q_EMIT inlineMessageChanged({});
+        ResourcesModel::global()->switchToUpdates();
+    });
+
+    info = i18n("A new major version has been released");
+    QSharedPointer<InlineMessage> majorUpgradeAvailableMessage =
+        QSharedPointer<InlineMessage>::create(InlineMessage::Positive, QStringLiteral("application-x-rpm"), info, majorUpgrade);
+
+    // Allow upgrade only if up to date on the current release
+    if (!m_updatesPackageId.isEmpty()) {
+        Q_EMIT inlineMessageChanged(updateBeforeMajorUpgradeMessage);
+        return;
+    }
+
+    // No updates pending or avaiable. We are good to offer the upgrade to the
+    // next major version!
+    Q_EMIT inlineMessageChanged(majorUpgradeAvailableMessage);
 }
 
 // Copy of Transaction::packageName that doesn't create a copy but just pass a reference
@@ -968,9 +1086,10 @@ AbstractBackendUpdater *PackageKitBackend::backendUpdater() const
     return m_updater;
 }
 
-QVector<AppPackageKitResource *> PackageKitBackend::extendedBy(const QString &id) const
+QVector<AbstractResource *> PackageKitBackend::extendedBy(const QString &id) const
 {
-    return m_packages.extendedBy[id];
+    const auto components = m_appdata->componentsById(id);
+    return resourcesByComponents<QVector<AbstractResource *>>(components);
 }
 
 AbstractReviewsBackend *PackageKitBackend::reviewsBackend() const
@@ -985,18 +1104,22 @@ QString PackageKitBackend::displayName() const
 
 int PackageKitBackend::fetchingUpdatesProgress() const
 {
-    if (!m_getUpdatesTransaction)
-        return 0;
+    if (!m_refresher)
+        return 100;
 
-    if (m_getUpdatesTransaction->status() == PackageKit::Transaction::StatusWait
-        || m_getUpdatesTransaction->status() == PackageKit::Transaction::StatusUnknown) {
-        return m_getUpdatesTransaction->property("lastPercentage").toInt();
+    int percentage = m_refresher->percentage();
+    if (percentage > 100) {
+        return m_refresher->property("lastPercentage").toInt();
     }
-    int percentage = percentageWithStatus(m_getUpdatesTransaction->status(), m_getUpdatesTransaction->percentage());
-    m_getUpdatesTransaction->setProperty("lastPercentage", percentage);
+    m_refresher->setProperty("lastPercentage", percentage);
     return percentage;
 }
 
+uint PackageKitBackend::fetchingUpdatesProgressWeight() const
+{
+    return 50;
+}
+
 InlineMessage *PackageKitBackend::explainDysfunction() const
 {
     const auto error = m_appdata->lastError();
