@@ -23,6 +23,10 @@
 #include <QTextStream>
 #include <QTimer>
 
+#include <AppStreamQt/pool.h>
+#include <AppStreamQt/release.h>
+#include <appstream/AppStreamIntegration.h>
+
 #include "libdiscover_backend_debug.h"
 #include "pk-offline-private.h"
 
@@ -32,15 +36,19 @@ PackageKitNotifier::PackageKitNotifier(QObject *parent)
     : BackendNotifierModule(parent)
     , m_securityUpdates(0)
     , m_normalUpdates(0)
+    , m_hasDistUpgrade(false)
+    , m_appdata(new AppStream::Pool)
 {
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::updatesChanged, this, &PackageKitNotifier::recheckSystemUpdateNeeded);
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::transactionListChanged, this, &PackageKitNotifier::transactionListChanged);
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::restartScheduled, this, &PackageKitNotifier::nowNeedsReboot);
     connect(PackageKit::Daemon::global()->offline(), &PackageKit::Offline::changed, this, [this] {
-        if (PackageKit::Daemon::global()->offline()->updateTriggered())
+        if (auto offline = PackageKit::Daemon::global()->offline(); offline->updateTriggered() || offline->upgradeTriggered())
             nowNeedsReboot();
     });
 
+    m_appdata->load();
+
     // Check if there's packages after 5'
     QTimer::singleShot(5min, this, &PackageKitNotifier::refreshDatabase);
 
@@ -100,7 +108,7 @@ void PackageKitNotifier::checkOfflineUpdates()
 
     const bool success = group.readEntry("Success", false);
     const QString packagesJoined = group.readEntry("Packages");
-    const auto packages = packagesJoined.splitRef(QLatin1Char(','));
+    const auto packages = QStringView(packagesJoined).split(QLatin1Char(','));
     const bool isMobile = QByteArrayList{"1", "true"}.contains(qgetenv("QT_QUICK_CONTROLS_MOBILE"));
     const QString errorCode = group.readEntry("ErrorCode");
     static QSet<QString> allowedAlreadyInstalled = {
@@ -114,19 +122,20 @@ void PackageKitNotifier::checkOfflineUpdates()
         notification->setIconName(QStringLiteral("dialog-error"));
         notification->setTitle(i18n("Failed Offline Update"));
         notification->setText(i18np("Failed to update %1 package\n%2", "Failed to update %1 packages\n%2", packages.count(), errorDetails));
-        notification->setActions(QStringList{i18nc("@action:button", "Open Discover"), i18nc("@action:button", "Repair System")});
         notification->setComponentName(QStringLiteral("discoverabstractnotifier"));
 
-        connect(notification, &KNotification::action1Activated, this, []() {
+        auto openDiscoverAction = notification->addAction(i18nc("@action:button", "Open Discover"));
+        connect(openDiscoverAction, &KNotificationAction::activated, this, [] {
             QProcess::startDetached(QStringLiteral("plasma-discover"), QStringList());
         });
-        connect(notification, &KNotification::action2Activated, this, [this]() {
+
+        auto repairAction = notification->addAction(i18nc("@action:button", "Repair System"));
+        connect(repairAction, &KNotificationAction::activated, this, [this] {
             qInfo() << "Repairing system";
             auto trans = PackageKit::Daemon::global()->repairSystem();
             KNotification::event(QStringLiteral("OfflineUpdateRepairStarted"),
                                  i18n("Repairing failed offline update"),
                                  {},
-                                 {},
                                  KNotification::CloseOnTimeout,
                                  QStringLiteral("discoverabstractnotifier"));
 
@@ -134,7 +143,6 @@ void PackageKitNotifier::checkOfflineUpdates()
                 KNotification::event(QStringLiteral("OfflineUpdateRepairFailed"),
                                      i18n("Repair Failed"),
                                      xi18nc("@info", "%1<nl/>Please report this error to your distribution.", details),
-                                     {},
                                      KNotification::Persistent,
                                      QStringLiteral("discoverabstractnotifier"));
             });
@@ -146,7 +154,6 @@ void PackageKitNotifier::checkOfflineUpdates()
                     KNotification::event(QStringLiteral("OfflineUpdateRepairSuccessful"),
                                          i18n("Repaired Successfully"),
                                          {},
-                                         {},
                                          KNotification::CloseOnTimeout,
                                          QStringLiteral("discoverabstractnotifier"));
                 }
@@ -155,7 +162,7 @@ void PackageKitNotifier::checkOfflineUpdates()
             // No matter what happened, clean up the results file if it still exists
             // because at this point, there's nothing anyone can do with it
             if (QFile::exists(QStringLiteral(PK_OFFLINE_RESULTS_FILENAME))) {
-                qDebug() << "Removed offline results file";
+                qCDebug(LIBDISCOVER_BACKEND_LOG) << "Removed offline results file";
                 PackageKit::Daemon::global()->offline()->clearResults();
             }
         });
@@ -169,10 +176,10 @@ void PackageKitNotifier::checkOfflineUpdates()
             notification->setIconName(QStringLiteral("system-software-update"));
             notification->setTitle(i18n("Offline Updates"));
             notification->setText(i18np("Successfully updated %1 package", "Successfully updated %1 packages", packages.count()));
-            notification->setActions(QStringList{i18nc("@action:button", "Open Discover")});
             notification->setComponentName(QStringLiteral("discoverabstractnotifier"));
 
-            connect(notification, &KNotification::action1Activated, this, []() {
+            auto openDiscoverAction = notification->addAction(i18nc("@action:button", "Open Discover"));
+            connect(openDiscoverAction, &KNotificationAction::activated, this, []() {
                 QProcess::startDetached(QStringLiteral("plasma-discover"), QStringList());
             });
 
@@ -192,7 +199,7 @@ void PackageKitNotifier::recheckSystemUpdateNeeded()
         first = false;
     }
 
-    if (PackageKit::Daemon::global()->offline()->updateTriggered())
+    if (auto offline = PackageKit::Daemon::global()->offline(); offline->updateTriggered() || offline->upgradeTriggered())
         return;
 
     m_recheckTimer->start();
@@ -200,7 +207,7 @@ void PackageKitNotifier::recheckSystemUpdateNeeded()
 
 void PackageKitNotifier::recheckSystemUpdate()
 {
-    if (PackageKit::Daemon::global()->isRunning()) {
+    if (PackageKit::Daemon::global()->isRunning() && !PackageKit::Daemon::global()->offline()->upgradeTriggered()) {
         PackageKit::Daemon::getUpdates();
     }
 }
@@ -249,7 +256,7 @@ void PackageKitNotifier::finished(PackageKit::Transaction::Exit /*exit*/, uint)
 
 bool PackageKitNotifier::hasUpdates()
 {
-    return m_normalUpdates > 0;
+    return m_normalUpdates > 0 || m_hasDistUpgrade;
 }
 
 bool PackageKitNotifier::hasSecurityUpdates()
@@ -257,19 +264,24 @@ bool PackageKitNotifier::hasSecurityUpdates()
     return m_securityUpdates > 0;
 }
 
-void PackageKitNotifier::onDistroUpgrade(PackageKit::Transaction::DistroUpgrade /*type*/, const QString &name, const QString &description)
+void PackageKitNotifier::checkDistroUpgrade()
 {
-    auto a = new UpgradeAction(name, description, this);
-    connect(a, &UpgradeAction::triggered, this, [](const QString &name) {
-        PackageKit::Daemon::upgradeSystem(name, PackageKit::Transaction::UpgradeKindDefault);
-    });
-    Q_EMIT foundUpgradeAction(a);
+    auto nextRelease = AppStreamIntegration::global()->getDistroUpgrade(m_appdata.get());
+    if (nextRelease) {
+        m_hasDistUpgrade = true;
+        const QString &fullName = QStringLiteral("%1 %2").arg(AppStreamIntegration::global()->osRelease()->name(), nextRelease->version());
+        auto a = new UpgradeAction(nextRelease->version(), fullName, this);
+        connect(a, &UpgradeAction::triggered, this, [a](const QString &) {
+            Q_EMIT a->showDiscoverUpdates();
+        });
+        Q_EMIT foundUpgradeAction(a);
+    }
 }
 
 void PackageKitNotifier::refreshDatabase()
 {
     if (auto offline = PackageKit::Daemon::global()->offline();
-            offline->updatePrepared() || offline->upgradePrepared() || offline->updateTriggered() || offline->upgradeTriggered()) {
+        offline->updatePrepared() || offline->upgradePrepared() || offline->updateTriggered() || offline->upgradeTriggered()) {
         return;
     }
 
@@ -282,12 +294,13 @@ void PackageKitNotifier::refreshDatabase()
 
     if (!m_refresher) {
         m_refresher = PackageKit::Daemon::refreshCache(false);
+        // Limit the cache-age so that we actually download new caches if necessary
+        m_refresher->setHints(QStringLiteral("cache-age=300" /* 5 minutes */));
         connect(m_refresher.data(), &PackageKit::Transaction::finished, this, &PackageKitNotifier::recheckSystemUpdateNeeded);
     }
 
-    if (!m_distUpgrades && (PackageKit::Daemon::roles() & PackageKit::Transaction::RoleUpgradeSystem)) {
-        m_distUpgrades = PackageKit::Daemon::getDistroUpgrades();
-        connect(m_distUpgrades, &PackageKit::Transaction::distroUpgrade, this, &PackageKitNotifier::onDistroUpgrade);
+    if (PackageKit::Daemon::roles() & PackageKit::Transaction::RoleUpgradeSystem) {
+        checkDistroUpgrade();
     }
 }
 
@@ -295,7 +308,7 @@ QProcess *PackageKitNotifier::checkAptVariable(const QString &aptconfig, const Q
 {
     QProcess *process = new QProcess;
     process->start(aptconfig, {QStringLiteral("dump")});
-    connect(process, qOverload<int, QProcess::ExitStatus>(&QProcess::finished), this, [func, process, varname](int code) {
+    connect(process, &QProcess::finished, this, [func, process, varname](int code) {
         if (code != 0)
             return;
 
@@ -311,13 +324,13 @@ QProcess *PackageKitNotifier::checkAptVariable(const QString &aptconfig, const Q
         }
         func({});
     });
-    connect(process, qOverload<int, QProcess::ExitStatus>(&QProcess::finished), process, &QObject::deleteLater);
+    connect(process, &QProcess::finished, process, &QObject::deleteLater);
     return process;
 }
 
 void PackageKitNotifier::transactionListChanged(const QStringList &tids)
 {
-    if (PackageKit::Daemon::global()->offline()->updateTriggered())
+    if (auto offline = PackageKit::Daemon::global()->offline(); offline->updateTriggered() || offline->upgradeTriggered())
         return;
 
     for (const auto &tid : tids) {
