@@ -5,10 +5,15 @@
  */
 #include "PackageKitUpdater.h"
 #include "PackageKitMessages.h"
+#include <AppStreamQt/release.h>
+#include <appstream/AppStreamIntegration.h>
 
 #include <PackageKit/Daemon>
 #include <PackageKit/Offline>
 #include <QCryptographicHash>
+#include <QDBusConnection>
+#include <QDBusMessage>
+#include <QDBusReply>
 #include <QDebug>
 #include <QSet>
 
@@ -18,7 +23,10 @@
 #include <KLocalizedString>
 #include <KSharedConfig>
 
+#include <optional>
+
 #include "libdiscover_backend_debug.h"
+#include "pk-offline-private.h"
 #include "utils.h"
 
 int percentageWithStatus(PackageKit::Transaction::Status status, uint percentage)
@@ -72,7 +80,7 @@ public:
 
         m_updateSizeTimer->setInterval(100);
         m_updateSizeTimer->setSingleShot(true);
-        connect(m_updateSizeTimer, &QTimer::timeout, this, &SystemUpgrade::updateSizeChanged);
+        connect(m_updateSizeTimer, &QTimer::timeout, this, &SystemUpgrade::refreshResource);
     }
 
     QString packageName() const override
@@ -81,6 +89,9 @@ public:
     }
     QString name() const override
     {
+        if (isDistroUpgrade()) {
+            return i18nc("distro upgrade: name version", "%1 %2", AppStreamIntegration::global()->osRelease()->name(), m_distroUpgrade->version());
+        }
         return i18n("System upgrade");
     }
     QString comment() override
@@ -119,7 +130,7 @@ public:
     {
         QVector<PackageKitResource *> ret;
         QSet<QString> donePkgs;
-        for (auto res : qAsConst(m_resources)) {
+        for (auto res : std::as_const(m_resources)) {
             PackageKitResource *app = qobject_cast<PackageKitResource *>(res);
             QString pkgname = app->packageName();
             if (!donePkgs.contains(pkgname)) {
@@ -133,6 +144,10 @@ public:
     quint64 size() override
     {
         quint64 ret = 0;
+        if (isDistroUpgrade()) {
+            return ret;
+        }
+
         const auto resources = withoutDuplicates();
         for (auto res : resources) {
             ret += res->size();
@@ -142,7 +157,7 @@ public:
     QJsonArray licenses() override
     {
         QJsonArray ret;
-        for (auto res : qAsConst(m_resources)) {
+        for (auto res : std::as_const(m_resources)) {
             ret += res->licenses();
         }
         kRemoveDuplicates(ret, [](const QJsonValueRef &val) -> QString {
@@ -186,11 +201,19 @@ public:
             }
         }
         changes.sort();
+
+        if (isDistroUpgrade()) {
+            changes.prepend(m_distroUpgrade->description());
+        }
         return changes.join(QString());
     }
     void fetchChangelog() override
     {
-        for (auto res : qAsConst(m_resources)) {
+        if (isDistroUpgrade()) {
+            return;
+        }
+
+        for (auto res : std::as_const(m_resources)) {
             res->fetchUpdateDetails();
         }
         Q_EMIT changelogFetched({});
@@ -221,7 +244,7 @@ public:
     QSet<QString> allPackageNames() const
     {
         QSet<QString> ret;
-        for (auto res : qAsConst(m_resources)) {
+        for (auto res : std::as_const(m_resources)) {
             ret += kToSet(qobject_cast<PackageKitResource *>(res)->allPackageNames());
         }
         return ret;
@@ -230,25 +253,53 @@ public:
     void refreshResource()
     {
         Q_EMIT m_backend->resourcesChanged(this, {"size", "license"});
-        m_updateSizeTimer->start();
+        Q_EMIT updateSizeChanged();
     }
 
     void setCandidates(const QSet<AbstractResource *> &candidates)
     {
         const auto toDisconnect = (m_resources - candidates);
         for (auto res : toDisconnect) {
-            disconnect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::refreshResource);
-            disconnect(res, &AbstractResource::changelogFetched, this, &SystemUpgrade::longDescriptionChanged);
+            disconnect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::startIfStopped);
+            disconnect(res, &AbstractResource::changelogFetched, this, &SystemUpgrade::startIfStopped);
         }
 
         const auto newCandidates = (candidates - m_resources);
         m_resources = candidates;
         for (auto res : newCandidates) {
-            connect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::refreshResource);
-            connect(res, &AbstractResource::changelogFetched, this, &SystemUpgrade::longDescriptionChanged);
+            connect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::startIfStopped);
+            connect(res, &AbstractResource::changelogFetched, this, &SystemUpgrade::startIfStopped);
         }
     }
 
+    void startIfStopped()
+    {
+        if (!m_updateSizeTimer->isActive()) {
+            m_updateSizeTimer->start();
+        }
+    }
+
+    void setDistroUpgrade(const AppStream::Release &release)
+    {
+        m_distroUpgrade = release;
+    }
+
+    void clearDistroUpgrade()
+    {
+        m_distroUpgrade = std::nullopt;
+        Q_EMIT m_backend->inlineMessageChanged({});
+    }
+
+    bool isDistroUpgrade() const
+    {
+        return m_distroUpgrade.has_value();
+    }
+
+    const AppStream::Release &getDistroUpgrade()
+    {
+        return m_distroUpgrade.value();
+    }
+
 Q_SIGNALS:
     void updateSizeChanged();
 
@@ -256,6 +307,7 @@ private:
     QSet<AbstractResource *> m_resources;
     PackageKitBackend *const m_backend;
     QTimer *m_updateSizeTimer;
+    std::optional<AppStream::Release> m_distroUpgrade;
 };
 
 PackageKitUpdater::PackageKitUpdater(PackageKitBackend *parent)
@@ -277,13 +329,18 @@ PackageKitUpdater::~PackageKitUpdater()
 
 void PackageKitUpdater::prepare()
 {
-    if (PackageKit::Daemon::global()->offline()->updateTriggered()) {
+    if (auto offline = PackageKit::Daemon::global()->offline(); offline->updateTriggered() || offline->upgradeTriggered()) {
         m_toUpgrade.clear();
         m_allUpgradeable.clear();
-        enableNeedsReboot();
+        setNeedsReboot(true);
         return;
     }
 
+    if (QFile::exists(QStringLiteral(PK_OFFLINE_RESULTS_FILENAME))) {
+        qCDebug(LIBDISCOVER_BACKEND_LOG) << "Removed offline results file";
+        PackageKit::Daemon::global()->offline()->clearResults();
+    }
+
     Q_ASSERT(!m_transaction);
     const auto candidates = m_backend->upgradeablePackages();
     if (useOfflineUpdates() && !candidates.isEmpty()) {
@@ -301,12 +358,12 @@ void PackageKitUpdater::prepare()
 
 void PackageKitUpdater::checkFreeSpace()
 {
-    auto j = KIO::fileSystemFreeSpace(QUrl::fromLocalFile("/usr"));
-    connect(j, &KIO::FileSystemFreeSpaceJob::result, this, [this](KIO::Job * /*job*/, KIO::filesize_t /*size*/, KIO::filesize_t available) {
-        if (available < updateSize()) {
+    auto j = KIO::fileSystemFreeSpace(QUrl::fromLocalFile(QStringLiteral("/usr")));
+    connect(j, &KJob::result, this, [this, j]() {
+        if (j->availableSize() < updateSize()) {
             setErrorMessage(i18nc("@info:status %1 is a formatted disk space string e.g. '240 MiB'",
                                   "Not enough space to perform the update; only %1 of space are available.",
-                                  KFormat().formatByteSize(available)));
+                                  KFormat().formatByteSize(j->availableSize())));
         }
     });
 }
@@ -314,9 +371,15 @@ void PackageKitUpdater::checkFreeSpace()
 void PackageKitUpdater::setupTransaction(PackageKit::Transaction::TransactionFlags flags)
 {
     m_packagesModified.clear();
-    auto pkgs = involvedPackages(m_toUpgrade).values();
-    pkgs.sort();
-    m_transaction = PackageKit::Daemon::updatePackages(pkgs, flags);
+    if (m_toUpgrade.contains(m_upgrade) && m_upgrade->isDistroUpgrade()) {
+        const QString &upgradeVersion = m_upgrade->getDistroUpgrade().version();
+        m_transaction = PackageKit::Daemon::upgradeSystem(upgradeVersion, PackageKit::Transaction::UpgradeKind::UpgradeKindComplete, flags);
+        m_transaction->setHints(m_backend->globalHints() << QStringLiteral("cache-age=86400" /* 24*60*60 */));
+    } else {
+        auto pkgs = involvedPackages(m_toUpgrade).values();
+        pkgs.sort();
+        m_transaction = PackageKit::Daemon::updatePackages(pkgs, flags);
+    }
     m_isCancelable = m_transaction->allowCancel();
     cancellableChanged();
 
@@ -327,17 +390,19 @@ void PackageKitUpdater::setupTransaction(PackageKit::Transaction::TransactionFla
     connect(m_transaction.data(), &PackageKit::Transaction::eulaRequired, this, &PackageKitUpdater::eulaRequired);
     connect(m_transaction.data(), &PackageKit::Transaction::repoSignatureRequired, this, &PackageKitUpdater::repoSignatureRequired);
     connect(m_transaction.data(), &PackageKit::Transaction::allowCancelChanged, this, &PackageKitUpdater::cancellableChanged);
-    connect(m_transaction.data(), &PackageKit::Transaction::percentageChanged, this, &PackageKitUpdater::percentageChanged);
     connect(m_transaction.data(), &PackageKit::Transaction::itemProgress, this, &PackageKitUpdater::itemProgress);
     connect(m_transaction.data(), &PackageKit::Transaction::speedChanged, this, [this] {
         Q_EMIT downloadSpeedChanged(downloadSpeed());
     });
-    if (m_toUpgrade.contains(m_upgrade)) {
-        connect(m_transaction, &PackageKit::Transaction::percentageChanged, this, [this] {
-            if (m_transaction->status() == PackageKit::Transaction::StatusDownload) {
-                Q_EMIT resourceProgressed(m_upgrade, m_transaction->percentage(), Downloading);
-            }
-        });
+    if (!(flags & PackageKit::Transaction::TransactionFlagSimulate)) {
+        connect(m_transaction.data(), &PackageKit::Transaction::percentageChanged, this, &PackageKitUpdater::percentageChanged);
+        if (m_toUpgrade.contains(m_upgrade)) {
+            connect(m_transaction, &PackageKit::Transaction::percentageChanged, this, [this] {
+                if (m_transaction->status() == PackageKit::Transaction::StatusDownload) {
+                    Q_EMIT resourceProgressed(m_upgrade, m_transaction->percentage(), Downloading);
+                }
+            });
+        }
     }
 }
 
@@ -348,7 +413,7 @@ QSet<AbstractResource *> PackageKitUpdater::packagesForPackageId(const QSet<QStr
     });
 
     QSet<AbstractResource *> ret;
-    for (AbstractResource *res : qAsConst(m_allUpgradeable)) {
+    for (AbstractResource *res : std::as_const(m_allUpgradeable)) {
         if (auto upgrade = dynamic_cast<SystemUpgrade *>(res)) {
             if (packages.contains(upgrade->allPackageNames())) {
                 ret += upgrade;
@@ -417,7 +482,7 @@ void PackageKitUpdater::proceed()
 
 bool PackageKitUpdater::useOfflineUpdates() const
 {
-    return m_useOfflineUpdates || qEnvironmentVariableIntValue("PK_OFFLINE_UPDATE");
+    return m_useOfflineUpdates || m_upgrade->isDistroUpgrade() || qEnvironmentVariableIntValue("PK_OFFLINE_UPDATE");
 }
 
 void PackageKitUpdater::setOfflineUpdates(bool use)
@@ -425,6 +490,21 @@ void PackageKitUpdater::setOfflineUpdates(bool use)
     m_useOfflineUpdates = use;
 }
 
+void PackageKitUpdater::setDistroUpgrade(const AppStream::Release &release)
+{
+    m_upgrade->setDistroUpgrade(release);
+}
+
+void PackageKitUpdater::clearDistroUpgrade()
+{
+    m_upgrade->clearDistroUpgrade();
+}
+
+bool PackageKitUpdater::isDistroUpgrade() const
+{
+    return m_upgrade->isDistroUpgrade();
+}
+
 void PackageKitUpdater::start()
 {
     Q_ASSERT(!isProgressing());
@@ -433,7 +513,7 @@ void PackageKitUpdater::start()
     setProgressing(true);
 
     if (useOfflineUpdates()) {
-        enableNeedsReboot();
+        setNeedsReboot(true);
     }
 }
 
@@ -468,10 +548,10 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
             }
         };
 
-        if (PackageKit::Daemon::backendName() == "dnf") {
+        if (PackageKit::Daemon::backendName() == QLatin1String("dnf")) {
             // Fedora has some packages that it uninstalls then eventually creates on its own. No need to
             // notify about these.
-            toremove = kFilter<typeof(toremove)>(toremove, [](const QString &pkgid) {
+            toremove = kFilter<QStringList>(toremove, [](const QString &pkgid) {
                 return !PackageKit::Transaction::packageName(pkgid).startsWith(QLatin1String("kmod"));
             });
         }
@@ -499,7 +579,7 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
                 Q_EMIT distroErrorMessage(msg);
             } else {
                 Q_EMIT proceedRequest(
-                    i18n("Packages to remove"),
+                    i18n("Confirm Changes"),
                     i18n("The following packages will be removed by the update:<ul><li>%1</li></ul><br/>in order to install:<ul><li>%2</li></ul>",
                          PackageKitResource::joinPackages(toremove, QStringLiteral("</li><li>"), {}),
                          PackageKitResource::joinPackages(toinstall, QStringLiteral("</li><li>"), {})));
@@ -510,12 +590,35 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
         return;
     }
 
-    setProgressing(false);
-    m_backend->fetchUpdates();
-    fetchLastUpdateTime();
+    if (!m_toUpgrade.contains(m_upgrade) || !m_upgrade->isDistroUpgrade()) {
+        // If the distro-upgrade was prepared successfully, fetching updates now would invalidate it.
+        // If the distro-upgrade failed, fetching updates now would find nothing and we would propose to reboot.
+        setProgressing(false);
+        m_backend->fetchUpdates();
+        fetchLastUpdateTime();
+    } else if (exit != PackageKit::Transaction::ExitSuccess) {
+        Q_EMIT resourceProgressed(m_upgrade, 0, None);
+        setNeedsReboot(false);
+        setProgressing(false);
+    }
 
     if (useOfflineUpdates() && exit == PackageKit::Transaction::ExitSuccess) {
-        PackageKit::Daemon::global()->offline()->trigger(PackageKit::Offline::ActionReboot);
+        if (m_upgrade->isDistroUpgrade()) {
+            QDBusPendingReply<void> reply = PackageKit::Daemon::global()->offline()->triggerUpgrade(PackageKit::Offline::ActionReboot);
+            // Call may fail because of authorization
+            reply.waitForFinished();
+            if (reply.isError()) {
+                Q_EMIT resourceProgressed(m_upgrade, 0, None);
+                setNeedsReboot(false);
+                setProgressing(false);
+                Q_EMIT passiveMessage(reply.error().message());
+                return;
+            }
+            m_backend->clear();
+            setProgressing(false);
+        } else {
+            PackageKit::Daemon::global()->offline()->trigger(PackageKit::Offline::ActionReboot);
+        }
         enableReadyToReboot();
     }
 }
@@ -530,7 +633,10 @@ void PackageKitUpdater::cancellableChanged()
 
 void PackageKitUpdater::percentageChanged()
 {
-    const auto actualPercentage = percentageWithStatus(m_transaction->status(), m_transaction->percentage());
+    const int percentage = m_transaction->percentage();
+    if (percentage > 100)
+        return;
+    const auto actualPercentage = useOfflineUpdates() ? percentage : percentageWithStatus(m_transaction->status(), percentage);
     if (actualPercentage >= 0 && m_percentage != actualPercentage) {
         m_percentage = actualPercentage;
         Q_EMIT progressChanged(m_percentage);
@@ -709,7 +815,7 @@ void PackageKitUpdater::itemProgress(const QString &itemID, PackageKit::Transact
 void PackageKitUpdater::fetchChangelog() const
 {
     QStringList pkgids;
-    for (AbstractResource *res : qAsConst(m_allUpgradeable)) {
+    for (AbstractResource *res : std::as_const(m_allUpgradeable)) {
         if (auto upgrade = dynamic_cast<SystemUpgrade *>(res)) {
             upgrade->fetchChangelog();
         } else {
