@@ -8,11 +8,11 @@
 #include "RpmOstreeBackend.h"
 #include "RpmOstreeSourcesBackend.h"
 
+#include "Category/Category.h"
 #include "Transaction/TransactionModel.h"
 
 #include <AppStreamQt/release.h>
 #include <AppStreamQt/utils.h>
-#include <Category/Category.h>
 #include <KLocalizedString>
 #include <appstream/AppStreamIntegration.h>
 
@@ -119,7 +119,7 @@ void RpmOstreeBackend::initializeBackend()
     // Register ourselves as update driver
     if (!m_registrered) {
         QVariantMap options;
-        options["id"] = QVariant{QStringLiteral("discover")};
+        options[QLatin1String("id")] = QVariant{QStringLiteral("discover")};
         auto reply = m_interface->RegisterClient(options);
         QDBusPendingCallWatcher *callWatcher = new QDBusPendingCallWatcher(reply, this);
         connect(callWatcher, &QDBusPendingCallWatcher::finished, [this, callWatcher]() {
@@ -185,7 +185,7 @@ void RpmOstreeBackend::refreshDeployments()
             m_currentlyBootedDeployment = deployment;
         } else if (deployment->isPending()) {
             // Signal that we have a pending update
-            m_updater->enableNeedsReboot();
+            m_updater->setNeedsReboot(true);
         }
     }
 
@@ -277,7 +277,7 @@ void RpmOstreeBackend::checkForUpdates()
             if (deployment->version() == newVersion) {
                 qInfo() << "rpm-ostree-backend: Found existing deployment for new version. Skipping.";
                 // Let the user know that the update is pending a reboot
-                m_updater->enableNeedsReboot();
+                m_updater->setNeedsReboot(true);
                 if (m_currentlyBootedDeployment->getNextMajorVersion().isEmpty()) {
                     Q_EMIT inlineMessageChanged(nullptr);
                 } else {
@@ -311,12 +311,12 @@ void RpmOstreeBackend::lookForNextMajorVersion()
     // Get the DistroComponentId. For Fedora Kinoite, we follow Fedora's
     // release schedule so we don't have our own ID.
     QString distroId = AppStream::Utils::currentDistroComponentId();
-    if (distroId == "org.fedoraproject.kinoite.fedora") {
-        distroId = "org.fedoraproject.fedora";
+    if (distroId == QLatin1String("org.fedoraproject.kinoite.fedora")) {
+        distroId = QStringLiteral("org.fedoraproject.fedora");
     }
 
     // Look at releases to see if we have a new major version available.
-    const QList<AppStream::Component> distroComponents = m_appdata->componentsById(distroId);
+    const auto distroComponents = m_appdata->componentsById(distroId);
     if (distroComponents.isEmpty()) {
         qWarning() << "rpm-ostree-backend: No component found for" << distroId;
         return;
@@ -325,7 +325,11 @@ void RpmOstreeBackend::lookForNextMajorVersion()
     QString currentVersion = AppStreamIntegration::global()->osRelease()->versionId();
     QString nextVersion;
     for (const AppStream::Component &dc : distroComponents) {
+#if ASQ_CHECK_VERSION(1, 0, 0)
+        const auto releases = dc.releasesPlain().entries();
+#else
         const auto releases = dc.releases();
+#endif
         for (const auto &r : releases) {
             // Only look at stable releases unless development mode is enabled
             if ((r.kind() != AppStream::Release::KindStable) && !m_developmentEnabled) {
@@ -414,13 +418,13 @@ void RpmOstreeBackend::foundNewMajorVersion(const QString &newMajorVersion)
     while (iterator.hasNext()) {
         RpmOstreeResource *deployment = iterator.next();
         QString deploymentVersion = deployment->version();
-        QStringList deploymentVersionSplit = deploymentVersion.split('.');
+        QStringList deploymentVersionSplit = deploymentVersion.split(QLatin1Char('.'));
         if (!deploymentVersionSplit.empty()) {
             deploymentVersion = deploymentVersionSplit.at(0);
         }
         if (deploymentVersion == newMajorVersion) {
             qInfo() << "rpm-ostree-backend: Found existing deployment for new major version";
-            m_updater->enableNeedsReboot();
+            m_updater->setNeedsReboot(true);
             Q_EMIT inlineMessageChanged(nullptr);
             return;
         }
@@ -434,7 +438,7 @@ void RpmOstreeBackend::foundNewMajorVersion(const QString &newMajorVersion)
         RpmOstreeResource *deployment = iterator.next();
         if ((deployment->version() == newVersion) || deployment->isPending()) {
             qInfo() << "rpm-ostree-backend: Found pending or updated deployment for current version";
-            m_updater->enableNeedsReboot();
+            m_updater->setNeedsReboot(true);
             Q_EMIT inlineMessageChanged(m_rebootBeforeRebaseMessage);
             return;
         }
@@ -446,7 +450,7 @@ void RpmOstreeBackend::foundNewMajorVersion(const QString &newMajorVersion)
     // version.
     if (m_currentlyBootedDeployment->state() == AbstractResource::Upgradeable) {
         qInfo() << "rpm-ostree-backend: Found pending update for current version";
-        m_updater->enableNeedsReboot();
+        m_updater->setNeedsReboot(true);
         Q_EMIT inlineMessageChanged(m_rebootBeforeRebaseMessage);
         return;
     }
