@@ -97,7 +97,7 @@ void SteamOSBackend::hasUpdateChanged(bool hasUpdate)
 void SteamOSBackend::needRebootChanged()
 {
     // Tell gui we need to reboot
-    m_updater->enableNeedsReboot();
+    m_updater->setNeedsReboot(true);
 }
 
 void SteamOSBackend::acquireFetching(bool f)
@@ -150,8 +150,8 @@ void SteamOSBackend::checkForUpdatesFinished(QDBusPendingCallWatcher *call)
         } else {
             m_updateBuild = versions.keys().at(0);
             QVariantMap data = versions.value(m_updateBuild);
-            m_updateVersion = data.value("version").toString();
-            m_updateSize = data.value("estimated_size").toUInt();
+            m_updateVersion = data.value(QLatin1String("version")).toString();
+            m_updateSize = data.value(QLatin1String("estimated_size")).toUInt();
             qDebug() << "steamos-backend: Data values: " << data.values();
             hasUpdateChanged(true);
         }
@@ -170,9 +170,9 @@ int SteamOSBackend::updatesCount() const
 // give a void stream
 ResultsStream *SteamOSBackend::search(const AbstractResourcesBackend::Filters &filter)
 {
-    QVector<AbstractResource *> res;
+    QVector<StreamResult> res;
     if (m_resource && m_resource->state() >= filter.state)
-        res << m_resource;
+        res << StreamResult{m_resource, 0};
     return new ResultsStream(QLatin1String("SteamOS-stream"), res);
 }
 
