@@ -32,12 +32,14 @@ ResourcesModel *ResourcesModel::s_self = nullptr;
 
 ResourcesModel *ResourcesModel::global()
 {
-    if (!s_self)
+    if (!s_self) {
         s_self = new ResourcesModel;
+        s_self->init(true);
+    }
     return s_self;
 }
 
-ResourcesModel::ResourcesModel(QObject *parent, bool load)
+ResourcesModel::ResourcesModel(QObject *parent)
     : QObject(parent)
     , m_isFetching(false)
     , m_initializingBackendsCount(0)
@@ -46,13 +48,11 @@ ResourcesModel::ResourcesModel(QObject *parent, bool load)
     , m_updatesCount(
           0,
           [this] {
-              {
-                  int ret = 0;
-                  for (AbstractResourcesBackend *backend : qAsConst(m_backends)) {
-                      ret += backend->updatesCount();
-                  }
-                  return ret;
+              int ret = 0;
+              for (AbstractResourcesBackend *backend : std::as_const(m_backends)) {
+                  ret += backend->updatesCount();
               }
+              return ret;
           },
           [this](int count) {
               Q_EMIT updatesCountChanged(count);
@@ -60,29 +60,27 @@ ResourcesModel::ResourcesModel(QObject *parent, bool load)
     , m_fetchingUpdatesProgress(
           0,
           [this] {
-              {
-                  if (m_backends.isEmpty())
-                      return 0;
+              if (m_backends.isEmpty())
+                  return 0;
 
-                  int sum = 0;
-                  for (auto backend : qAsConst(m_backends)) {
-                      sum += backend->fetchingUpdatesProgress();
-                  }
-                  return sum / (int)m_backends.count();
+              int sum = 0;
+              int weights = 0;
+              for (auto backend : std::as_const(m_backends)) {
+                  sum += backend->fetchingUpdatesProgress() * backend->fetchingUpdatesProgressWeight();
+                  weights += backend->fetchingUpdatesProgressWeight();
               }
+              return sum / weights;
           },
           [this](int progress) {
               Q_EMIT fetchingUpdatesProgressChanged(progress);
           })
 {
-    init(load);
     connect(this, &ResourcesModel::allInitialized, this, &ResourcesModel::slotFetching);
     connect(this, &ResourcesModel::backendsChanged, this, &ResourcesModel::initApplicationsBackend);
 }
 
 void ResourcesModel::init(bool load)
 {
-    Q_ASSERT(!s_self);
     Q_ASSERT(QCoreApplication::instance()->thread() == QThread::currentThread());
 
     m_allInitializedEmitter->setSingleShot(true);
@@ -94,8 +92,9 @@ void ResourcesModel::init(bool load)
         }
     });
 
-    if (load)
-        QMetaObject::invokeMethod(this, "registerAllBackends", Qt::QueuedConnection);
+    if (load) {
+        registerAllBackends();
+    }
 
     m_updateAction = new DiscoverAction(this);
     m_updateAction->setIconName(QStringLiteral("system-software-update"));
@@ -110,10 +109,11 @@ void ResourcesModel::init(bool load)
 }
 
 ResourcesModel::ResourcesModel(const QString &backendName, QObject *parent)
-    : ResourcesModel(parent, false)
+    : ResourcesModel(parent)
 {
     s_self = this;
     registerBackendByName(backendName);
+    init(false);
 }
 
 ResourcesModel::~ResourcesModel()
@@ -150,7 +150,7 @@ void ResourcesModel::addResourcesBackend(AbstractResourcesBackend *backend)
     });
     connect(backend, &AbstractResourcesBackend::resourceRemoved, this, &ResourcesModel::resourceRemoved);
     connect(backend, &AbstractResourcesBackend::passiveMessage, this, &ResourcesModel::passiveMessage);
-    connect(backend, &AbstractResourcesBackend::inlineMessage, this, &ResourcesModel::inlineMessage);
+    connect(backend, &AbstractResourcesBackend::inlineMessageChanged, this, &ResourcesModel::setInlineMessage);
     connect(backend->backendUpdater(), &AbstractBackendUpdater::progressingChanged, this, &ResourcesModel::slotFetching);
     if (backend->reviewsBackend()) {
         connect(backend->reviewsBackend(), &AbstractReviewsBackend::error, this, &ResourcesModel::passiveMessage, Qt::UniqueConnection);
@@ -211,7 +211,7 @@ bool ResourcesModel::hasSecurityUpdates() const
 {
     bool ret = false;
 
-    for (AbstractResourcesBackend *backend : qAsConst(m_backends)) {
+    for (AbstractResourcesBackend *backend : std::as_const(m_backends)) {
         ret |= backend->hasSecurityUpdates();
     }
 
@@ -271,7 +271,7 @@ bool ResourcesModel::isInitializing() const
 void ResourcesModel::slotFetching()
 {
     bool newFetching = false;
-    for (AbstractResourcesBackend *b : qAsConst(m_backends)) {
+    for (AbstractResourcesBackend *b : std::as_const(m_backends)) {
         // isFetching should sort of be enough. However, sometimes the backend itself
         // will still be operating on things, which from a model point of view would
         // still mean something going on. So, interpret that as fetching as well, for
@@ -295,8 +295,8 @@ bool ResourcesModel::isBusy() const
 bool ResourcesModel::isExtended(const QString &id)
 {
     bool ret = true;
-    for (AbstractResourcesBackend *backend : qAsConst(m_backends)) {
-        ret = backend->extends().contains(id);
+    for (AbstractResourcesBackend *backend : std::as_const(m_backends)) {
+        ret = backend->extends(id);
         if (ret)
             break;
     }
@@ -325,10 +325,10 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream *> &st
 
 AggregatedResultsStream::~AggregatedResultsStream() = default;
 
-void AggregatedResultsStream::addResults(const QVector<AbstractResource *> &res)
+void AggregatedResultsStream::addResults(const QVector<StreamResult> &res)
 {
     for (auto r : res)
-        connect(r, &QObject::destroyed, this, &AggregatedResultsStream::resourceDestruction);
+        connect(r.resource, &QObject::destroyed, this, &AggregatedResultsStream::resourceDestruction);
 
     m_results += res;
 
@@ -347,7 +347,12 @@ void AggregatedResultsStream::emitResults()
 
 void AggregatedResultsStream::resourceDestruction(QObject *obj)
 {
-    m_results.removeAll(qobject_cast<AbstractResource *>(obj));
+    for (auto it = m_results.begin(); it != m_results.end(); ++it) {
+        if (obj == it->resource)
+            it = m_results.erase(it);
+        else
+            ++it;
+    }
 }
 
 void AggregatedResultsStream::streamDestruction(QObject *obj)
@@ -379,7 +384,7 @@ AggregatedResultsStream *ResourcesModel::search(const AbstractResourcesBackend::
 
 void ResourcesModel::checkForUpdates()
 {
-    for (auto backend : qAsConst(m_backends))
+    for (auto backend : std::as_const(m_backends))
         backend->checkForUpdates();
 }
 
@@ -427,7 +432,22 @@ QString ResourcesModel::applicationSourceName() const
     return settings.readEntry<QString>("currentApplicationBackend", QStringLiteral("packagekit-backend"));
 }
 
+QString ResourcesModel::distroName() const
+{
+    return KOSRelease().name();
+}
+
 QUrl ResourcesModel::distroBugReportUrl()
 {
     return QUrl(KOSRelease().bugReportUrl());
 }
+
+void ResourcesModel::setInlineMessage(const QSharedPointer<InlineMessage> &inlineMessage)
+{
+    if (inlineMessage == m_inlineMessage) {
+        return;
+    }
+
+    m_inlineMessage = inlineMessage;
+    Q_EMIT inlineMessageChanged(inlineMessage);
+}
