@@ -10,6 +10,7 @@
 #include "libdiscover_debug.h"
 #include <QMetaProperty>
 #include <cmath>
+#include <qnamespace.h>
 #include <utils.h>
 
 #include "ResourcesModel.h"
@@ -18,33 +19,81 @@
 #include <ReviewsBackend/Rating.h>
 #include <Transaction/TransactionModel.h>
 
+const QHash<int, QByteArray> ResourcesProxyModel::s_roles = {{NameRole, "name"},
+                                                             {IconRole, "icon"},
+                                                             {CommentRole, "comment"},
+                                                             {StateRole, "state"},
+                                                             {RatingRole, "rating"},
+                                                             {RatingPointsRole, "ratingPoints"},
+                                                             {RatingCountRole, "ratingCount"},
+                                                             {SortableRatingRole, "sortableRating"},
+                                                             {SearchRelevanceRole, "searchRelevance"},
+                                                             {InstalledRole, "isInstalled"},
+                                                             {ApplicationRole, "application"},
+                                                             {OriginRole, "origin"},
+                                                             {DisplayOriginRole, "displayOrigin"},
+                                                             {CanUpgrade, "canUpgrade"},
+                                                             {PackageNameRole, "packageName"},
+                                                             {CategoryRole, "category"},
+                                                             {SectionRole, "section"},
+                                                             {MimeTypes, "mimetypes"},
+                                                             {LongDescriptionRole, "longDescription"},
+                                                             {SourceIconRole, "sourceIcon"},
+                                                             {SizeRole, "size"},
+                                                             {ReleaseDateRole, "releaseDate"}};
+
+int levenshteinDistance(const QString &source, const QString &target)
+{
+    if (source == target) {
+        return 0;
+    }
+
+    // Do a case insensitive version of it
+    const QString &sourceUp = source.toUpper();
+    const QString &targetUp = target.toUpper();
+
+    if (sourceUp == targetUp) {
+        return 0;
+    }
+
+    const int sourceCount = sourceUp.size();
+    const int targetCount = targetUp.size();
+
+    if (sourceUp.isEmpty()) {
+        return targetCount;
+    }
+
+    if (targetUp.isEmpty()) {
+        return sourceCount;
+    }
+
+    if (sourceCount > targetCount) {
+        return levenshteinDistance(targetUp, sourceUp);
+    }
+
+    QVector<int> column;
+    column.fill(0, targetCount + 1);
+    QVector<int> previousColumn;
+    previousColumn.reserve(targetCount + 1);
+    for (int i = 0; i < targetCount + 1; i++) {
+        previousColumn.append(i);
+    }
+
+    for (int i = 0; i < sourceCount; i++) {
+        column[0] = i + 1;
+        for (int j = 0; j < targetCount; j++) {
+            column[j + 1] = std::min({1 + column.at(j), 1 + previousColumn.at(1 + j), previousColumn.at(j) + ((sourceUp.at(i) == targetUp.at(j)) ? 0 : 1)});
+        }
+        column.swap(previousColumn);
+    }
+
+    return previousColumn.at(targetCount);
+}
+
 ResourcesProxyModel::ResourcesProxyModel(QObject *parent)
     : QAbstractListModel(parent)
     , m_sortRole(NameRole)
     , m_sortOrder(Qt::AscendingOrder)
-    , m_sortByRelevancy(false)
-    , m_roles({{NameRole, "name"},
-               {IconRole, "icon"},
-               {CommentRole, "comment"},
-               {StateRole, "state"},
-               {RatingRole, "rating"},
-               {RatingPointsRole, "ratingPoints"},
-               {RatingCountRole, "ratingCount"},
-               {SortableRatingRole, "sortableRating"},
-               {InstalledRole, "isInstalled"},
-               {ApplicationRole, "application"},
-               {OriginRole, "origin"},
-               {DisplayOriginRole, "displayOrigin"},
-               {CanUpgrade, "canUpgrade"},
-               {PackageNameRole, "packageName"},
-               {CategoryRole, "category"},
-               {CategoryDisplayRole, "categoryDisplay"},
-               {SectionRole, "section"},
-               {MimeTypes, "mimetypes"},
-               {LongDescriptionRole, "longDescription"},
-               {SourceIconRole, "sourceIcon"},
-               {SizeRole, "size"},
-               {ReleaseDateRole, "releaseDate"}})
     , m_currentStream(nullptr)
 {
     // new QAbstractItemModelTester(this, this);
@@ -75,7 +124,7 @@ void ResourcesProxyModel::componentComplete()
 
 QHash<int, QByteArray> ResourcesProxyModel::roleNames() const
 {
-    return m_roles;
+    return s_roles;
 }
 
 void ResourcesProxyModel::setSortRole(Roles sortRole)
@@ -101,28 +150,24 @@ void ResourcesProxyModel::setSortOrder(Qt::SortOrder sortOrder)
 void ResourcesProxyModel::setSearch(const QString &_searchText)
 {
     // 1-character searches are painfully slow. >= 2 chars are fine, though
-    const QString searchText = _searchText.count() <= 1 ? QString() : _searchText;
+    const QString searchText = _searchText.size() <= 1 ? QString() : _searchText;
 
     const bool diff = searchText != m_filters.search;
 
     if (diff) {
         m_filters.search = searchText;
-        if (m_sortByRelevancy == searchText.isEmpty()) {
-            m_sortByRelevancy = !searchText.isEmpty();
-            Q_EMIT sortByRelevancyChanged(m_sortByRelevancy);
-        }
         invalidateFilter();
         Q_EMIT searchChanged(m_filters.search);
     }
 }
 
-void ResourcesProxyModel::removeDuplicates(QVector<AbstractResource *> &resources)
+void ResourcesProxyModel::removeDuplicates(QVector<StreamResult> &resources)
 {
     const auto cab = ResourcesModel::global()->currentApplicationBackend();
     QHash<QString, QString> aliases;
-    QHash<QString, QVector<AbstractResource *>::iterator> storedIds;
+    QHash<QString, QVector<StreamResult>::iterator> storedIds;
     for (auto it = m_displayedResources.begin(); it != m_displayedResources.end(); ++it) {
-        const auto appstreamid = (*it)->appstreamId();
+        const auto appstreamid = it->resource->appstreamId();
         if (appstreamid.isEmpty()) {
             continue;
         }
@@ -134,15 +179,15 @@ void ResourcesProxyModel::removeDuplicates(QVector<AbstractResource *> &resource
             Q_UNREACHABLE();
         }
 
-        const auto alts = (*it)->alternativeAppstreamIds();
+        const auto alts = it->resource->alternativeAppstreamIds();
         for (const auto &alias : alts) {
             aliases[alias] = appstreamid;
         }
     }
 
-    QHash<QString, QVector<AbstractResource *>::iterator> ids;
+    QHash<QString, QVector<StreamResult>::iterator> ids;
     for (auto it = resources.begin(); it != resources.end();) {
-        const auto appstreamid = (*it)->appstreamId();
+        const auto appstreamid = it->resource->appstreamId();
         if (appstreamid.isEmpty()) {
             ++it;
             continue;
@@ -156,7 +201,7 @@ void ResourcesProxyModel::removeDuplicates(QVector<AbstractResource *> &resource
         }
 
         if (at == storedIds.end()) {
-            const auto alts = (*it)->alternativeAppstreamIds();
+            const auto alts = it->resource->alternativeAppstreamIds();
             for (const auto &alt : alts) {
                 at = storedIds.find(alt);
                 if (at == storedIds.end())
@@ -179,7 +224,7 @@ void ResourcesProxyModel::removeDuplicates(QVector<AbstractResource *> &resource
                 }
             }
             if (at == ids.end()) {
-                const auto alts = (*it)->alternativeAppstreamIds();
+                const auto alts = it->resource->alternativeAppstreamIds();
                 for (const auto &alt : alts) {
                     at = ids.find(alt);
                     if (at != ids.end())
@@ -195,19 +240,19 @@ void ResourcesProxyModel::removeDuplicates(QVector<AbstractResource *> &resource
             }
             if (at == ids.end()) {
                 ids[appstreamid] = it;
-                const auto alts = (*it)->alternativeAppstreamIds();
+                const auto alts = it->resource->alternativeAppstreamIds();
                 for (const auto &alias : alts) {
                     aliases[alias] = appstreamid;
                 }
                 ++it;
             } else {
-                if ((*it)->backend() == cab && (*it)->backend() != (**at)->backend()) {
+                if (it->resource->backend() == cab && it->resource->backend() != (**at).resource->backend()) {
                     qSwap(*it, **at);
                 }
                 it = resources.erase(it);
             }
         } else {
-            if ((*it)->backend() == cab) {
+            if (it->resource->backend() == cab) {
                 **at = *it;
                 auto pos = index(*at - m_displayedResources.begin(), 0);
                 Q_EMIT dataChanged(pos, pos);
@@ -217,7 +262,7 @@ void ResourcesProxyModel::removeDuplicates(QVector<AbstractResource *> &resource
     }
 }
 
-void ResourcesProxyModel::addResources(const QVector<AbstractResource *> &_res)
+void ResourcesProxyModel::addResources(const QVector<StreamResult> &_res)
 {
     auto res = _res;
     m_filters.filterJustInCase(res);
@@ -225,10 +270,9 @@ void ResourcesProxyModel::addResources(const QVector<AbstractResource *> &_res)
     if (res.isEmpty())
         return;
 
-    if (!m_sortByRelevancy)
-        std::sort(res.begin(), res.end(), [this](AbstractResource *res, AbstractResource *res2) {
-            return lessThan(res, res2);
-        });
+    std::sort(res.begin(), res.end(), [this](auto res, auto res2) {
+        return orderedLessThan(res, res2);
+    });
 
     sortedInsertion(res);
     fetchSubcategories();
@@ -239,13 +283,11 @@ void ResourcesProxyModel::invalidateSorting()
     if (m_displayedResources.isEmpty())
         return;
 
-    if (!m_sortByRelevancy) {
-        beginResetModel();
-        std::sort(m_displayedResources.begin(), m_displayedResources.end(), [this](AbstractResource *res, AbstractResource *res2) {
-            return lessThan(res, res2);
-        });
-        endResetModel();
-    }
+    beginResetModel();
+    std::sort(m_displayedResources.begin(), m_displayedResources.end(), [this](auto res, auto res2) {
+        return orderedLessThan(res, res2);
+    });
+    endResetModel();
 }
 
 QString ResourcesProxyModel::lastSearch() const
@@ -268,6 +310,32 @@ QString ResourcesProxyModel::originFilter() const
     return m_filters.origin;
 }
 
+QString ResourcesProxyModel::filteredCategoryName() const
+{
+    return m_categoryName;
+}
+
+void ResourcesProxyModel::setFilteredCategoryName(const QString &cat)
+{
+    if (cat == m_categoryName)
+        return;
+
+    m_categoryName = cat;
+
+    auto category = CategoryModel::global()->findCategoryByName(cat);
+    if (category) {
+        setFiltersFromCategory(category);
+    } else {
+        qDebug() << "looking up wrong category or too early" << m_categoryName;
+        auto f = [this, cat] {
+            auto category = CategoryModel::global()->findCategoryByName(cat);
+            setFiltersFromCategory(category);
+        };
+        auto one = new OneTimeAction(f, this);
+        connect(CategoryModel::global(), &CategoryModel::rootCategoriesChanged, one, &OneTimeAction::trigger);
+    }
+}
+
 void ResourcesProxyModel::setFiltersFromCategory(Category *category)
 {
     if (category == m_filters.category)
@@ -285,7 +353,7 @@ void ResourcesProxyModel::fetchSubcategories()
     const int count = rowCount();
     QSet<Category *> done;
     for (int i = 0; i < count && !cats.isEmpty(); ++i) {
-        AbstractResource *res = m_displayedResources[i];
+        AbstractResource *res = m_displayedResources[i].resource;
         const auto found = res->categoryObjects(kSetToVector(cats));
         done.unite(found);
         cats.subtract(found);
@@ -311,6 +379,10 @@ void ResourcesProxyModel::invalidateFilter()
         return;
     }
 
+    if (!m_categoryName.isEmpty() && m_filters.category == nullptr) {
+        return;
+    }
+
     if (m_currentStream) {
         qCWarning(LIBDISCOVER_LOG) << "last stream isn't over yet" << m_filters << this;
         delete m_currentStream;
@@ -337,16 +409,25 @@ int ResourcesProxyModel::rowCount(const QModelIndex &parent) const
     return parent.isValid() ? 0 : m_displayedResources.count();
 }
 
-bool ResourcesProxyModel::lessThan(AbstractResource *leftPackage, AbstractResource *rightPackage) const
+bool ResourcesProxyModel::orderedLessThan(const StreamResult &left, const StreamResult &right) const
 {
+    bool less = lessThan(left, right);
+    return m_sortOrder == Qt::AscendingOrder ? less : !less;
+}
+
+bool ResourcesProxyModel::lessThan(const StreamResult &left, const StreamResult &right) const
+{
+    AbstractResource *leftPackage = left.resource;
+    AbstractResource *rightPackage = right.resource;
+
     auto role = m_sortRole;
     Qt::SortOrder order = m_sortOrder;
     QVariant leftValue;
     QVariant rightValue;
     // if we're comparing two equal values, we want the model sorted by application name
     if (role != NameRole) {
-        leftValue = roleToValue(leftPackage, role);
-        rightValue = roleToValue(rightPackage, role);
+        leftValue = roleToValue(left, role);
+        rightValue = roleToValue(right, role);
 
         if (leftValue == rightValue) {
             role = NameRole;
@@ -360,13 +441,9 @@ bool ResourcesProxyModel::lessThan(AbstractResource *leftPackage, AbstractResour
     } else if (role == CanUpgrade) {
         ret = leftValue.toBool();
     } else {
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-        ret = leftValue < rightValue;
-#else
         const auto order = QVariant::compare(leftValue, rightValue);
         Q_ASSERT(order != QPartialOrdering::Unordered);
         return order == QPartialOrdering::Less;
-#endif
     }
     return ret != (order != Qt::AscendingOrder);
 }
@@ -468,13 +545,13 @@ QVariant ResourcesProxyModel::data(const QModelIndex &index, int role) const
     if (!index.isValid()) {
         return QVariant();
     }
-
-    AbstractResource *const resource = m_displayedResources[index.row()];
-    return roleToValue(resource, role);
+    const auto result = m_displayedResources[index.row()];
+    return roleToValue(result, role);
 }
 
-QVariant ResourcesProxyModel::roleToValue(AbstractResource *resource, int role) const
+QVariant ResourcesProxyModel::roleToValue(const StreamResult &result, int role) const
 {
+    AbstractResource *resource = result.resource;
     switch (role) {
     case ApplicationRole:
         return QVariant::fromValue<QObject *>(resource);
@@ -490,10 +567,28 @@ QVariant ResourcesProxyModel::roleToValue(AbstractResource *resource, int role)
             return prop.readOnGadget(rating);
         } else {
             QVariant val(0);
-            val.convert(prop.type());
+            val.convert(prop.metaType());
             return val;
         }
     }
+    case SearchRelevanceRole: {
+        qreal rating = roleToValue(result, SortableRatingRole).value<qreal>();
+
+        qreal reverseDistance = 0;
+        for (const QString &word : resource->name().split(QLatin1Char(' '))) {
+            const qreal maxLength = std::max(word.length(), m_filters.search.length());
+            reverseDistance =
+                std::max(reverseDistance, (maxLength - std::min(reverseDistance, qreal(levenshteinDistance(word, m_filters.search)))) / maxLength * 10.0);
+        }
+
+        qreal exactMatch = 0.0;
+        if (resource->name().toUpper() == m_filters.search.toUpper()) {
+            exactMatch = 10.0;
+        } else if (resource->name().contains(m_filters.search, Qt::CaseInsensitive)) {
+            exactMatch = 5.0;
+        }
+        return qreal(result.sortScore) / 100 + rating + reverseDistance + exactMatch;
+    }
     case Qt::DecorationRole:
     case Qt::DisplayRole:
     case Qt::StatusTipRole:
@@ -517,11 +612,13 @@ QVariant ResourcesProxyModel::roleToValue(AbstractResource *resource, int role)
     }
 }
 
-bool ResourcesProxyModel::isSorted(const QVector<AbstractResource *> &resources)
+bool ResourcesProxyModel::isSorted(const QVector<StreamResult> &resources)
 {
     auto last = resources.constFirst();
     for (auto it = resources.constBegin() + 1, itEnd = resources.constEnd(); it != itEnd; ++it) {
-        if (!lessThan(last, *it)) {
+        auto v1 = roleToValue(last, m_sortRole), v2 = roleToValue(*it, m_sortRole);
+        if (!lessThan(last, *it) && v1 != v2) {
+            qDebug() << "faulty sort" << last.resource->name() << (*it).resource->name() << last.resource << (*it).resource;
             return false;
         }
         last = *it;
@@ -529,8 +626,10 @@ bool ResourcesProxyModel::isSorted(const QVector<AbstractResource *> &resources)
     return true;
 }
 
-void ResourcesProxyModel::sortedInsertion(const QVector<AbstractResource *> &_res)
+void ResourcesProxyModel::sortedInsertion(const QVector<StreamResult> &_res)
 {
+    Q_ASSERT(_res.size() == QSet(_res.constBegin(), _res.constEnd()).size());
+
     auto resources = _res;
     Q_ASSERT(!resources.isEmpty());
 
@@ -540,8 +639,7 @@ void ResourcesProxyModel::sortedInsertion(const QVector<AbstractResource *> &_re
             return;
     }
 
-    if (m_sortByRelevancy || m_displayedResources.isEmpty()) {
-        // Q_ASSERT(m_sortByRelevancy || isSorted(resources));
+    if (m_displayedResources.isEmpty()) {
         int rows = rowCount();
         beginInsertRows({}, rows, rows + resources.count() - 1);
         m_displayedResources += resources;
@@ -549,18 +647,18 @@ void ResourcesProxyModel::sortedInsertion(const QVector<AbstractResource *> &_re
         return;
     }
 
-    for (auto resource : qAsConst(resources)) {
-        const auto finder = [this](AbstractResource *resource, AbstractResource *res) {
-            return lessThan(resource, res);
+    for (auto result : std::as_const(resources)) {
+        const auto finder = [this](StreamResult result, StreamResult res) {
+            return orderedLessThan(result, res);
         };
-        const auto it = std::upper_bound(m_displayedResources.constBegin(), m_displayedResources.constEnd(), resource, finder);
+        const auto it = std::upper_bound(m_displayedResources.constBegin(), m_displayedResources.constEnd(), result, finder);
         const auto newIdx = it == m_displayedResources.constEnd() ? m_displayedResources.count() : (it - m_displayedResources.constBegin());
 
-        if ((it - 1) != m_displayedResources.constEnd() && *(it - 1) == resource)
+        if ((it - 1) != m_displayedResources.constEnd() && (it - 1)->resource == result.resource)
             continue;
 
         beginInsertRows({}, newIdx, newIdx);
-        m_displayedResources.insert(newIdx, resource);
+        m_displayedResources.insert(newIdx, result);
         endInsertRows();
         // Q_ASSERT(isSorted(resources));
     }
@@ -568,7 +666,7 @@ void ResourcesProxyModel::sortedInsertion(const QVector<AbstractResource *> &_re
 
 void ResourcesProxyModel::refreshResource(AbstractResource *resource, const QVector<QByteArray> &properties)
 {
-    const auto residx = m_displayedResources.indexOf(resource);
+    const auto residx = indexOf(resource);
     if (residx < 0) {
         return;
     }
@@ -583,19 +681,19 @@ void ResourcesProxyModel::refreshResource(AbstractResource *resource, const QVec
     const QModelIndex idx = index(residx, 0);
     Q_ASSERT(idx.isValid());
     const auto roles = propertiesToRoles(properties);
-    if (!m_sortByRelevancy && roles.contains(m_sortRole)) {
+    if (roles.contains(m_sortRole)) {
         beginRemoveRows({}, residx, residx);
         m_displayedResources.removeAt(residx);
         endRemoveRows();
 
-        sortedInsertion({resource});
+        sortedInsertion({{resource, 0}});
     } else
         Q_EMIT dataChanged(idx, idx, roles);
 }
 
 void ResourcesProxyModel::removeResource(AbstractResource *resource)
 {
-    const auto residx = m_displayedResources.indexOf(resource);
+    const auto residx = indexOf(resource);
     if (residx < 0)
         return;
     beginRemoveRows({}, residx, residx);
@@ -611,18 +709,18 @@ void ResourcesProxyModel::refreshBackend(AbstractResourcesBackend *backend, cons
     bool found = false;
 
     for (int i = 0; i < count; ++i) {
-        if (backend != m_displayedResources[i]->backend())
+        if (backend != m_displayedResources[i].resource->backend())
             continue;
 
         int j = i + 1;
-        for (; j < count && backend == m_displayedResources[j]->backend(); ++j) { }
+        for (; j < count && backend == m_displayedResources[j].resource->backend(); ++j) { }
 
         Q_EMIT dataChanged(index(i, 0), index(j - 1, 0), roles);
         i = j;
         found = true;
     }
 
-    if (found && properties.contains(m_roles.value(m_sortRole))) {
+    if (found && properties.contains(s_roles.value(m_sortRole))) {
         invalidateSorting();
     }
 }
@@ -638,12 +736,14 @@ QVector<int> ResourcesProxyModel::propertiesToRoles(const QVector<QByteArray> &p
 
 int ResourcesProxyModel::indexOf(AbstractResource *res)
 {
-    return m_displayedResources.indexOf(res);
+    return kIndexOf(m_displayedResources, [res](auto result) {
+        return result.resource == res;
+    });
 }
 
 AbstractResource *ResourcesProxyModel::resourceAt(int row) const
 {
-    return m_displayedResources[row];
+    return m_displayedResources[row].resource;
 }
 
 bool ResourcesProxyModel::canFetchMore(const QModelIndex &parent) const
@@ -660,11 +760,6 @@ void ResourcesProxyModel::fetchMore(const QModelIndex &parent)
     Q_EMIT m_currentStream->fetchMore();
 }
 
-bool ResourcesProxyModel::sortByRelevancy() const
-{
-    return m_sortByRelevancy;
-}
-
 QString ResourcesProxyModel::roughCount() const
 {
     const int rows = rowCount();
