@@ -5,8 +5,7 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef RESOURCESPROXYMODEL_H
-#define RESOURCESPROXYMODEL_H
+#pragma once
 
 #include <QQmlParserStatus>
 #include <QSortFilterProxyModel>
@@ -28,6 +27,7 @@ class DISCOVERCOMMON_EXPORT ResourcesProxyModel : public QAbstractListModel, pub
     Q_PROPERTY(Roles sortRole READ sortRole WRITE setSortRole NOTIFY sortRoleChanged)
     Q_PROPERTY(Qt::SortOrder sortOrder READ sortOrder WRITE setSortOrder NOTIFY sortOrderChanged)
     Q_PROPERTY(Category *filteredCategory READ filteredCategory WRITE setFiltersFromCategory NOTIFY categoryChanged)
+    Q_PROPERTY(QString filteredCategoryName READ filteredCategoryName WRITE setFilteredCategoryName NOTIFY categoryChanged)
     Q_PROPERTY(QString originFilter READ originFilter WRITE setOriginFilter)
     Q_PROPERTY(AbstractResource::State stateFilter READ stateFilter WRITE setStateFilter NOTIFY stateFilterChanged)
     Q_PROPERTY(bool filterMinimumState READ filterMinimumState WRITE setFilterMinimumState NOTIFY filterMinimumStateChanged)
@@ -40,7 +40,6 @@ class DISCOVERCOMMON_EXPORT ResourcesProxyModel : public QAbstractListModel, pub
     Q_PROPERTY(QVariantList subcategories READ subcategories NOTIFY subcategoriesChanged)
     Q_PROPERTY(bool isBusy READ isBusy NOTIFY busyChanged)
     Q_PROPERTY(int count READ rowCount NOTIFY countChanged)
-    Q_PROPERTY(bool sortByRelevancy READ sortByRelevancy NOTIFY sortByRelevancyChanged)
     Q_PROPERTY(QString roughCount READ roughCount NOTIFY roughCountChanged)
 public:
     explicit ResourcesProxyModel(QObject *parent = nullptr);
@@ -53,6 +52,7 @@ public:
         RatingPointsRole,
         RatingCountRole,
         SortableRatingRole,
+        SearchRelevanceRole,
         InstalledRole,
         ApplicationRole,
         OriginRole,
@@ -60,7 +60,6 @@ public:
         CanUpgrade,
         PackageNameRole,
         CategoryRole,
-        CategoryDisplayRole,
         SectionRole,
         MimeTypes,
         SizeRole,
@@ -93,6 +92,8 @@ public:
     bool filterMinimumState() const;
 
     Category *filteredCategory() const;
+    QString filteredCategoryName() const;
+    void setFilteredCategoryName(const QString &cat);
 
     QString mimeTypeFilter() const;
     void setMimeTypeFilter(const QString &mime);
@@ -122,13 +123,13 @@ public:
         return m_currentStream != nullptr;
     }
 
-    bool lessThan(AbstractResource *rl, AbstractResource *rr) const;
+    bool lessThan(const StreamResult &left, const StreamResult &right) const;
+    bool orderedLessThan(const StreamResult &left, const StreamResult &right) const;
     Q_SCRIPTABLE void invalidateFilter();
     void invalidateSorting();
 
     bool canFetchMore(const QModelIndex &parent) const override;
     void fetchMore(const QModelIndex &parent) override;
-    bool sortByRelevancy() const;
 
     void classBegin() override
     {
@@ -143,26 +144,26 @@ private Q_SLOTS:
     void removeResource(AbstractResource *resource);
 
 private:
-    void sortedInsertion(const QVector<AbstractResource *> &res);
-    QVariant roleToValue(AbstractResource *res, int role) const;
+    void sortedInsertion(const QVector<StreamResult> &res);
+    QVariant roleToValue(const StreamResult &result, int role) const;
 
     QVector<int> propertiesToRoles(const QVector<QByteArray> &properties) const;
-    void addResources(const QVector<AbstractResource *> &res);
+    void addResources(const QVector<StreamResult> &res);
     void fetchSubcategories();
-    void removeDuplicates(QVector<AbstractResource *> &newResources);
-    bool isSorted(const QVector<AbstractResource *> &resources);
+    void removeDuplicates(QVector<StreamResult> &newResources);
+    bool isSorted(const QVector<StreamResult> &resources);
 
     Roles m_sortRole;
     Qt::SortOrder m_sortOrder;
 
-    bool m_sortByRelevancy;
     bool m_setup = false;
+    QString m_categoryName;
 
     AbstractResourcesBackend::Filters m_filters;
     QVariantList m_subcategories;
 
-    QVector<AbstractResource *> m_displayedResources;
-    const QHash<int, QByteArray> m_roles;
+    QVector<StreamResult> m_displayedResources;
+    static const QHash<int, QByteArray> s_roles;
     ResultsStream *m_currentStream;
 
 Q_SIGNALS:
@@ -177,7 +178,4 @@ Q_SIGNALS:
     void resourcesUrlChanged(const QUrl &url);
     void countChanged();
     void filterMinimumStateChanged(bool filterMinimumState);
-    void sortByRelevancyChanged(bool sortByRelevancy);
 };
-
-#endif
