@@ -26,16 +26,13 @@ int main(int argc, char **argv)
 {
     QApplication app(argc, argv);
     app.setOrganizationDomain(QStringLiteral("kde.org"));
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    app.setAttribute(Qt::AA_UseHighDpiPixmaps, true);
-#endif
     KCrash::setFlags(KCrash::AutoRestart);
 
     NotifierItem notifier;
     bool hide = false;
     KDBusService::StartupOptions startup = {};
     {
-        KAboutData about(QStringLiteral("DiscoverNotifier"),
+        KAboutData about(QStringLiteral("discover.notifier"),
                          i18n("Discover Notifier"),
                          version,
                          i18n("System update status notifier"),
